"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoolExecutor = void 0;
const executor_1 = require("./executor");
// export interface ExecutorWithTrans extends Executor {
//   trans<T>(
//     handler: (abort: (message?: string) => void) => Promise<T>,
//     isolationLevel?: ISOLATION_LEVEL
//   ): Promise<T>;
// }
class PoolExecutor extends executor_1.Executor {
    constructor(pool) {
        super();
        this.pool = pool;
    }
    async trans(handler, isolationLevel) {
        const connection = await this.pool.acquire();
        try {
            const res = await connection.trans(abort => {
                return handler(connection, abort);
            }, isolationLevel);
            return res;
        }
        finally {
            this.pool.release(connection);
        }
    }
    async doQuery(sql, params) {
        const connection = await this.pool.acquire();
        try {
            const res = await connection['doQuery'](sql, params);
            return res;
        }
        finally {
            this.pool.release(connection);
        }
    }
    get sqlifier() {
        return this.pool.provider.sqlifier;
    }
}
exports.PoolExecutor = PoolExecutor;
