"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseConnectOption = exports.parseConnectionUrl = exports.getPoolOptions = exports.getDefaultPoolOptions = exports.getConnectionOptions = exports.getDefaultConnectionOptions = exports.loadConfig = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const url_1 = require("url");
const OPTIONS_FILE = '.lubejs';
async function loadConfig() {
    let configFile = (0, path_1.join)(process.cwd(), OPTIONS_FILE);
    if ((0, fs_1.existsSync)(configFile + '.js')) {
        configFile = configFile + '.js';
    }
    else if ((0, fs_1.existsSync)(configFile + '.ts')) {
        configFile = configFile + '.ts';
    }
    else {
        throw new Error(`Configure file '.lubejs(.ts|.js)' not found in dir ${process.cwd()}, use 'lube init' to create it.`);
    }
    let config;
    try {
        const imported = await Promise.resolve().then(() => __importStar(require(configFile)));
        config = imported?.default ?? imported;
    }
    catch (error) {
        throw new Error(`Occur error at load configure ${configFile}, error info: ${error}`);
    }
    return config;
}
exports.loadConfig = loadConfig;
async function getDefaultConnectionOptions() {
    const config = await loadConfig();
    if (!config.defaultConnection) {
        throw new Error(`Invalid configure, 'defaultConnection' setion not found.`);
    }
    const opts = config.connections[config.defaultConnection];
    if (!opts) {
        throw new Error(`Invalid configure, default connection option ${config.defaultPool} not found.`);
    }
    return opts;
}
exports.getDefaultConnectionOptions = getDefaultConnectionOptions;
async function getConnectionOptions(key) {
    const config = await loadConfig();
    const opts = config.connections[key];
    if (!opts) {
        throw new Error(`Connection option ${key} not found in configure.`);
    }
    return opts;
}
exports.getConnectionOptions = getConnectionOptions;
async function getDefaultPoolOptions() {
    const config = await loadConfig();
    const opts = config.pools[config.defaultPool];
    if (!opts) {
        throw new Error(`Invalid configure, default pool option ${config.defaultPool} not found.`);
    }
    const connOpts = config.connections[opts.connection];
    if (!connOpts) {
        throw new Error(`Invalid configure, pool options ${config.defaultPool} connection ${opts.connection} not found.`);
    }
    return Object.assign({}, opts, {
        connection: config.connections[opts.connection],
    });
}
exports.getDefaultPoolOptions = getDefaultPoolOptions;
async function getPoolOptions(key) {
    const config = await loadConfig();
    const opts = config.pools[key];
    if (!opts) {
        throw new Error(`Pool option ${key} not found in configure.`);
    }
    const connOpts = config.connections[opts.connection];
    if (!connOpts) {
        throw new Error(`Invalid configure, pool options ${config.defaultPool} connection ${opts.connection} not found.`);
    }
    return Object.assign({}, opts, {
        connection: config.connections[opts.connection],
    });
}
exports.getPoolOptions = getPoolOptions;
function parseConnectionUrl(url) {
    const uri = new url_1.URL(url);
    const params = uri.searchParams;
    const urlOptions = {};
    for (const [key, value] of params.entries()) {
        if (value !== undefined) {
            urlOptions[key] = value;
        }
    }
    const dialect = uri.protocol.substr(0, uri.protocol.length - 1).toLowerCase();
    const paths = uri.pathname.split('/');
    let database;
    if (!paths[0]) {
        database = paths[1];
    }
    else {
        database = paths[0];
    }
    if (!database) {
        throw new Error('Database must specify.');
    }
    const options = {
        dialect,
        host: uri.host,
        port: uri.port ? parseInt(uri.port) : undefined,
        user: uri.username,
        password: uri.password,
        database,
        ...urlOptions,
    };
    return options;
}
exports.parseConnectionUrl = parseConnectionUrl;
// const DEFAULT_CONNECT_OPTIONS: Partial<ConnectOptions> = {
//   connectionTimeout: 30,
//   requestTimeout: 10 * 60,
//   // minConnections: 0,
//   // maxConnections: 5,
// };
/**
 * 准备连接选项
 */
async function parseConnectOption(optOrUrlOrCfg) {
    let options;
    if (!optOrUrlOrCfg) {
        options = await getDefaultConnectionOptions();
        if (!options) {
            throw new Error(`Default connection options not found on config file.`);
        }
    }
    else if (typeof optOrUrlOrCfg === 'string') {
        try {
            new url_1.URL(optOrUrlOrCfg);
            options = parseConnectionUrl(optOrUrlOrCfg);
        }
        catch {
            const config = await loadConfig();
            options = config.connections[optOrUrlOrCfg];
            if (!options) {
                throw new Error(`Connection options ${optOrUrlOrCfg} not found on config file.`);
            }
        }
    }
    else {
        options = optOrUrlOrCfg;
    }
    return options;
}
exports.parseConnectOption = parseConnectOption;
