"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPool = exports.createConnection = void 0;
const register_1 = require("./register");
const pool_1 = require("./base/pool");
const config_1 = require("./config");
async function createConnection(optOrUrlOrCfg) {
    const options = await (0, config_1.parseConnectOption)(optOrUrlOrCfg);
    (0, register_1.prepareConnectOption)(options);
    return options.provider.getConnection(options);
}
exports.createConnection = createConnection;
async function createPool(optOrCfg) {
    let options;
    if (!optOrCfg) {
        options = await (0, config_1.getDefaultPoolOptions)();
    }
    else if (typeof optOrCfg === 'string') {
        options = await (0, config_1.getPoolOptions)(optOrCfg);
        if (!options) {
            throw new Error(`Pool configure not found.`);
        }
    }
    else {
        options = optOrCfg;
    }
    (0, register_1.prepareConnectOption)(options.connection);
    return new pool_1.Pool(options.connection.provider, options, options.connection);
}
exports.createPool = createPool;
