"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareConnectOption = exports.getProvider = exports.register = void 0;
/**
 * 注册一个方言支持
 * @param driver 驱动可以是connect函数，亦可以是npm包或路径
 */
async function register(name, driver) {
    if (dialects[name]) {
        throw new Error(`Driver ${name} is exists.`);
    }
    dialects[name] = driver();
}
exports.register = register;
function getProvider(dialect) {
    const provider = dialects[dialect];
    if (!provider)
        throw new Error(`Dialect ${dialect} is not register, please import provider module before use it.`);
    return provider;
}
exports.getProvider = getProvider;
function prepareConnectOption(options) {
    if (!options.provider) {
        if (!options.dialect) {
            throw new Error('ConnectOptions must provide one of dialect or dirver');
        }
        const provider = getProvider(options.dialect);
        if (!provider) {
            throw new Error(`Unregister dialect ${options.dialect}.`);
        }
        options.provider = provider;
    }
    if (!options.dialect) {
        options.dialect = options.provider.dialect;
    }
}
exports.prepareConnectOption = prepareConnectOption;
const dialects = {};
