"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SQL = void 0;
const sql_1 = require("./sql");
const statement_1 = require("./sql/statement");
exports.SQL = {
    get type() {
        return sql_1.DbType;
    },
    // use(database: string): Use {
    //   return new Use(database);
    // },
    if(condition) {
        return new sql_1.If(condition);
    },
    while(condition) {
        return new sql_1.While(condition);
    },
    break() {
        return new sql_1.Break();
    },
    continue() {
        return new sql_1.Continue();
    },
    return(value) {
        return new sql_1.Return(value);
    },
    /**
     * 创建一个SQL文档
     * @param statements 文档代码
     */
    doc(...statements) {
        const lines = Array.isArray(statements[0])
            ? statements[0]
            : statements;
        return new sql_1.Document(lines);
    },
    group(value) {
        if (sql_1.Condition.isCondition(value)) {
            return new sql_1.GroupCondition(value);
        }
        return new sql_1.GroupExpression(value);
    },
    /**
     * 负号运算符 -
     */
    neg(expr) {
        return new sql_1.UnaryOperation(sql_1.UNARY_OPERATION_OPERATOR.NEG, expr);
    },
    /**
     * 字符串连接运算
     */
    concat(...strs) {
        let exp = strs[0];
        for (let i = 1; i < strs.length; i++) {
            exp = new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.CONCAT, exp, strs[i]);
        }
        return exp;
    },
    /**
     * 算术运算 +
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    add(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.ADD, left, right);
    },
    /**
     * 算术运算 -
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    sub(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.SUB, left, right);
    },
    /**
     * 算术运算 *
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    mul(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.MUL, left, right);
    },
    /**
     * 算术运算 /
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    div(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.DIV, left, right);
    },
    /**
     * 算术运算 %
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    mod(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.MOD, left, right);
    },
    /*
     * 位算术运算 ^
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    xor(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.XOR, left, right);
    },
    /**
     * 位算术运算 << 仅支持整型
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    shl(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.SHL, left, right);
    },
    /**
     * 位算术运算 >>
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    shr(left, right) {
        return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.SHR, left, right);
    },
    literal(value, dbType) {
        return new sql_1.Literal(value, dbType);
    },
    var(name, type) {
        return new sql_1.Variant(name, type);
    },
    /**
     * 创建一个字段
     */
    field(name, rowset) {
        return new sql_1.Field(name, rowset);
    },
    and(...args) {
        if (Array.isArray(args[0])) {
            args = args[0];
        }
        if (sql_1.Expression.isExpression(args[0]) || (0, sql_1.isBaseScalar)(args[0])) {
            return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.AND, args[0], args[1]);
        }
        return sql_1.ConditionBuilder.and(args);
    },
    rownum(orderBy, partitionBy) {
        return new sql_1.VirtualExpression(exports.SQL.rownum.name, [orderBy, partitionBy]);
    },
    or(...args) {
        if (Array.isArray(args[0])) {
            args = args[0];
        }
        if (sql_1.Expression.isExpression(args[0]) || (0, sql_1.isBaseScalar)(args[0])) {
            return new sql_1.BinaryOperation(sql_1.BINARY_OPERATION_OPERATOR.OR, args[0], args[1]);
        }
        return sql_1.ConditionBuilder.or(args);
    },
    not(arg) {
        if (sql_1.Expression.isExpression(arg) || (0, sql_1.isBaseScalar)(arg)) {
            return new sql_1.UnaryOperation(sql_1.UNARY_OPERATION_OPERATOR.NOT, arg);
        }
        return new sql_1.UnaryLogicCondition(sql_1.LOGIC_OPERATOR.NOT, arg);
    },
    /**
     * 比较运算 IS NULL
     * @returns 返回比较运算符
     * @param expr 表达式
     */
    isNull(expr) {
        return new sql_1.UnaryCompareCondition(sql_1.UNARY_COMPARE_OPERATOR.IS_NULL, expr);
    },
    /**
     * 比较运算 IS NOT NULL
     * @param expr 表达式
     * @returns 返回比较运算符
     */
    isNotNull(expr) {
        return new sql_1.UnaryCompareCondition(sql_1.UNARY_COMPARE_OPERATOR.IS_NOT_NULL, expr);
    },
    eq(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.EQ, left, right);
    },
    /**
     * 比较运算 <>
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    neq(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.NEQ, left, right);
    },
    lt(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.LT, left, right);
    },
    /**
     * 比较运算 <=
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    lte(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.LTE, left, right);
    },
    /**
     * 比较运算 >
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    gt(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.GT, left, right);
    },
    /**
     * 比较运算 >=
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    gte(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.GTE, left, right);
    },
    /**
     * 比较运算 LIKE
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    like(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.LIKE, left, right);
    },
    /**
     * 比较运算 NOT LIKE
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    notLike(left, right) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.NOT_LIKE, left, right);
    },
    in(left, values) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.IN, left, sql_1.Select.isSelect(values)
            ? values.asValue()
            : values.map(v => sql_1.Expression.ensureExpression(v)));
    },
    /**
     * 比较运算 NOT IN
     * @param left 左值
     * @param values 要比较的值列表
     * @returns 返回比较运算对比条件
     */
    notIn(left, values) {
        return new sql_1.BinaryCompareCondition(sql_1.BINARY_COMPARE_OPERATOR.NOT_IN, left, values.map(v => sql_1.Expression.ensureExpression(v)));
    },
    /**
     * 判断是否存在
     * @param select 查询语句
     */
    exists(select) {
        return new sql_1.ExistsCondition(select);
    },
    table(name, builtInOrMembers = false) {
        if (typeof builtInOrMembers === 'function' ||
            Array.isArray(builtInOrMembers)) {
            return (0, sql_1.createTableVariant)(name, builtInOrMembers);
        }
        return sql_1.Table.create(name, builtInOrMembers);
    },
    /**
     * 声明一个函数
     */
    func(name, builtIn = false) {
        return new sql_1.Func(name, builtIn);
    },
    /**
     * 创建一个可供调用的存储过程函数
     */
    proc(name, builtIn = false) {
        return new sql_1.Procedure(name, builtIn);
    },
    builtIn(name) {
        return new sql_1.BuiltIn(name);
    },
    star() {
        return new sql_1.Star();
    },
    invokeAsTable(func, args) {
        return sql_1.Func.ensure(func).invokeAsTable(...args);
    },
    invokeAsScalar(func, args) {
        return sql_1.Func.ensure(func).invokeAsScalar(...args);
    },
    makeInvoke(type, name, builtIn = false) {
        if (type === 'table') {
            return function (...args) {
                return exports.SQL.invokeAsTable(exports.SQL.func(name, builtIn), args.map(p => (p === undefined ? null : p)));
            };
        }
        if (type === 'scalar') {
            return function (...args) {
                return exports.SQL.invokeAsScalar(exports.SQL.func(name, builtIn), args.map(p => (p === undefined ? null : p)));
            };
        }
        throw new Error('invalid arg value of `type`');
    },
    makeExec(name, builtIn = false) {
        return function (...args) {
            return exports.SQL.execute(exports.SQL.proc(name, builtIn), args.map(p => (p === undefined ? null : p)));
        };
    },
    //********************** statement **************************//
    /**
     * 赋值语句
     * @param left 左值
     * @param right 右值
     */
    set(left, right) {
        return new sql_1.Assignment(left, right);
    },
    /**
     * 变量声明
     * @param declares 变量列表
     */
    declare(...members) {
        if (members.length == 1 && Array.isArray(members[0])) {
            members = members[0];
        }
        return new sql_1.Declare(members);
    },
    /**
     * WHEN 语句块
     * @param expr
     * @param value
     */
    when(expr, value) {
        return new sql_1.When(expr, value);
    },
    case(expr) {
        return new sql_1.Case(expr);
    },
    /**
     * With语句
     */
    with(...rowsets) {
        return new sql_1.With(...rowsets);
    },
    union(...selects) {
        selects.forEach((sel, index) => {
            if (index < selects.length - 1)
                sel.union(selects[index + 1]);
        });
        return selects[0];
    },
    unionAll(...selects) {
        selects.forEach((sel, index) => {
            if (index < selects.length - 1)
                sel.unionAll(selects[index + 1]);
        });
        return selects[0];
    },
    get createTable() {
        return sql_1.CreateTableBuilder;
    },
    get alterTable() {
        return sql_1.AlterTableBuilder;
    },
    get createView() {
        return sql_1.CreateViewBuilder;
    },
    alterView(name) {
        return new sql_1.AlterView(name);
    },
    createIndex(name) {
        return new sql_1.CreateIndex(name);
    },
    get createProcedure() {
        return sql_1.CreateProcedureBuilder;
    },
    get alterProcedure() {
        return sql_1.AlterProcedureBuilder;
    },
    get createFunction() {
        return sql_1.CreateFunctionBuilder;
    },
    get alterFunction() {
        return sql_1.AlterFunctionBuilder;
    },
    get dropProcedure() {
        return statement_1.DropProcedureBuilder;
    },
    get dropTable() {
        return sql_1.DropTableBuilder;
    },
    get dropView() {
        return statement_1.DropViewBuilder;
    },
    get dropFunction() {
        return statement_1.DropFunctionBuilder;
    },
    dropIndex(table, name) {
        return new sql_1.DropIndex(table, name);
    },
    annotation(...text) {
        return new sql_1.Annotation('BLOCK', text.join('\n'));
    },
    note(text) {
        return new sql_1.Annotation('LINE', text);
    },
    /**
     * 插入至表,into的别名
     * @param table
     * @param fields
     */
    insert(table, fields) {
        return new sql_1.Insert(table, fields);
    },
    /**
     * 更新一个表格
     * @param table
     */
    update(table) {
        return new sql_1.Update(table);
    },
    /**
     * 删除一个表格
     * @param table 表格
     */
    delete(table) {
        return new sql_1.Delete(table);
    },
    select(...args) {
        // if (args.length === 0) {
        //   args = [SQL.star];
        // }
        return new sql_1.Select(...args);
    },
    raw(sql) {
        return new sql_1.Raw(sql);
    },
    block(...args) {
        if (args.length === 1 && Array.isArray(args[0])) {
            args = args[0];
        }
        return new sql_1.Block(args);
    },
    execute(proc, params
    // | Parameter<JsConstant, string>[] | InputObject
    ) {
        return new sql_1.Execute(proc, params);
    },
    get createDatabase() {
        return sql_1.CreateDatabaseBuilder;
    },
    alterDatabase(name) {
        return new sql_1.AlterDatabase(name);
    },
    get dropDatabase() {
        return statement_1.DropDatabaseBuilder;
    },
    get createSchema() {
        return statement_1.CreateSchemaBuilder;
    },
    get dropSchema() {
        return statement_1.DropSchemaBuilder;
    },
    get createSequence() {
        return sql_1.CreateSequenceBuilder;
    },
    get dropSequence() {
        return statement_1.DropSequenceBuilder;
    },
    input(name, typeOrValue, value) {
        let type;
        if ((0, sql_1.isDbType)(typeOrValue)) {
            type = typeOrValue;
        }
        else {
            value = typeOrValue;
        }
        return new sql_1.Parameter(name, type, value, 'IN');
    },
    /**
     * output参数
     */
    output(name, type, value) {
        return new sql_1.Parameter(name, type, value, 'OUT');
    },
    /**
     * 创建一个输入输出参数
     */
    inoutput(name, type, value) {
        return new sql_1.Parameter(name, type, value, 'INOUT');
    },
    //***************************End 参数声明 ***************************//
    sequence(name) {
        return new sql_1.Sequence(name);
    },
    count(expr) {
        return new sql_1.VirtualExpression(exports.SQL.count.name, [expr]);
    },
    avg(expr) {
        return new sql_1.VirtualExpression(exports.SQL.avg.name, [expr]);
    },
    sum(expr) {
        return new sql_1.VirtualExpression(exports.SQL.sum.name, [expr]);
    },
    max(expr) {
        return new sql_1.VirtualExpression(exports.SQL.max.name, [expr]);
    },
    min(expr) {
        return new sql_1.VirtualExpression(exports.SQL.min.name, [expr]);
    },
    /**
     * 获取标识列的最近插入值
     */
    identityValue(table, column) {
        return new sql_1.VirtualExpression(exports.SQL.identityValue.name, [table, column]);
    },
    /**
     * 转换数据类型
     */
    convert(expr, toType) {
        return new sql_1.VirtualExpression(exports.SQL.convert.name, [expr, toType]);
    },
    /**
     * 获取当前日期及时间
     */
    now() {
        return new sql_1.VirtualExpression(exports.SQL.now.name, []);
    },
    /**
     * 获取当前UTC时间
     */
    utcNow() {
        return new sql_1.VirtualExpression(exports.SQL.utcNow.name, []);
    },
    /**
     * 切换时区
     */
    switchTimezone(date, offset) {
        return new sql_1.VirtualExpression(exports.SQL.switchTimezone.name, [date, offset]);
    },
    /**
     * 格式化日期函数
     */
    formatDate(date, format) {
        return new sql_1.VirtualExpression(exports.SQL.formatDate.name, [date, format]);
    },
    /**
     * 获取日期中的年份
     * @param date
     * @returns
     */
    yearOf(date) {
        return new sql_1.VirtualExpression(exports.SQL.yearOf.name, [date]);
    },
    /**
     * 获取日期中的月份
     * @param date
     * @returns
     */
    monthOf(date) {
        return new sql_1.VirtualExpression(exports.SQL.monthOf.name, [date]);
    },
    /**
     * 获取日期中的日
     * @param date
     * @returns
     */
    dayOf(date) {
        return new sql_1.VirtualExpression(exports.SQL.dayOf.name, [date]);
    },
    /**
     * 计算两个日期之间的天数，小数
     * @param start
     * @param end
     * @returns
     */
    daysBetween(start, end) {
        return new sql_1.VirtualExpression(exports.SQL.daysBetween.name, [start, end]);
    },
    /**
     * 计算两个日期之间的月数，小数
     * @param start
     * @param end
     * @returns
     */
    monthsBetween(start, end) {
        return new sql_1.VirtualExpression(exports.SQL.monthsBetween.name, [start, end]);
    },
    /**
     * 计算两个日期之间的天数，小数
     * @param start
     * @param end
     * @returns
     */
    yearsBetween(start, end) {
        return new sql_1.VirtualExpression(exports.SQL.yearsBetween.name, [start, end]);
    },
    /**
     * 计算两个日期之间的天数，小数
     * @param start
     * @param end
     * @returns
     */
    hoursBetween(start, end) {
        return new sql_1.VirtualExpression(exports.SQL.hoursBetween.name, [start, end]);
    },
    /**
     * 计算两个日期之间的天数，小数
     * @param start
     * @param end
     * @returns
     */
    minutesBetween(start, end) {
        return new sql_1.VirtualExpression(exports.SQL.minutesBetween.name, [start, end]);
    },
    /**
     * 计算两个日期之间的天数，小数
     * @param start
     * @param end
     * @returns
     */
    secondsBetween(start, end) {
        return new sql_1.VirtualExpression(exports.SQL.secondsBetween.name, [start, end]);
    },
    addDays(date, days) {
        return new sql_1.VirtualExpression(exports.SQL.addDays.name, [date, days]);
    },
    addMonths(date, months) {
        return new sql_1.VirtualExpression(exports.SQL.addMonths.name, [date, months]);
    },
    addYears(date, years) {
        return new sql_1.VirtualExpression(exports.SQL.addYears.name, [date, years]);
    },
    addHours(date, hours) {
        return new sql_1.VirtualExpression(exports.SQL.addHours.name, [date, hours]);
    },
    addMinutes(date, minutes) {
        return new sql_1.VirtualExpression(exports.SQL.addMinutes.name, [date, minutes]);
    },
    addSeconds(date, seconds) {
        return new sql_1.VirtualExpression(exports.SQL.addSeconds.name, [date, seconds]);
    },
    /**
     * 获取字符串长度
     * @param str
     * @returns
     */
    strlen(str) {
        return new sql_1.VirtualExpression(exports.SQL.strlen.name, [str]);
    },
    /**
     * 截取字符串
     * @param str
     * @param start
     * @param length
     * @returns
     */
    substr(str, start, length) {
        return new sql_1.VirtualExpression(exports.SQL.substr.name, [start, length]);
    },
    /**
     * 替换字符串
     * @param str 需要被替换的字符串
     * @param search 查找字符串
     * @param to 替换成字符串
     * @param global 是否全局替换，默认为false
     * @returns
     */
    replace(str, search, to) {
        return new sql_1.VirtualExpression(exports.SQL.replace.name, [str, search, to]);
    },
    /**
     * 删除字符串两侧空格
     * @param str
     * @returns
     */
    trim(str) {
        return new sql_1.VirtualExpression(exports.SQL.trim.name, [str]);
    },
    /**
     * 删除字符串右侧空格
     * @param str
     * @returns
     */
    trimEnd(str) {
        return new sql_1.VirtualExpression(exports.SQL.trimEnd.name, [str]);
    },
    /**
     * 转换成大写字母
     * @param str
     * @returns
     */
    upper(str) {
        return new sql_1.VirtualExpression(exports.SQL.upper.name, [str]);
    },
    /**
     * 转换成小写字母
     * @param str
     * @returns
     */
    lower(str) {
        return new sql_1.VirtualExpression(exports.SQL.lower.name, [str]);
    },
    /**
     * 查找一个
     * @param str
     * @param search
     * @returns
     */
    strpos(str, search, startAt) {
        return new sql_1.VirtualExpression(exports.SQL.strpos.name, [str, search, startAt]);
    },
    // /**
    //  * 获取一个字符的ascii码
    //  * @param str 字符编码
    //  * @returns
    //  */
    // ascii(str: XExpression<string>): Expression<number> {
    //   return new VirtualExpression(SQL.ascii.name, [str]);
    // },
    // asciiChar(code: XExpression<number>): Expression<string> {
    //   return new VirtualExpression(SQL.asciiChar.name, [code]);
    // },
    // unicode(str: XExpression<string>): Expression<number> {
    //   return new VirtualExpression(SQL.unicode.name, [str]);
    // },
    // unicodeChar(code: XExpression<number>): Expression<string> {
    //   return new VirtualExpression(SQL.unicodeChar.name, [code]);
    // },
    nvl(value, defaultValue) {
        return new sql_1.VirtualExpression(exports.SQL.nvl.name, [value, defaultValue]);
    },
    abs(value) {
        return new sql_1.VirtualExpression(exports.SQL.abs.name, [value]);
    },
    exp(value) {
        return new sql_1.VirtualExpression(exports.SQL.exp.name, [value]);
    },
    ceil(value) {
        return new sql_1.VirtualExpression(exports.SQL.ceil.name, [value]);
    },
    floor(value) {
        return new sql_1.VirtualExpression(exports.SQL.floor.name, [value]);
    },
    ln(value) {
        return new sql_1.VirtualExpression(exports.SQL.ln.name, [value]);
    },
    log(value) {
        return new sql_1.VirtualExpression(exports.SQL.log.name, [value]);
    },
    pi() {
        return new sql_1.VirtualExpression(exports.SQL.pi.name, []);
    },
    power(a, b) {
        return new sql_1.VirtualExpression(exports.SQL.power.name, [a, b]);
    },
    radians(value) {
        return new sql_1.VirtualExpression(exports.SQL.radians.name, [value]);
    },
    degrees(value) {
        return new sql_1.VirtualExpression(exports.SQL.degrees.name, [value]);
    },
    random() {
        return new sql_1.VirtualExpression(exports.SQL.random.name, []);
    },
    round(value, s) {
        return new sql_1.VirtualExpression(exports.SQL.round.name, [value, s]);
    },
    sign(value) {
        return new sql_1.VirtualExpression(exports.SQL.sign.name, [value]);
    },
    sqrt(value) {
        return new sql_1.VirtualExpression(exports.SQL.sqrt.name, [value]);
    },
    cos(value) {
        return new sql_1.VirtualExpression(exports.SQL.cos.name, [value]);
    },
    sin(value) {
        return new sql_1.VirtualExpression(exports.SQL.sin.name, [value]);
    },
    tan(value) {
        return new sql_1.VirtualExpression(exports.SQL.tan.name, [value]);
    },
    acos(value) {
        return new sql_1.VirtualExpression(exports.SQL.acos.name, [value]);
    },
    asin(value) {
        return new sql_1.VirtualExpression(exports.SQL.asin.name, [value]);
    },
    atan(value) {
        return new sql_1.VirtualExpression(exports.SQL.atan.name, [value]);
    },
    atan2(value) {
        return new sql_1.VirtualExpression(exports.SQL.atan2.name, [value]);
    },
    cot(value) {
        return new sql_1.VirtualExpression(exports.SQL.cot.name, [value]);
    },
    currentDatabase() {
        return new sql_1.VirtualExpression(exports.SQL.currentDatabase.name, []);
    },
    defaultSchema() {
        return new sql_1.VirtualExpression(exports.SQL.defaultSchema.name, []);
    },
    nextValue(sequenceName) {
        return new sql_1.VirtualExpression(exports.SQL.nextValue.name, [sequenceName]);
    },
    existsTable(table) {
        return new sql_1.VirtualCondition(exports.SQL.existsTable.name, [table]);
    },
    existsDatabase(database) {
        return new sql_1.VirtualCondition(exports.SQL.existsDatabase.name, [database]);
    },
    existsView(name) {
        return new sql_1.VirtualCondition(exports.SQL.existsView.name, [name]);
    },
    existsFunction(name) {
        return new sql_1.VirtualCondition(exports.SQL.existsFunction.name, [name]);
    },
    existsProcedure(name) {
        return new sql_1.VirtualCondition(exports.SQL.existsProcedure.name, [name]);
    },
    existsSequence(name) {
        return new sql_1.VirtualCondition(exports.SQL.existsSequence.name, [name]);
    },
};
