import { Sql, SQL_SYMBOLE } from '../sql';
import { DefaultInputObject, RowObject, WhereObject } from '../types';
import { XRowsets } from '../rowset/rowset';
/**
 * 查询条件
 */
export declare abstract class Condition extends Sql {
    readonly $type: SQL_SYMBOLE.CONDITION;
    abstract readonly $kind: CONDITION_KIND;
    /**
     * and连接
     * @param condition 下一个查询条件
     * @returns 返回新的查询条件
     */
    and(condition: Condition): Condition;
    /**
     * OR语句
     * @param condition
     * @returns 返回新的查询条件
     */
    or(condition: Condition): Condition;
    static isCondition(object: any): object is Condition;
}
/**
 * 逻辑运算符列表
 */
export declare enum LOGIC_OPERATOR {
    AND = "AND",
    OR = "OR",
    NOT = "NOT"
}
/**
 * 比较运算符
 */
export declare type COMPARE_OPERATOR = UNARY_COMPARE_OPERATOR | BINARY_COMPARE_OPERATOR;
/**
 * 比较运算符列表
 */
export declare enum BINARY_COMPARE_OPERATOR {
    IN = "IN",
    NOT_IN = "NOT IN",
    EQ = "=",
    NEQ = "<>",
    GT = ">",
    GTE = ">=",
    LT = "<",
    LTE = "<=",
    LIKE = "LIKE",
    NOT_LIKE = "NOT LIKE"
}
export declare enum UNARY_COMPARE_OPERATOR {
    IS_NULL = "IS NULL",
    IS_NOT_NULL = "IS NOT NULL"
}
export declare enum CONDITION_KIND {
    GROUP = "GROUP",
    UNARY_COMPARE = "UNARY_COMPARE",
    BINARY_COMPARE = "BINARY_COMPARE",
    BINARY_LOGIC = "BINARY_LOGIC",
    UNARY_LOGIC = "UNARY_LOGIC",
    EXISTS = "EXISTS",
    STANDARD = "STANDARD"
}
/**
 * 可兼容的查询条件
 */
export declare type XCondition<T extends RowObject = DefaultInputObject> = Condition | WhereObject<T>;
export declare const ConditionBuilder: {
    group(condition: Condition): GroupCondition;
    and(...conditions: [Condition[]] | [Condition, Condition, ...Condition[]]): Condition;
    or(...conditions: [Condition[]] | [Condition, Condition, ...Condition[]]): Condition;
    parse<T extends object>(condition: WhereObject<T>, rowset?: XRowsets<T> | undefined): Condition;
};
export declare type ConditionBuilder = typeof ConditionBuilder;
import { GroupCondition } from './group-condition';
