"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConditionBuilder = exports.CONDITION_KIND = exports.UNARY_COMPARE_OPERATOR = exports.BINARY_COMPARE_OPERATOR = exports.LOGIC_OPERATOR = exports.Condition = void 0;
const sql_1 = require("../sql");
const rowset_1 = require("../rowset/rowset");
const expression_1 = require("../expression");
/**
 * 查询条件
 */
class Condition extends sql_1.Sql {
    constructor() {
        super(...arguments);
        this.$type = sql_1.SQL_SYMBOLE.CONDITION;
    }
    /**
     * and连接
     * @param condition 下一个查询条件
     * @returns 返回新的查询条件
     */
    and(condition) {
        return exports.ConditionBuilder.and(this, condition);
        // condition = ensureCondition(condition);
        // return new BinaryLogicCondition(LOGIC_OPERATOR.AND, this, SqlBuilder.group(condition));
    }
    /**
     * OR语句
     * @param condition
     * @returns 返回新的查询条件
     */
    or(condition) {
        return exports.ConditionBuilder.or(this, condition);
    }
    static isCondition(object) {
        return object?.$type === sql_1.SQL_SYMBOLE.CONDITION;
    }
}
exports.Condition = Condition;
/**
 * 逻辑运算符列表
 */
var LOGIC_OPERATOR;
(function (LOGIC_OPERATOR) {
    LOGIC_OPERATOR["AND"] = "AND";
    LOGIC_OPERATOR["OR"] = "OR";
    LOGIC_OPERATOR["NOT"] = "NOT";
})(LOGIC_OPERATOR = exports.LOGIC_OPERATOR || (exports.LOGIC_OPERATOR = {}));
/**
 * 比较运算符列表
 */
var BINARY_COMPARE_OPERATOR;
(function (BINARY_COMPARE_OPERATOR) {
    BINARY_COMPARE_OPERATOR["IN"] = "IN";
    BINARY_COMPARE_OPERATOR["NOT_IN"] = "NOT IN";
    BINARY_COMPARE_OPERATOR["EQ"] = "=";
    BINARY_COMPARE_OPERATOR["NEQ"] = "<>";
    BINARY_COMPARE_OPERATOR["GT"] = ">";
    BINARY_COMPARE_OPERATOR["GTE"] = ">=";
    BINARY_COMPARE_OPERATOR["LT"] = "<";
    BINARY_COMPARE_OPERATOR["LTE"] = "<=";
    BINARY_COMPARE_OPERATOR["LIKE"] = "LIKE";
    BINARY_COMPARE_OPERATOR["NOT_LIKE"] = "NOT LIKE";
})(BINARY_COMPARE_OPERATOR = exports.BINARY_COMPARE_OPERATOR || (exports.BINARY_COMPARE_OPERATOR = {}));
var UNARY_COMPARE_OPERATOR;
(function (UNARY_COMPARE_OPERATOR) {
    UNARY_COMPARE_OPERATOR["IS_NULL"] = "IS NULL";
    UNARY_COMPARE_OPERATOR["IS_NOT_NULL"] = "IS NOT NULL";
})(UNARY_COMPARE_OPERATOR = exports.UNARY_COMPARE_OPERATOR || (exports.UNARY_COMPARE_OPERATOR = {}));
var CONDITION_KIND;
(function (CONDITION_KIND) {
    CONDITION_KIND["GROUP"] = "GROUP";
    CONDITION_KIND["UNARY_COMPARE"] = "UNARY_COMPARE";
    CONDITION_KIND["BINARY_COMPARE"] = "BINARY_COMPARE";
    CONDITION_KIND["BINARY_LOGIC"] = "BINARY_LOGIC";
    CONDITION_KIND["UNARY_LOGIC"] = "UNARY_LOGIC";
    CONDITION_KIND["EXISTS"] = "EXISTS";
    CONDITION_KIND["STANDARD"] = "STANDARD";
})(CONDITION_KIND = exports.CONDITION_KIND || (exports.CONDITION_KIND = {}));
exports.ConditionBuilder = {
    group(condition) {
        return new group_condition_1.GroupCondition(condition);
    },
    and(...conditions) {
        const conds = (Array.isArray(conditions[0]) ? conditions[0] : conditions);
        (0, console_1.assert)(conds.length >= 2, `Conditions must more than 2 conditions.`);
        return join(LOGIC_OPERATOR.AND, conds);
    },
    or(...conditions) {
        const conds = (Array.isArray(conditions[0]) ? conditions[0] : conditions);
        (0, console_1.assert)(conds.length >= 2, `Conditions must more than 2 conditions.`);
        return join(LOGIC_OPERATOR.OR, conds);
    },
    parse(condition, rowset) {
        let makeField;
        if (rowset) {
            if (typeof rowset === 'string' || Array.isArray(rowset)) {
                makeField = (key) => new expression_1.Field(key, rowset);
            }
            else if (rowset_1.Rowset.isRowset(rowset)) {
                makeField = (key) => rowset.$field(key);
            }
        }
        else {
            makeField = (key) => new expression_1.Field(key);
        }
        const compares = Object.entries(condition).map(([key, value]) => {
            const field = makeField(key);
            if (value === null || value === undefined) {
                return field.isNull();
            }
            if (Array.isArray(value)) {
                return field.in(value);
            }
            return field.eq(value);
        });
        return compares.length >= 2 ? exports.ConditionBuilder.and(compares) : compares[0];
    },
};
function join(logic, conditions) {
    if (conditions.length < 2) {
        throw new Error(`Conditions must more than or equals 2 element.`);
    }
    const cond = conditions.reduce((previous, current) => {
        let condition = current;
        // 如果是二元逻辑条件运算，则将其用括号括起来，避免逻辑运算出现优先级的问题
        if (binary_logic_condition_1.BinaryLogicCondition.isBinaryLogicCondition(condition)) {
            condition = exports.ConditionBuilder.group(condition);
        }
        if (!previous)
            return condition;
        return new binary_logic_condition_1.BinaryLogicCondition(logic, previous, condition);
    });
    return exports.ConditionBuilder.group(cond);
}
const group_condition_1 = require("./group-condition");
const console_1 = require("console");
const binary_logic_condition_1 = require("./binary-logic-condition");
