"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Literal = void 0;
const sql_1 = require("../sql");
const db_type_1 = require("../db-type");
const scalar_1 = require("../scalar");
const expression_1 = require("./expression");
/**
 * 字面量表达式
 */
class Literal extends expression_1.Expression {
    constructor(value, dbType) {
        super();
        this.$type = sql_1.SQL_SYMBOLE.LITERAL;
        this.$value = value;
        // this.$dbType = dbType ? dbType : Literal.parseValueType(value);
        this.$dbType = dbType;
    }
    static isLiteral(object) {
        return object?.$type === sql_1.SQL_SYMBOLE.LITERAL;
    }
    valueOf() {
        return this.$value;
    }
    /**
     * 根据值自动填充类型
     */
    fitType() {
        if (!this.$dbType) {
            this.$dbType = Literal.parseValueType(this.$value);
        }
    }
    static ensureLiteral(value) {
        return Literal.isLiteral(value) ? value : new Literal(value);
    }
    /**
     * 解释值的类型
     */
    static parseValueType(value) {
        if (value === undefined)
            throw new Error('Do not parse DbType from null or undefined');
        // 如果为null值，默认为int32类型
        if (value === null)
            return db_type_1.DbType.int32;
        switch (value.constructor) {
            case String:
                return db_type_1.DbType.string(0);
            case Number:
                if (Number.isInteger(value) &&
                    value >= -2147483648 &&
                    value <= 2147483647) {
                    return db_type_1.DbType.int32;
                }
                return db_type_1.DbType.float64;
            case Date:
                return db_type_1.DbType.datetime;
            case Boolean:
                return db_type_1.DbType.boolean;
            case scalar_1.Uuid:
                return db_type_1.DbType.uuid;
            case scalar_1.Time:
                return db_type_1.DbType.time;
            case scalar_1.Decimal:
                return db_type_1.DbType.decimal(18, 8);
            case BigInt:
                return db_type_1.DbType.int64;
            case Buffer:
            case ArrayBuffer:
            case SharedArrayBuffer:
                return db_type_1.DbType.binary(db_type_1.DbType.MAX);
            case Array:
                return db_type_1.DbType.list((Literal.parseValueType(value[0]) ||
                    null));
            default:
                return db_type_1.DbType.json();
        }
    }
}
exports.Literal = Literal;
