/**
 * 数据库行集，混入类型
 */
import { Sql, SQL_SYMBOLE } from '../sql';
import { Field } from '../expression';
import { Star, XObjectName } from '../object';
import { ColumnsOf, DataRowValueType, RowObject } from '../types';
export declare type PropertyName = string;
export declare type ColumnName = string;
export declare type ColumnMap = Record<PropertyName, ColumnName>;
export declare abstract class Rowset<T extends RowObject = RowObject, R extends RowObject = T> extends Sql {
    protected constructor();
    /**
     * 继承Rowset后必须在调用此函数，以达到创建代理Rowset的目的。
     */
    protected $proxy(): XRowset<T>;
    $name?: XObjectName;
    /**
     * 别名
     */
    $alias?: string;
    $tag: SQL_SYMBOLE.ROWSET;
    /**
     * 属性-->字段 映射表
     */
    $map?: ColumnMap;
    /**
     * 为当前表添加别名
     */
    as(alias: string): XRowset<T>;
    /**
     * 字段
     * @param name 节点名称
     */
    $field<P extends ColumnsOf<T>>(name: P): Field<DataRowValueType<T[P]>>;
    /**
     * 获取所有字段
     */
    get star(): Star<T>;
    /**
     * 该方法为内部方法，用户请不要使用该方法
     */
    $around(columnMap: ColumnMap): this;
    static isRowset(object: any): object is XRowsets<any> | XRowsets;
}
export declare type XRowsets<T extends RowObject = {}> = Rowset<T> | XRowset<T> | Table<T> | XTable<T> | NamedSelect<T> | XNamedSelect<T> | TableVariant<T> | XTableVariant<T> | TableFuncInvoke<T> | XTableFuncInvoke<T>;
/**
 * 代理后的行集
 */
export declare type XRowset<T extends RowObject = RowObject> = Rowset<T> & {
    readonly [P in ColumnsOf<T>]: Field<DataRowValueType<T[P]>, P>;
};
import { NamedSelect, XNamedSelect } from './named-select';
import { XTable, Table } from './table';
import { XTableVariant, TableVariant } from './table-variant';
import { XTableFuncInvoke, TableFuncInvoke } from './table-func-invoke';
