"use strict";
/**
 * 数据库行集，混入类型
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Rowset = void 0;
const sql_1 = require("../sql");
const expression_1 = require("../expression");
const object_1 = require("../object");
// TIPS: 请勿改变Rowset声明方式，Typescript不支持类声明合并,而使用type别名方式中的动态属性亦不可被继承
// eslint-disable-next-line @typescript-eslint/ban-types
class Rowset extends sql_1.Sql {
    constructor() {
        super();
        this.$name = undefined;
        /**
         * 别名
         */
        this.$alias = undefined; // 保留此初值，避免代理覆盖属性
        this.$tag = sql_1.SQL_SYMBOLE.ROWSET;
        /**
         * 属性-->字段 映射表
         */
        this.$map = undefined;
        // *******************添加代理字段名属性******************
        // ***** 修改继承原型链为 this > Rowset & FieldProxy > AST ******
        // **********************************************************
    }
    /**
     * 继承Rowset后必须在调用此函数，以达到创建代理Rowset的目的。
     */
    $proxy() {
        const proxied_proto = new Proxy(Object.create(Object.getPrototypeOf(this)), {
            get: (proto, key) => {
                if (key in proto) {
                    return Reflect.get(proto, key, this);
                }
                if (typeof key === 'string') {
                    // 两个$表示转义符
                    if (key.startsWith('$$')) {
                        key = key.substr(1);
                    }
                    return this.$field(key);
                }
                return undefined;
            },
        });
        Object.setPrototypeOf(this, proxied_proto);
        return this;
    }
    /**
     * 为当前表添加别名
     */
    as(alias) {
        if (this.$alias) {
            throw new Error(`Rowset is exists alias: ${this.$alias}`);
        }
        this.$alias = alias;
        return this;
    }
    /**
     * 字段
     * @param name 节点名称
     */
    $field(name) {
        if (!this.$name) {
            throw new Error('You must named rowset befor use field.');
        }
        const column = this.$map ? this.$map[name] : name;
        if (!column) {
            throw new Error(`Rowset map is not exisits property ${name} map`);
        }
        return new expression_1.Field(column, this);
    }
    /**
     * 获取所有字段
     */
    get star() {
        if (!this.$name) {
            throw new Error('You must named rowset befor use field.');
        }
        return new object_1.Star(this);
    }
    /**
     * 该方法为内部方法，用户请不要使用该方法
     */
    $around(columnMap) {
        this.$map = columnMap;
        return this;
    }
    static isRowset(object) {
        return object?.$tag === sql_1.SQL_SYMBOLE.ROWSET;
    }
}
exports.Rowset = Rowset;
