import { SQL_SYMBOLE } from '../sql';
import { Field } from '../expression';
import { ColumnsOf, DataRowValueType, RowObject } from '../types';
import { Rowset, XRowset } from './rowset';
import { DbType } from '../db-type';
import type { Condition } from '../condition';
export declare type TableVariantMember = PrimaryKey | CheckConstraint | UniqueKey | ColumnDeclareForAdd;
export declare class TableVariant<T extends RowObject = any> extends Rowset<T> {
    readonly $type: SQL_SYMBOLE.TABLE_VARIANT;
    $builtin: boolean;
    $body?: TableVariantMember[];
    $name: string;
    static isTableVariant(object: any): object is TableVariant;
    constructor(name: string);
    /**
     * 创建用于查询的别名
     */
    as(alias: string): XRowset<T>;
    /**
     * 表变量成员声明
     */
    as(members: TableVariantMember[]): this;
}
export declare type XTableVariant<T extends RowObject = RowObject> = TableVariant<T> & {
    readonly [P in ColumnsOf<T>]: Field<DataRowValueType<T[P]>, P>;
};
export declare function createTableVariant<T extends RowObject = any>(name: string, members: ((builder: TableVariantBuilder) => TableVariantMember[]) | TableVariantMember[]): XTableVariant<T>;
export declare namespace createTableVariant {
    var primaryKey: (name?: string | undefined) => PrimaryKey;
    var column: <N extends string, T extends DbType>(name: N, type: T) => ColumnDeclareForAdd<N, DbType>;
    var check: (nameOrSql: string | Condition, _sql?: Condition | undefined) => CheckConstraint;
    var uniqueKey: (name?: string | undefined) => UniqueKey;
}
export declare type TableVariantBuilder = typeof createTableVariant;
import { CheckConstraint, ColumnDeclareForAdd, PrimaryKey, UniqueKey } from '../statement';
