import Decimal from 'decimal.js-light';
import { Uuid } from './uuid';
import { Time } from './time';
export declare type UuidConstructor = typeof Uuid;
export declare type TimeConstructor = typeof Time;
export declare type DecimalConstructor = typeof Decimal;
export declare type Binary = Uint8Array;
/**
 * 对应数据库中的数组类型
 */
export declare type List<T extends Exclude<BaseScalar, Binary> = Exclude<BaseScalar, Binary>> = Array<T>;
/**
 * 数据库值类型
 */
export declare type Scalar = BaseScalar | List | Json;
/**
 * Json中的类型
 */
export declare type JsonTypes = string | number | boolean | null | JsonObject;
/**
 * JSON数据类型，对应数据库中的JSON类型
 */
export declare type JsonObject = {
    [K: string]: JsonTypes;
};
/**
 * Json类型，限定为JsonObject是为了避免类型冲突。
 */
export declare type Json = JsonObject;
/**
 * 标量类型，不含JSON类型及
 * 对应数据库标量类型的JS类型集
 */
export declare type BaseScalar = string | boolean | number | bigint | Decimal | Date | Time | Binary | Uuid | null;
export { Decimal, Uuid, Time };
/**
 * 所有数值类型
 */
export declare type Numeric = Decimal | bigint | number;
/**
 * 所有整形
 */
export declare type Integer = number | bigint;
/**
 * 浮点类型
 */
export declare type Float = number;
