"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isBinary = exports.isList = exports.isScalar = exports.isBaseScalar = void 0;
const scalar_1 = require("./scalar");
const time_1 = require("./time");
function isBaseScalar(value) {
    return (value === null ||
        typeof value === 'string' ||
        typeof value === 'boolean' ||
        typeof value === 'bigint' ||
        typeof value === 'number' ||
        value === undefined ||
        value instanceof Date ||
        value instanceof scalar_1.Uuid ||
        value instanceof scalar_1.Decimal ||
        value instanceof time_1.Time ||
        isBinary(value));
}
exports.isBaseScalar = isBaseScalar;
/**
 * 判断一个对象是否是集合
 */
function isScalar(value) {
    return (value !== undefined && !(Array.isArray(value) && !isBaseScalar(value[0])));
}
exports.isScalar = isScalar;
/**
 * 判断一个对象是不是一个数据库合法的列表(数组)类型
 */
function isList(value, MAX_SEARCH_SIZE = 10) {
    if (!Array.isArray(value))
        return false;
    for (let i = 0; i < Math.min(MAX_SEARCH_SIZE, value.length); i++) {
        if (!isBaseScalar(value[i]))
            return false;
    }
    return true;
}
exports.isList = isList;
/**
 * 判断是否为二进制类型
 * @param value
 * @returns
 */
function isBinary(value) {
    return value?.[Symbol.toStringTag] === 'Uint8Array';
}
exports.isBinary = isBinary;
