"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Uuid = void 0;
const uuid_1 = require("uuid");
class Uuid {
    constructor(strOrBuffer) {
        if (!(this instanceof Uuid)) {
            return new Uuid(strOrBuffer);
        }
        if (typeof strOrBuffer === 'string') {
            this._buffer = Array.from((0, uuid_1.parse)(strOrBuffer));
        }
        else if (strOrBuffer) {
            this._buffer = Array.from(strOrBuffer);
        }
        else {
            this._buffer = Uuid.DEFAULT;
        }
    }
    toString() {
        return (0, uuid_1.stringify)(this._buffer);
    }
    static new() {
        const uuid = new Uuid();
        (0, uuid_1.v4)({}, uuid._buffer);
        return uuid;
    }
    valueOf() {
        return this._buffer || Uuid.DEFAULT;
    }
    toJSON() {
        return this.toString();
    }
    static equals(left, right) {
        for (let i = 0; i < 16; i++) {
            if (left._buffer[i] !== right._buffer[i]) {
                return false;
            }
        }
        return true;
    }
}
exports.Uuid = Uuid;
Uuid.DEFAULT = [
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
];
Uuid.empty = new Uuid(Uuid.DEFAULT);
