"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.If = void 0;
const assert_1 = __importDefault(require("assert"));
const statement_1 = require("../statement");
const block_1 = require("./block");
class If extends statement_1.Statement {
    constructor(condition) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.IF;
        this.$tag = statement_1.STATEMENT_TAG.CONTROL;
        this.$condition = condition;
    }
    then(...args) {
        (0, assert_1.default)(!this.$then && !this.$else, `Syntax error.`);
        let then;
        if (args.length === 1) {
            if (Array.isArray(args[0])) {
                then = new block_1.Block(args[0]);
            }
            else {
                then = args[0];
            }
        }
        else {
            then = new block_1.Block(args);
        }
        this.$then = then;
        return this;
    }
    elseif(condition) {
        (0, assert_1.default)(!this.$else && this.$then, `Syntax error`);
        if (!this.$elseif) {
            this.$elseif = [];
        }
        const elseif = [condition, undefined];
        this.$elseif.push(elseif);
        return {
            then: (...args) => {
                let then;
                if (args.length === 1) {
                    if (Array.isArray(args[0])) {
                        then = new block_1.Block(args[0]);
                    }
                    else {
                        then = args[0];
                    }
                }
                else {
                    then = new block_1.Block(args);
                }
                elseif[1] = then;
                return this;
            },
        };
    }
    else(...args) {
        (0, assert_1.default)(this.$then && !this.$else, `Syntax error.`);
        let _else;
        if (args.length === 1) {
            if (Array.isArray(args[0])) {
                _else = new block_1.Block(args[0]);
            }
            else {
                _else = args[0];
            }
        }
        else {
            _else = new block_1.Block(args);
        }
        this.$else = _else;
        return this;
    }
    static isIf(object) {
        return statement_1.Statement.isStatement(object) && object.$kind === statement_1.STATEMENT_KIND.IF;
    }
}
exports.If = If;
