"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Fromable = void 0;
const assert_1 = __importDefault(require("assert"));
const condition_1 = require("../../../condition");
const rowset_1 = require("../../../rowset");
const join_1 = require("./join");
const statement_1 = require("../../statement");
class Fromable extends statement_1.Statement {
    /**
     * 从表中查询，可以查询多表
     * @param tables
     */
    from(...tables) {
        this.$froms = tables.map(table => rowset_1.Rowset.isRowset(table) ? table : rowset_1.Table.create(table));
        this.$froms.forEach(table => {
            if (!table.$alias) {
                if (!table.$name) {
                    throw new Error('Rowset must give a name before query');
                }
            }
        });
        return this;
    }
    /**
     * 表联接
     * @param table
     * @param on
     * @param left
     * @memberof Select
     */
    join(table, on, left) {
        (0, assert_1.default)(this.$froms, 'join must after from clause');
        if (!this.$joins) {
            this.$joins = [];
        }
        this.$joins.push(new join_1.Join(table, on, left));
        return this;
    }
    /**
     * 左联接
     * @param table
     * @param on
     */
    leftJoin(table, on) {
        return this.join(table, on, true);
    }
    ensureCondition(condition) {
        if (!condition_1.Condition.isCondition(condition)) {
            // 严格限制语法，字段名必须带上表别名
            (0, assert_1.default)(this.$froms, 'The `.from` must call before where when use `WhereObject<T>` condition type.');
            if (this.$froms?.length !== 1 || this.$joins) {
                throw new Error('WhereObject<T> condition is not support for multi rowset source select statement.');
            }
            const rowset = this.$froms?.[0];
            return condition_1.Condition.isCondition(condition)
                ? condition
                : condition_1.ConditionBuilder.parse(condition, rowset);
        }
        return condition;
    }
    where(...conditions) {
        (0, assert_1.default)(!this.$where, 'where is declared');
        if (conditions.length > 1) {
            this.$where = condition_1.ConditionBuilder.and(...conditions.map(cond => this.ensureCondition(cond)));
            return this;
        }
        this.$where = this.ensureCondition(conditions[0]);
        return this;
    }
    /**
     * 追加查询条件
     * @param condition
     * @returns
     */
    andWhere(condition) {
        if (!this.$where)
            return this.where(condition);
        this.$where = condition_1.ConditionBuilder.and(this.$where, this.ensureCondition(condition));
        return this;
    }
}
exports.Fromable = Fromable;
