"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Update = void 0;
const assert_1 = __importDefault(require("assert"));
const condition_1 = require("../../condition");
const expression_1 = require("../../expression");
const rowset_1 = require("../../rowset");
const statement_1 = require("../statement");
const fromable_1 = require("./common/fromable");
const control_1 = require("../control");
/**
 * Update 语句
 */
class Update extends fromable_1.Fromable {
    constructor(table) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.UPDATE;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        const tb = rowset_1.Table.isTable(table) || rowset_1.TableVariant.isTableVariant(table)
            ? table
            : rowset_1.Table.create(table);
        if (tb.$alias) {
            this.from(tb);
        }
        this.$table = tb;
    }
    set(...sets) {
        (0, assert_1.default)(!this.$sets, 'set statement is declared');
        (0, assert_1.default)(sets.length > 0, 'sets must have more than 0 items');
        if (sets.length === 1) {
            if (Array.isArray(sets[0])) {
                this.$sets = sets[0];
                return this;
            }
            else {
                const item = sets[0];
                this.$sets = Object.entries(item).map(([key, value]) => new control_1.Assignment(this.$table.$field(key), expression_1.Expression.ensureExpression(value)));
                return this;
            }
        }
        this.$sets = sets;
        return this;
    }
    ensureCondition(condition) {
        if (!condition_1.Condition.isCondition(condition)) {
            return condition_1.ConditionBuilder.parse(condition, this.$table);
        }
        return condition;
    }
    static isUpdate(object) {
        return (statement_1.Statement.isStatement(object) && object.$kind === statement_1.STATEMENT_KIND.UPDATE);
    }
}
exports.Update = Update;
