"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.With = void 0;
const sql_1 = require("../../sql");
const rowset_1 = require("../../rowset");
const delete_1 = require("./delete");
const insert_1 = require("./insert");
const select_1 = require("./select");
const update_1 = require("./update");
class With extends sql_1.Sql {
    /**
     * With结构
     */
    constructor(...items) {
        super();
        this.$type = sql_1.SQL_SYMBOLE.WITH;
        /**
         * select查询
         */
        this.select = (...args) => {
            const sql = new select_1.Select(...args);
            sql.$with = this;
            return sql;
        };
        if (items.length === 0 &&
            !Array.isArray(items[0]) &&
            !rowset_1.WithSelect.isWithSelect(items[0])) {
            this.$rowsets = Object.entries(items[0]).map(([name, sel]) => rowset_1.WithSelect.create(name, sel));
        }
        else if (Array.isArray(items[0])) {
            this.$rowsets = items[0];
        }
        else {
            this.$rowsets = items;
        }
    }
    /**
     * 插入至表,into的别名
     * @param table
     * @param fields
     */
    insert(table, fields) {
        const sql = new insert_1.Insert(table, fields);
        sql.$with = this;
        return sql;
    }
    /**
     * 更新一个表格
     * @param table
     */
    update(table) {
        const sql = new update_1.Update(table);
        sql.$with = this;
        return sql;
    }
    /**
     * 删除一个表格
     * @param table 表格
     */
    delete(table) {
        const sql = new delete_1.Delete(table);
        sql.$with = this;
        return sql;
    }
}
exports.With = With;
