"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlterFunctionBuilder = exports.AlterFunction = void 0;
const statement_1 = require("../statement");
const expression_1 = require("../../expression");
class AlterFunction extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.ALTER_FUNCTION;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isAlterFunction(object) {
        return (statement_1.Statement.isStatement(object) &&
            object.$kind === statement_1.STATEMENT_KIND.ALTER_FUNCTION);
    }
    params(...args) {
        if (this.$params) {
            throw new Error(`AlterFunction parameters is declared.`);
        }
        if (args.length === 1 && typeof args[0] === 'function') {
            args = args[0](AlterFunctionBuilder.param);
        }
        if (args.findIndex((p) => !p) >= 0) {
            throw new Error(`Function parameter must not empty.`);
        }
        this.$params = args;
        return this;
    }
    returns(returns) {
        this.$returns = returns;
        return this;
    }
    /**
     * 定义函数体
     */
    as(...sql) {
        if (typeof sql[0] === 'function') {
            sql = sql[0](this.$params || []);
        }
        if (sql.length === 1 && Array.isArray(sql[0])) {
            sql = sql[0];
        }
        this.$body = sql;
        return this;
    }
}
exports.AlterFunction = AlterFunction;
function AlterFunctionBuilder(name) {
    return new AlterFunction(name);
}
exports.AlterFunctionBuilder = AlterFunctionBuilder;
AlterFunctionBuilder.param = (name, dataType, defaultValue) => {
    return expression_1.FunctionParameter.create(name, dataType, defaultValue);
};
