"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlterProcedureBuilder = exports.AlterProcedure = void 0;
const expression_1 = require("../../expression");
const statement_1 = require("../statement");
class AlterProcedure extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.ALTER_PROCEDURE;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isAlterProcedure(object) {
        return (statement_1.Statement.isStatement(object) &&
            object.$kind === statement_1.STATEMENT_KIND.ALTER_PROCEDURE);
    }
    params(...args) {
        if (this.$params) {
            throw new Error(`AlterProcedure parameters is declared.`);
        }
        if (args.length === 1 && typeof args[0] === 'function') {
            args = args[0](exports.AlterProcedureBuilder.param);
        }
        if (args.findIndex((p) => !p) >= 0) {
            throw new Error(`Procedure parameter must not empty.`);
        }
        this.$params = args;
        return this;
    }
    as(...sql) {
        if (typeof sql[0] === 'function') {
            sql = sql[0](this.$params || []);
        }
        if (sql.length === 1 && Array.isArray(sql[0])) {
            sql = sql[0];
        }
        this.$body = sql;
        return this;
    }
}
exports.AlterProcedure = AlterProcedure;
const AlterProcedureBuilder = (name) => {
    return new AlterProcedure(name);
};
exports.AlterProcedureBuilder = AlterProcedureBuilder;
exports.AlterProcedureBuilder.param = (name, dataType, direct = 'IN', defaultValue) => expression_1.ProcedureParameter.create(name, dataType, direct, defaultValue);
