import { Condition } from '../../condition';
import { DbType } from '../../db-type';
import { XObjectName } from '../../object';
import { Statement, STATEMENT_KIND, STATEMENT_TAG } from '../statement';
import { AlterTableDropMember } from './alter-table-drop-member';
import { CheckConstraint, CheckConstraintBuilder } from './check-constraint';
import { ColumnDeclareForAdd } from './column-declare-for-add';
import { ColumnDeclareForAlter } from './column-declare-for-alter';
import { CreateTableMember } from './create-table';
import { ForeignKey } from './foreign-key';
import { PrimaryKey } from './primary-key';
import { UniqueKey } from './unique-key';
export declare class AlterTable<N extends string = string> extends Statement {
    static isAlterTable(object: any): object is AlterTable;
    $kind: STATEMENT_KIND.ALTER_TABLE;
    $name: XObjectName<N>;
    $adds?: AlterTableAddMember[];
    $drop?: AlterTableDropMember;
    $alterColumn?: ColumnDeclareForAlter;
    $tag: STATEMENT_TAG.EXECUTABLE;
    constructor(name: XObjectName<N>);
    private _assertDrop;
    private _assertAdd;
    add(build: (builder: AlterTableBuilder['add']) => AlterTableAddMember | AlterTableAddMember[]): this;
    add(...members: AlterTableAddMember[]): this;
    addColumn(columnOrBuilder: ColumnDeclareForAdd | ((columnBuilder: <N extends string, T extends DbType>(name: N, type: T) => ColumnDeclareForAdd<N>) => ColumnDeclareForAdd)): this;
    addPrimaryKey(keyOrBuilder: PrimaryKey | ((keyBuilder: (name?: string) => PrimaryKey) => PrimaryKey)): this;
    addForeignKey(keyOrBuilder: ForeignKey | ((keyBuilder: (name?: string) => ForeignKey) => ForeignKey)): this;
    addUnionKey(keyOrBuilder: UniqueKey | ((keyBuilder: (name?: string) => UniqueKey) => UniqueKey)): this;
    addCheck(keyOrBuilder: CheckConstraint | ((checkBuilder: CheckConstraintBuilder) => CheckConstraint)): this;
    drop(member: AlterTableDropMember): this;
    drop(build: (builder: AlterTableBuilder['drop']) => AlterTableDropMember): this;
    dropColumn(name: string): this;
    dropPrimaryKey(name: string): this;
    dropForeignKey(name: string): this;
    dropCheck(name: string): this;
    dropUniqueKey(name: string): this;
    alterColumn(buildColumn: ColumnDeclareForAlter | ((builder: (name: string, type: DbType) => ColumnDeclareForAlter) => ColumnDeclareForAlter)): this;
}
export declare function AlterTableBuilder(name: XObjectName): AlterTable;
export declare namespace AlterTableBuilder {
    var add: {
        column<N extends string, T extends DbType>(name: N, type: T): ColumnDeclareForAdd<N, DbType>;
        primaryKey(name?: string | undefined): PrimaryKey;
        foreignKey(name?: string | undefined): ForeignKey;
        check(nameOrSql: string | Condition, _sql?: Condition | undefined): CheckConstraint;
        uniqueKey(name?: string | undefined): UniqueKey;
    };
    var drop: {
        column(name: string): AlterTableDropMember;
        primaryKey(name: string): AlterTableDropMember;
        foreignKey(name: string): AlterTableDropMember;
        check(name: string): AlterTableDropMember;
        uniqueKey(name: string): AlterTableDropMember;
    };
    var alter: {
        column<N extends string, T extends DbType>(name: N, type: T): ColumnDeclareForAlter<N, DbType>;
    };
}
export declare type AlterTableBuilder = typeof AlterTableBuilder;
export declare type AlterTableAddMember = CreateTableMember;
