"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlterTableBuilder = exports.AlterTable = void 0;
const statement_1 = require("../statement");
const alter_table_drop_member_1 = require("./alter-table-drop-member");
const check_constraint_1 = require("./check-constraint");
const column_declare_for_add_1 = require("./column-declare-for-add");
const column_declare_for_alter_1 = require("./column-declare-for-alter");
const foreign_key_1 = require("./foreign-key");
const primary_key_1 = require("./primary-key");
const table_member_1 = require("./table-member");
const unique_key_1 = require("./unique-key");
class AlterTable extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.ALTER_TABLE;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isAlterTable(object) {
        return (statement_1.Statement.isStatement(object) &&
            object.$kind === statement_1.STATEMENT_KIND.ALTER_TABLE);
    }
    _assertDrop() {
        if (this.$adds || this.$alterColumn) {
            throw new Error(`A alter statement is only used by add or drop.`);
        }
        if (this.$drop) {
            throw new Error(`A drop item is exists in it.`);
        }
        // if (!this.$drops) {
        //   this.$drops = [];
        // }
    }
    _assertAdd() {
        if (this.$drop || this.$alterColumn) {
            throw new Error(`A alter statement is only used by add or drop.`);
        }
        if (!this.$adds) {
            this.$adds = [];
        }
    }
    add(...members) {
        this._assertAdd();
        if (typeof members[0] === 'function') {
            const ret = members[0](AlterTableBuilder.add);
            members = Array.isArray(ret) ? ret : [ret];
            this.add(...members);
            return this;
        }
        this.$adds.push(...members);
        return this;
    }
    addColumn(columnOrBuilder) {
        this._assertAdd();
        let column;
        if (typeof columnOrBuilder === 'function') {
            column = columnOrBuilder(AlterTableBuilder.add.column);
        }
        else {
            column = columnOrBuilder;
        }
        this.$adds.push(column);
        return this;
    }
    addPrimaryKey(keyOrBuilder) {
        this._assertAdd();
        let key;
        if (typeof keyOrBuilder === 'function') {
            key = keyOrBuilder(AlterTableBuilder.add.primaryKey);
        }
        else {
            key = keyOrBuilder;
        }
        this.$adds.push(key);
        return this;
    }
    addForeignKey(keyOrBuilder) {
        this._assertAdd();
        let key;
        if (typeof keyOrBuilder === 'function') {
            key = keyOrBuilder(AlterTableBuilder.add.foreignKey);
        }
        else {
            key = keyOrBuilder;
        }
        this.$adds.push(key);
        return this;
    }
    addUnionKey(keyOrBuilder) {
        this._assertAdd();
        let key;
        if (typeof keyOrBuilder === 'function') {
            key = keyOrBuilder(AlterTableBuilder.add.uniqueKey);
        }
        else {
            key = keyOrBuilder;
        }
        this.$adds.push(key);
        return this;
    }
    addCheck(keyOrBuilder) {
        this._assertAdd();
        let check;
        if (typeof keyOrBuilder === 'function') {
            check = keyOrBuilder(AlterTableBuilder.add.check);
        }
        else {
            check = keyOrBuilder;
        }
        this.$adds.push(check);
        return this;
    }
    drop(memberOrBuilder) {
        this._assertDrop();
        if (typeof memberOrBuilder === 'function') {
            const member = memberOrBuilder(AlterTableBuilder.drop);
            this.$drop = member;
            return this;
        }
        this.$drop = memberOrBuilder;
        return this;
    }
    dropColumn(name) {
        this._assertDrop();
        this.$drop = AlterTableBuilder.drop.column(name);
        return this;
    }
    dropPrimaryKey(name) {
        this._assertDrop();
        this.$drop = AlterTableBuilder.drop.primaryKey(name);
        return this;
    }
    dropForeignKey(name) {
        this._assertDrop();
        this.$drop = AlterTableBuilder.drop.foreignKey(name);
        return this;
    }
    dropCheck(name) {
        this._assertDrop();
        this.$drop = AlterTableBuilder.drop.check(name);
        return this;
    }
    dropUniqueKey(name) {
        this._assertDrop();
        this.$drop = AlterTableBuilder.drop.uniqueKey(name);
        return this;
    }
    alterColumn(buildColumn) {
        if (this.$adds || this.$drop || this.$alterColumn) {
            throw new Error(`Alter statement is used only one of add or drop or alterColumn.`);
        }
        if (typeof buildColumn === 'function') {
            this.$alterColumn = buildColumn(AlterTableBuilder.alter.column);
        }
        else {
            this.$alterColumn = buildColumn;
        }
        return this;
    }
}
exports.AlterTable = AlterTable;
function AlterTableBuilder(name) {
    return new AlterTable(name);
}
exports.AlterTableBuilder = AlterTableBuilder;
AlterTableBuilder.add = {
    column(name, type) {
        return new column_declare_for_add_1.ColumnDeclareForAdd(name, type);
    },
    primaryKey(name) {
        return new primary_key_1.PrimaryKey(name);
    },
    foreignKey(name) {
        return new foreign_key_1.ForeignKey(name);
    },
    check(nameOrSql, _sql) {
        let name;
        let sql;
        if (typeof nameOrSql === 'string') {
            name = nameOrSql;
            sql = _sql;
        }
        else {
            sql = nameOrSql;
        }
        return new check_constraint_1.CheckConstraint(sql, name);
    },
    uniqueKey(name) {
        return new unique_key_1.UniqueKey(name);
    },
};
AlterTableBuilder.drop = {
    column(name) {
        return new alter_table_drop_member_1.AlterTableDropMember(table_member_1.SQL_SYMBOLE_TABLE_MEMBER.COLUMN, name);
    },
    primaryKey(name) {
        return new alter_table_drop_member_1.AlterTableDropMember(table_member_1.SQL_SYMBOLE_TABLE_MEMBER.PRIMARY_KEY, name);
    },
    foreignKey(name) {
        return new alter_table_drop_member_1.AlterTableDropMember(table_member_1.SQL_SYMBOLE_TABLE_MEMBER.FOREIGN_KEY, name);
    },
    check(name) {
        return new alter_table_drop_member_1.AlterTableDropMember(table_member_1.SQL_SYMBOLE_TABLE_MEMBER.CHECK_CONSTRAINT, name);
    },
    uniqueKey(name) {
        return new alter_table_drop_member_1.AlterTableDropMember(table_member_1.SQL_SYMBOLE_TABLE_MEMBER.UNIQUE_KEY, name);
    },
};
AlterTableBuilder.alter = {
    column(name, type) {
        return new column_declare_for_alter_1.ColumnDeclareForAlter(name, type);
    },
};
