"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateFunctionBuilder = exports.CreateFunction = void 0;
const statement_1 = require("../statement");
const expression_1 = require("../../expression");
class CreateFunction extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.CREATE_FUNCTION;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isCreateFunction(object) {
        return (statement_1.Statement.isStatement(object) &&
            object.$kind === statement_1.STATEMENT_KIND.CREATE_FUNCTION);
    }
    params(...args) {
        if (this.$params) {
            throw new Error(`CreateFunction parameters is declared.`);
        }
        if (args.length === 1 && typeof args[0] === 'function') {
            args = args[0](CreateFunctionBuilder.param);
        }
        if (args.findIndex((p) => !p) >= 0) {
            throw new Error(`Function parameter must not empty.`);
        }
        this.$params = args;
        return this;
    }
    returns(returns) {
        this.$returns = returns;
        return this;
    }
    as(...sql) {
        if (typeof sql[0] === 'function') {
            sql = sql[0](this.$params || []);
        }
        if (sql.length === 1 && Array.isArray(sql[0])) {
            sql = sql[0];
        }
        this.$body = sql;
        return this;
    }
}
exports.CreateFunction = CreateFunction;
function CreateFunctionBuilder(name) {
    return new CreateFunction(name);
}
exports.CreateFunctionBuilder = CreateFunctionBuilder;
CreateFunctionBuilder.param = function param(name, dataType, defaultValue) {
    return expression_1.FunctionParameter.create(name, dataType, defaultValue);
};
CreateFunctionBuilder.ifNotExists = function ifNotExists(name) {
    const statement = CreateFunctionBuilder(name);
    statement.$behavior = 'CREATE_IF_NOT_EXISTS';
    return statement;
};
