"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateIndex = void 0;
const statement_1 = require("../statement");
class CreateIndex extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.CREATE_INDEX;
        this.$clustered = false;
        this.$unique = false;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isCreateIndex(object) {
        return statement_1.Statement.isStatement(object) && object.$kind === statement_1.STATEMENT_KIND.CREATE_INDEX;
    }
    clustered() {
        this.$clustered = true;
        return this;
    }
    unique() {
        this.$unique = true;
        return this;
    }
    on(table, columns) {
        if (this.$table) {
            throw new Error(`Table & Columns is defined.`);
        }
        this.$table = table;
        if (this.$columns) {
            throw new Error(`Columns is defined.`);
        }
        if (Array.isArray(columns)) {
            if (columns.length === 0) {
                throw new Error(`Primary key must have a column.`);
            }
            if (typeof columns[0] === 'string') {
                this.$columns = columns.map(name => ({
                    name,
                    sort: 'ASC',
                }));
            }
            else {
                this.$columns = columns;
            }
            return this;
        }
        this.$columns = Object.entries(columns).map(([name, sort]) => ({
            name,
            sort,
        }));
        return this;
    }
}
exports.CreateIndex = CreateIndex;
