"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateProcedureBuilder = exports.CreateProcedure = void 0;
const expression_1 = require("../../expression");
const statement_1 = require("../statement");
class CreateProcedure extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.CREATE_PROCEDURE;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isCreateProcedure(object) {
        return (statement_1.Statement.isStatement(object) &&
            object.$kind === statement_1.STATEMENT_KIND.CREATE_PROCEDURE);
    }
    params(...args) {
        if (this.$params) {
            throw new Error(`CreateProcedure parameters is declared.`);
        }
        if (args.length === 1 && typeof args[0] === 'function') {
            args = args[0](exports.CreateProcedureBuilder.param);
        }
        if (args.findIndex((p) => !p) >= 0) {
            throw new Error(`Procedure parameter must not empty.`);
        }
        this.$params = args;
        return this;
    }
    as(...sql) {
        if (typeof sql[0] === 'function') {
            sql = sql[0](this.$params || []);
        }
        this.$body = sql;
        return this;
    }
}
exports.CreateProcedure = CreateProcedure;
const CreateProcedureBuilder = (name) => {
    return new CreateProcedure(name);
};
exports.CreateProcedureBuilder = CreateProcedureBuilder;
exports.CreateProcedureBuilder.param = (name, dataType, direct = 'IN', defaultValue) => expression_1.ProcedureParameter.create(name, dataType, direct, defaultValue);
exports.CreateProcedureBuilder.ifNotExists = function (name) {
    const statement = (0, exports.CreateProcedureBuilder)(name);
    statement.$behavior = 'CREATE_IF_NOT_EXISTS';
    return statement;
};
