import { Condition } from '../../condition';
import { DbType } from '../../db-type';
import { XObjectName } from '../../object';
import { Statement, STATEMENT_KIND, STATEMENT_TAG } from '../statement';
import { CheckConstraint } from './check-constraint';
import { ColumnDeclareForAdd } from './column-declare-for-add';
import { ForeignKey } from './foreign-key';
import { PrimaryKey } from './primary-key';
import { CreateBehavior } from './types';
import { UniqueKey } from './unique-key';
export declare class CreateTable<N extends string = string> extends Statement {
    static isCreateTable(object: any): object is CreateTable;
    readonly $kind: STATEMENT_KIND.CREATE_TABLE;
    $body?: CreateTableMember[];
    $name: XObjectName<N>;
    $behavior: CreateBehavior;
    $tag: STATEMENT_TAG.EXECUTABLE;
    constructor(name: XObjectName<N>);
    /**
     * 定义表成员，使用构造器
     */
    as(build: (builder: CreateTableBuilder) => CreateTableMember[]): this;
    /**
     * 定义表成员，使用表成员变量
     */
    as(...members: CreateTableMember[]): this;
    as(members: CreateTableMember[]): this;
}
export declare type CreateTableMember = ColumnDeclareForAdd | PrimaryKey | ForeignKey | CheckConstraint | UniqueKey;
export declare function CreateTableBuilder<N extends string>(name: XObjectName<N>): CreateTable<N>;
export declare namespace CreateTableBuilder {
    var primaryKey: (name?: string | undefined) => PrimaryKey;
    var column: <N extends string, T extends DbType>(name: N, type: T) => ColumnDeclareForAdd<N, DbType>;
    var check: (nameOrSql: string | Condition, _sql?: Condition | undefined) => CheckConstraint;
    var uniqueKey: (name?: string | undefined) => UniqueKey;
    var foreignKey: (name?: string | undefined) => ForeignKey;
    var ifNotExists: <N extends string>(name: XObjectName<N>) => CreateTable<N>;
}
export declare type CreateTableBuilder = typeof CreateTableBuilder;
