"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateTableBuilder = exports.CreateTable = void 0;
const statement_1 = require("../statement");
const check_constraint_1 = require("./check-constraint");
const column_declare_for_add_1 = require("./column-declare-for-add");
const foreign_key_1 = require("./foreign-key");
const primary_key_1 = require("./primary-key");
const unique_key_1 = require("./unique-key");
class CreateTable extends statement_1.Statement {
    constructor(name) {
        super();
        this.$kind = statement_1.STATEMENT_KIND.CREATE_TABLE;
        this.$behavior = 'CREATE';
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$name = name;
    }
    static isCreateTable(object) {
        return (statement_1.Statement.isStatement(object) &&
            object.$kind === statement_1.STATEMENT_KIND.CREATE_TABLE);
    }
    as(...members) {
        if (this.$body) {
            throw new Error(`CreateTable body is declared.`);
        }
        if (Array.isArray(members[0])) {
            members = members[0];
        }
        if (typeof members[0] === 'function') {
            this.as(...members[0](CreateTableBuilder));
            return this;
        }
        this.$body = members;
        return this;
    }
}
exports.CreateTable = CreateTable;
function CreateTableBuilder(name) {
    return new CreateTable(name);
}
exports.CreateTableBuilder = CreateTableBuilder;
CreateTableBuilder.primaryKey = (name) => {
    return new primary_key_1.PrimaryKey(name);
};
CreateTableBuilder.column = (name, type) => {
    return new column_declare_for_add_1.ColumnDeclareForAdd(name, type);
};
CreateTableBuilder.check = (nameOrSql, _sql) => {
    let name;
    let sql;
    if (typeof nameOrSql === 'string') {
        name = nameOrSql;
        sql = _sql;
    }
    else {
        sql = nameOrSql;
    }
    return new check_constraint_1.CheckConstraint(sql, name);
};
CreateTableBuilder.uniqueKey = function uniqueKey(name) {
    return new unique_key_1.UniqueKey(name);
};
CreateTableBuilder.foreignKey = function foreignKey(name) {
    return new foreign_key_1.ForeignKey(name);
};
CreateTableBuilder.ifNotExists = function ifNotExists(name
// members?:
//   | ((builder: CreateTableBuilder) => CreateTableMember[])
//   | CreateTableMember[]
) {
    const table = CreateTableBuilder(name);
    table.$behavior = 'CREATE_IF_NOT_EXISTS';
    return table;
};
