"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrimaryKey = void 0;
const sql_1 = require("../../sql");
class PrimaryKey extends sql_1.Sql {
    constructor(name, columns) {
        super();
        this.$type = sql_1.SQL_SYMBOLE.PRIMARY_KEY;
        /**
         * 声明为非聚焦主键
         */
        this.$nonclustered = false;
        this.$name = name;
        if (columns) {
            this.on(columns);
        }
    }
    static isPrimaryKey(object) {
        return object?.$type === sql_1.SQL_SYMBOLE.PRIMARY_KEY;
    }
    on(...args) {
        if (this.$columns) {
            throw new Error(`Columns is defined.`);
        }
        if (args.length === 0) {
            throw new Error(`Primary key must have a column.`);
        }
        let keyColumns;
        if (typeof args[0] === 'string') {
            keyColumns = args.map(name => ({
                name,
                sort: 'ASC',
            }));
        }
        else if (Array.isArray(args[0])) {
            if (typeof args[0][0] === 'string') {
                keyColumns = args[0].map(name => ({
                    name,
                    sort: 'ASC',
                }));
            }
            else {
                keyColumns = args[0];
            }
        }
        else if (typeof args[0] === 'object') {
            if (Reflect.get(args[0], 'name') && Reflect.get(args[0], 'sort')) {
                // 如果是单个 KeyColumn
                keyColumns = args;
            }
            else {
                keyColumns = Object.entries(args[0]).map(([name, sort]) => ({
                    name,
                    sort,
                }));
            }
        }
        else {
            throw new Error(`Invalid arguments.`);
        }
        this.$columns = keyColumns;
        return this;
    }
    withNoclustered() {
        this.$nonclustered = true;
        return this;
    }
}
exports.PrimaryKey = PrimaryKey;
