"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareList = exports.compareObject = exports.compareScalar = exports.getType = exports.isObejctType = exports.isCompareScalarType = exports.isListType = exports.ValueType = void 0;
const core_1 = require("../core");
var ValueType;
(function (ValueType) {
    ValueType[ValueType["scalar"] = 0] = "scalar";
    ValueType[ValueType["object"] = 1] = "object";
    ValueType[ValueType["list"] = 2] = "list";
})(ValueType = exports.ValueType || (exports.ValueType = {}));
function isListType(value) {
    return getType(value) === ValueType.list;
}
exports.isListType = isListType;
function isCompareScalarType(value) {
    return getType(value) === ValueType.scalar;
}
exports.isCompareScalarType = isCompareScalarType;
function isObejctType(value) {
    return getType(value) === ValueType.object;
}
exports.isObejctType = isObejctType;
function getType(value) {
    if ((0, core_1.isBaseScalar)(value))
        return ValueType.scalar;
    if (Array.isArray(value)) {
        const el1Type = getType(value[0]);
        if (el1Type === ValueType.object) {
            return ValueType.list;
        }
        return ValueType.scalar;
    }
    if (typeof value === 'object') {
        return ValueType.object;
    }
    return ValueType.scalar;
}
exports.getType = getType;
/**
 * 对比标量类型值是否相同
 * @param source
 * @param target
 * @returns
 */
function compareScalar(source, target, equalsComparator, path = '') {
    if (source === target)
        return null;
    // 优先使用指定的比较器
    let changed = equalsComparator?.(source, target, path);
    if (changed !== undefined) {
        if (changed) {
            return null;
        }
        else {
            return {
                source,
                target,
            };
        }
    }
    if ((source === null || source === undefined) &&
        (target === null || target === undefined)) {
        return null;
    }
    if (source === null ||
        source === undefined ||
        target === null ||
        target === undefined)
        return {
            source,
            target,
        };
    if (changed !== undefined) {
        if (Array.isArray(source) && Array.isArray(target)) {
            if (source.length !== target.length) {
                changed = true;
            }
            else {
                for (let i = 0; i < source.length; i++) {
                    if (!compareScalar(source[i], target[i], equalsComparator, path + '[]')) {
                        changed = true;
                        break;
                    }
                }
                changed = false;
            }
        }
        else if ((0, core_1.isBinary)(source)) {
            changed =
                Buffer.compare(Buffer.from(source), Buffer.from(target)) === 0;
        }
        else if (source instanceof Date && target instanceof Date) {
            changed = source.getTime() === target.getTime();
        }
    }
    if (changed) {
        return {
            target,
            source,
        };
    }
    return null;
}
exports.compareScalar = compareScalar;
// export function compareScalar1<T extends ScalarType>(
//   source: any,
//   target: any,
//   equalsComparator?: (
//     left: ScalarType,
//     right: ScalarType,
//     path: string
//   ) => boolean | undefined,
//   path: string = ''
// ): ScalarDifference<T> | null {
//   if (source === null) source = undefined;
//   if (target === null) target = undefined;
//   if (source === target) return null;
//   if (source === undefined || target === undefined)
//     return {
//       source,
//       target,
//     };
//   // 优先使用指定的比较器
//   const comparatorResult = equalsComparator?.(source, target, path);
//   if (comparatorResult !== undefined) {
//     if (comparatorResult) {
//       return null;
//     } else {
//       return {
//         source,
//         target,
//       };
//     }
//   }
//   if (!compareScalar(source, target)) {
//     return {
//       source,
//       target,
//     };
//   }
//   return null;
// }
function compareObject(source, target, isKeyEquals, isEquals, 
// 对比的路径
path = '') {
    if (source === target)
        return null;
    const equals = isEquals?.(source, target, path);
    if (equals === true) {
        return null;
    }
    if (!source && !target) {
        return null;
    }
    if (source && !target) {
        return {
            source,
            target,
            added: source,
        };
    }
    if (!source && target) {
        return {
            source,
            target,
            removed: target,
        };
    }
    const keyTypes = {};
    Object.entries(source).forEach(([key, value]) => {
        keyTypes[key] = getType(value);
    });
    Object.entries(target).forEach(([key, value]) => {
        keyTypes[key] = Math.max(keyTypes[key] || 0, getType(value));
    });
    const changes = {};
    for (const [key, type] of Object.entries(keyTypes)) {
        const propSource = Reflect.get(source, key);
        const propTarget = Reflect.get(target, key);
        if (type === ValueType.scalar) {
            const ch = compareScalar(propSource, propTarget, isEquals, [path, key].filter(p => p).join('.'));
            if (ch) {
                changes[key] = ch;
            }
        }
        else if (type === ValueType.list) {
            if (!isKeyEquals)
                throw new Error(`List object or list property compare must provide 'isSameObject' argument.`);
            const compareResult = compareList(propSource, propTarget, isKeyEquals, isEquals, [path, key].filter(p => p).join('.'));
            if (compareResult) {
                changes[key] = compareResult;
            }
        }
        else if (type === ValueType.object) {
            const compareResult = compareObject(propSource, propTarget, isKeyEquals, isEquals, [path, key].filter(p => p).join('.'));
            if (compareResult) {
                changes[key] = compareResult;
            }
        }
    }
    if (Object.keys(changes).length > 0) {
        return {
            source,
            target,
            changes,
        };
    }
    return null;
}
exports.compareObject = compareObject;
function compareList(source, target, 
// keyer: (item: T) => K,
// keyComparator: (left: K, right: K) => boolean
// 确定是否同一对象
isKeyEquals, isEquals, 
// 对比的路径
path = '') {
    if (source === target)
        return null;
    const equals = isEquals?.(source, target, path);
    if (equals === true) {
        return null;
    }
    // const sourceMap = map(source, keyer);
    // const targetMap = map(target, keyer);
    if (!source && target) {
        return {
            source,
            target,
            addeds: [],
            removeds: target,
            changes: [],
        };
    }
    if (!target && source) {
        return {
            source,
            target,
            addeds: source,
            removeds: [],
            changes: [],
        };
    }
    const addeds = source.filter(left => !target.find(right => isKeyEquals(left, right, path)));
    const removeds = target.filter(left => !source.find(right => isKeyEquals(left, right, path)));
    const changes = [];
    source.forEach(source => target.forEach(target => {
        if (isKeyEquals(source, target, path)) {
            const itemChanges = compareObject(source, target, isKeyEquals, isEquals, path + '[]');
            if (itemChanges) {
                changes.push(itemChanges);
            }
        }
    }));
    if (addeds.length === 0 && removeds.length === 0 && changes.length === 0) {
        return null;
    }
    return {
        source,
        target,
        addeds,
        removeds,
        changes,
    };
}
exports.compareList = compareList;
// export function compare<T extends ScalarType>(
//   source: T,
//   target: T
// ): ScalarDifference<T> | null;
// export function compare<T extends ListType>(
//   source: T,
//   target: T,
//   isSameObject: ObjectKeyCompartor
// ): ListDifference<T> | null;
// export function compare<T extends ObjectType>(
//   source: T,
//   target: T,
//   isSameObject: ObjectKeyCompartor
// ): ObjectDifference<T> | null;
// export function compare(
//   source: any,
//   target: any,
//   isSameObject?: ObjectKeyCompartor
// ): ScalarDifference<any> | ListDifference<any> | ObjectDifference<any> | null {
//   const type = Math.max(getType(source), getType(target));
//   if (type === ValueType.scalar) {
//     return compareScalar(source, target);
//   }
//   if (type === ValueType.list) {
//     if (!isSameObject) {
//       throw new Error(`Compare list must provide argument: 'isSanmeObject'`);
//     }
//     return compareList(source, target, isSameObject!);
//   }
//   return compareObject(source, target, isSameObject);
// }
