import { AlterFunction, AlterProcedure, AlterTable, AlterView, ColumnsOf, XObjectName, XTables, CreateFunction, CreateIndex, CreateProcedure, CreateSequence, CreateTable, CreateView, Delete, DropFunction, DropIndex, DropProcedure, DropSequence, DropTable, DropView, Field, Insert, RowObject, Scalar, Statement, Update, XExpression, ExecutableStatement } from '../core';
export declare abstract class MigrateBuilder {
    sql(statement: Statement | string): ExecutableStatement;
    createTable(name: XObjectName): CreateTable;
    alterTable(name: XObjectName): AlterTable;
    dropTable(name: XObjectName): DropTable;
    createView(name: XObjectName): CreateView;
    alterView(name: XObjectName): AlterView;
    dropView(name: XObjectName): DropView;
    createProcedure(name: XObjectName): CreateProcedure;
    alterProcedure(name: XObjectName): AlterProcedure;
    dropProcedure(name: XObjectName): DropProcedure;
    createFunction(name: XObjectName): CreateFunction;
    alterFunction(name: XObjectName): AlterFunction;
    dropFunction(name: XObjectName): DropFunction;
    createSequence(name: XObjectName): CreateSequence;
    createIndex(name: string): CreateIndex;
    dropIndex(table: XObjectName, name: string): DropIndex;
    dropSequence(name: XObjectName): DropSequence;
    insert<T extends RowObject = any>(table: XTables<T>, fields?: ColumnsOf<T>[] | Field<Scalar, ColumnsOf<T>>[]): Insert<T>;
    /**
     * 更新一个表格
     * @param table
     */
    update<T extends RowObject = any>(table: XTables<T>): Update<T>;
    /**
     * 删除一个表格
     * @param table 表格
     */
    delete<T extends RowObject = any>(table: XTables<T>): Delete<T>;
    abstract renameTable(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract renameColumn(table: XObjectName, name: string, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract renameView(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract renameIndex(table: XObjectName, name: string, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract renameSequence(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract renameProcedure(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract renameFunction(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    abstract setTableComment(name: XObjectName, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setViewComment(name: XObjectName, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setColumnComment(table: XObjectName, name: string, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setIndexComment(table: XObjectName, name: string, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setConstraintComment(table: XObjectName, name: string, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setSchemaComment(name: string, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setSequenceComment(name: XObjectName, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setProcedureComment(name: XObjectName, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract setFunctionComment(name: XObjectName, comment: string): ExecutableStatement | ExecutableStatement[];
    abstract dropSchemaComment(name: string): ExecutableStatement | ExecutableStatement[];
    abstract dropSequenceComment(name: XObjectName): ExecutableStatement | ExecutableStatement[];
    abstract dropProcedureComment(name: XObjectName): ExecutableStatement | ExecutableStatement[];
    abstract dropFunctionComment(name: XObjectName): ExecutableStatement | ExecutableStatement[];
    abstract dropTableComment(name: XObjectName): ExecutableStatement | ExecutableStatement[];
    abstract dropColumnComment(table: XObjectName, name: string): ExecutableStatement | ExecutableStatement[];
    abstract dropIndexComment(table: XObjectName, name: string): ExecutableStatement | ExecutableStatement[];
    abstract dropConstraintComment(table: XObjectName, name: string): ExecutableStatement | ExecutableStatement[];
    /**
     * 将列修改为自动行标识列
     */
    abstract setAutoRowflag(table: XObjectName, column: string): ExecutableStatement | ExecutableStatement[];
    /**
     * 删除列行标识属性
     */
    abstract dropAutoRowflag(table: XObjectName, column: string): ExecutableStatement | ExecutableStatement[];
    abstract setDefaultValue(table: XObjectName, column: string, defaultValue: XExpression): ExecutableStatement | ExecutableStatement[];
    abstract dropDefaultValue(table: XObjectName, column: string): ExecutableStatement | ExecutableStatement[];
    abstract setIdentity(table: XObjectName, column: string, startValue: number, increment: number): ExecutableStatement | ExecutableStatement[];
    abstract dropIdentity(table: XObjectName, column: string): ExecutableStatement | ExecutableStatement[];
}
