"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrateCli = exports.outputCommand = exports.LUBE_MIGRATE_TABLE_NAME = void 0;
require("colors");
const fs_1 = require("fs");
const path_1 = require("path");
const promises_1 = require("fs/promises");
const md5_1 = __importDefault(require("crypto-js/md5"));
const orm_1 = require("../orm");
const migrate_snapshot_1 = require("./migrate-snapshot");
const core_1 = require("../core");
const sql_migrate_scripter_1 = require("./sql-migrate-scripter");
const code_migrate_scripter_1 = require("./code-migrate-scripter");
require("./types");
const { readdir, stat, writeFile } = fs_1.promises;
exports.LUBE_MIGRATE_TABLE_NAME = '__LubeMigrate';
const MIGRATE_FILE_REGX = /^(\d{14})_(\w[\w_\d]*)\.(ts|js)$/i;
const NAME_REGX = /^[a-zA-Z_]\w*$/i;
function createMigrateBuilder(builder, statements) {
    return new Proxy(builder, {
        get(target, key) {
            const val = Reflect.get(target, key);
            if (typeof val === 'function') {
                return function (...args) {
                    const ret = val.call(target, ...args);
                    if (ret instanceof Promise) {
                        return ret.then(ret => {
                            if (core_1.Statement.isStatement(ret)) {
                                statements.push(ret);
                            }
                            return ret;
                        });
                    }
                    else if (core_1.Statement.isStatement(ret) || core_1.Raw.isRaw(ret)) {
                        statements.push(ret);
                    }
                    return ret;
                };
            }
            return val;
        },
    });
}
function assertName(name) {
    if (!NAME_REGX.test(name)) {
        throw new Error(`Migrate class name '${name}' invalid.`);
    }
}
function outputCommand(cmd, writer) {
    writer.write(`[${new Date().toISOString()}]: execute command\n`);
    writer.write('sql: ' + cmd.sql.gray + '\n');
    if (cmd.params && cmd.params.length > 0) {
        writer.write('params: {\n' +
            cmd.params
                .map(p => `${p.name}: ${JSON.stringify(p.value)}`)
                .join(',\n') +
            '\n}\n');
    }
}
exports.outputCommand = outputCommand;
/**
 * 迁移命令行
 */
class MigrateCliClass {
    constructor(context, migrateDir = './migrates') {
        this.migrateDir = migrateDir;
        return (async () => {
            await this.init(context);
            return this;
        })();
    }
    async init(context) {
        let Ctr;
        let metadata;
        if (!context || typeof context === 'string') {
            if (context) {
                metadata = orm_1.metadataStore.getContext(context);
                Ctr = metadata.class;
            }
            else {
                metadata = orm_1.metadataStore.defaultContext;
                Ctr = metadata.class;
            }
            const config = await (0, core_1.loadConfig)();
            if (!config.connections[Ctr.name]) {
                throw new Error(`DbContext ${context}, connection config not found.`);
            }
            const options = Object.assign({}, config.connections[Ctr.name]);
            // 不直接进入指定数据库，而是进入系统数据库
            if (options.database) {
                this.targetDatabase = options.database;
                options.database = undefined;
            }
            this.connection = await (0, core_1.createConnection)(options);
            await this.connection.open();
            this.dbContext = await (0, orm_1.createContext)(Ctr, this.connection);
        }
        else {
            this.dbContext = context;
            this.connection = context.connection;
        }
        if (!this.targetDatabase) {
            this.targetDatabase = this.dbContext.metadata.database;
        }
        // // 进入后的数据库名称
        // this.initDatabase = await this.connection.getDatabaseName();
        // if (this.initDatabase === this.targetDatabase) {
        //   await this.connection.close();
        //   throw new Error('Migrate must run with super administrator user.');
        // }
        this.provider = (0, core_1.getProvider)(this.connection.options.dialect);
        this.migrateBuilder = this.provider.getMigrateBuilder();
        this.schemaLoader = this.provider.getSchemaLoader(this.dbContext.connection);
        this._targetSchemaName = await this.connection.getSchemaName();
    }
    // /**
    //  * 确保在目标数据库中运行
    //  */
    // private async runInTargetDatabase<T>(handler: () => Promise<T>): Promise<T> {
    //   const currentDb = await this.connection.getDatabaseName();
    //   if (currentDb !== this.targetDatabase) {
    //     await this.connection.changeDatabase(this.targetDatabase);
    //     try {
    //       const result = await handler();
    //       await this.connection.changeDatabase(currentDb);
    //       return result;
    //     } finally {
    //       await this.connection.changeDatabase(currentDb);
    //     }
    //   }
    //   return handler();
    // }
    async runMigrate(Ctr, action, builder) {
        const instance = new Ctr();
        const statements = [];
        const scripter = createMigrateBuilder(builder, statements);
        if (action === 'up') {
            await instance.up(scripter, this.connection.options.provider.dialect);
        }
        else {
            await instance.down(scripter, this.connection.options.provider.dialect);
        }
        return statements;
    }
    // todo 待实现
    async ensureSchema() {
        throw new Error('尚未实现');
    }
    /**
     * 数据库是否存在
     * @returns
     */
    async existsTargetDatabase() {
        const result = await this.connection.queryScalar(core_1.SQL.select(core_1.SQL.literal(true).to(core_1.DbType.boolean)).where(core_1.SQL.existsDatabase(this.targetDatabase)));
        return result === true;
    }
    async _script(start, end) {
        const startIndex = start?.index ?? -1;
        const endIndex = end?.index ?? -1;
        const migrates = await this._list();
        await this.snapshotAll();
        if (startIndex === endIndex) {
            console.log(`源和目标版本一致或未找到可供生成的的迁移文件。`.yellow);
            return [];
        }
        const isUpgrade = endIndex > startIndex;
        const isDemotion = endIndex < startIndex;
        const statements = [];
        if (isUpgrade) {
            for (let i = startIndex + 1; i <= endIndex; i++) {
                const info = migrates[i];
                statements.push(core_1.SQL.note(`===============Migrate up script from "${info.path}"====================`));
                const Migrate = await importMigrate(info);
                const codes = await this.runMigrate(Migrate, 'up', this.migrateBuilder);
                statements.push(...codes);
                // 创建迁移记录表
                if (i === 0) {
                    statements.push(core_1.SQL.createTable(exports.LUBE_MIGRATE_TABLE_NAME).as(builder => [
                        builder.column('migrate_id', core_1.DbType.string(100)).primaryKey(),
                        builder.column('hash', core_1.DbType.string(32)).notNull(),
                        builder.column('date', core_1.DbType.datetime).default(core_1.SQL.now()),
                    ]));
                }
                statements.push(core_1.SQL.insert(exports.LUBE_MIGRATE_TABLE_NAME).values({
                    migrate_id: info.id,
                    hash: info.hash,
                }));
            }
        }
        if (isDemotion) {
            for (let i = startIndex; i > endIndex; i--) {
                const info = migrates[i];
                const t = core_1.SQL.table(exports.LUBE_MIGRATE_TABLE_NAME).as('t');
                statements.push(core_1.SQL.note(`Migrate down script from "${info.path}"`));
                // statements.push(SQL.note('Check migrate file change'));
                // statements.push(
                //   SQL.if(
                //     SQL.not(
                //       SQL.exists(
                //         SQL.select(1)
                //           .from(t)
                //           .where(t.migrate_id.eq(info.id).and(t.hash.eq(info.hash!)))
                //       )
                //     )
                //   ).then(
                //     this.migrateBuilder.throw(
                //       `No migrate in database or migrate file is changed after migration ${info.id} up. For data security, please handle this migration manually`
                //     )
                //   )
                // );
                const Migrate = await importMigrate(info);
                const codes = await this.runMigrate(Migrate, 'down', this.migrateBuilder);
                statements.push(...codes);
                statements.push(core_1.SQL.delete(t).where(t.migrate_id.eq(info.id).and(t.hash.eq(info.hash))));
                if (i === 0) {
                    statements.push(core_1.SQL.dropTable(exports.LUBE_MIGRATE_TABLE_NAME));
                }
            }
        }
        const scripts = statements.map(statement => this.connection.sqlifier.sqlify(statement));
        return scripts;
    }
    async dispose() {
        if (this.connection.opened) {
            await this.connection.close();
        }
    }
    /**
     * 获取数据库当前迁移版本
     * @returns
     */
    async getDbMigrate() {
        if (!(await this.existsTargetDatabase()))
            return undefined;
        await this.connection.changeDatabase(this.targetDatabase);
        try {
            const [item] = await this.connection.select(exports.LUBE_MIGRATE_TABLE_NAME, {
                offset: 0,
                limit: 1,
                sorts: t => [t.$field('migrate_id').desc()],
            });
            if (!item)
                return;
            const id = item.migrate_id;
            return this.getMigrate(id);
        }
        catch {
            return undefined;
        }
    }
    async getLastMigrate() {
        const items = await this._list();
        return items[items.length - 1];
    }
    /**
     * 创建数据库
     */
    async ensureTargetDatabase() {
        await this.connection.query(core_1.SQL.createDatabase.ifNotExists(this.targetDatabase));
        await this.connection.changeDatabase(this.targetDatabase);
    }
    /**
     * 如果数据库已经存在，删除目标数据库
     */
    async dropTargetDatabase() {
        try {
            await this.connection.changeToSysdb();
            await this.connection.query(core_1.SQL.dropDatabase.ifExists(this.targetDatabase));
        }
        catch (error) {
            const e = new Error(`DropDatabase error:` + error.toString());
            throw e;
        }
    }
    // private async ensureMigrateTable(): Promise<void> {
    //   if (!this.dbSchema.tables.find(t => t.name === LUBE_MIGRATE_TABLE_NAME)) {
    //     const sql = SQL.createTable(LUBE_MIGRATE_TABLE_NAME).as(builder => [
    //       builder.column('migrate_id', DbType.string(100)).primaryKey(),
    //     ]);
    //     await this.dbContext.executor.query(sql);
    //   }
    // }
    getTimestamp() {
        const now = new Date();
        const timestamp = now.getFullYear().toString().padStart(4, '0') +
            (now.getMonth() + 1).toString().padStart(2, '0') +
            now.getDate().toString().padStart(2, '0') +
            now.getHours().toString().padStart(2, '0') +
            now.getMinutes().toString().padStart(2, '0') +
            now.getSeconds().toString().padStart(2, '0');
        return timestamp;
    }
    // /**
    //  * 创建一个空的迁移文件
    //  * @param name
    //  */
    // async create(name?: string): Promise<void> {
    //   const timestamp = this.getTimestamp();
    //   if (!name) {
    //     name = 'Migrate' + timestamp;
    //   }
    //   const exists = await this.findMigrate(name);
    //   if (exists) {
    //     throw new Error(`迁移文件${name}已经存在：${exists.path}`);
    //   }
    //   const id = `${timestamp}_${name}`;
    //   const codes = this.generateMigrateClass(name);
    //   if (!existsSync(this.migrateDir)) {
    //     await promises.mkdir(this.migrateDir);
    //   }
    //   const filePath = join(this.migrateDir, `${id}.ts`);
    //   await writeFile(filePath, codes);
    //   console.info(`迁移文件创建成功：${filePath}`);
    // }
    /**
     * 将数据库架构与当前架构进行对比，并生成新的迁移文件
     * @param name
     */
    async add(name, notResolverType = false) {
        // 文件夹不存在则创建
        if (!(0, fs_1.existsSync)(this.migrateDir)) {
            (0, fs_1.mkdirSync)(this.migrateDir);
        }
        const timestamp = this.getTimestamp();
        if (!name) {
            name = 'Migrate' + timestamp;
        }
        assertName(name);
        const exists = await this.findMigrate(name);
        if (exists) {
            throw new Error(`迁移文件${name}已经存在：${exists.path}`);
        }
        const lastMigrate = await this.getLastMigrate();
        let lastSchema;
        if (lastMigrate) {
            lastSchema = (await Promise.resolve().then(() => __importStar(require(lastMigrate.snapshotPath)))).default;
        }
        // const dbSchema = await this.loadSchema();
        const entityScheam = (0, orm_1.generateSchema)(this.dbContext);
        const code = await this.generateMigrate(name, entityScheam, lastSchema, notResolverType);
        const id = `${timestamp}_${name}`;
        const filePath = (0, path_1.resolve)((0, path_1.join)(this.migrateDir, `${id}.ts`));
        await writeFile(filePath, code);
        console.log(`Generate migrate file successed, and output to file ${filePath}`.green);
        const snapshotPath = (0, path_1.resolve)((0, path_1.join)(this.migrateDir, `${id}.snapshot.ts`));
        //     // 创建快照文件
        //     await writeFile(
        //       snapshotPath,
        //       `
        // import { DatabaseSchema } from 'lubejs';
        // export const schema: DatabaseSchema = ${JSON.stringify(entityScheam)};
        // export default schema;
        // `
        //     );
        const info = {
            id,
            name,
            timestamp,
            path: filePath,
            snapshotPath,
            kind: 'typescript',
            index: (await this._list()).length,
        };
        (await this._list()).push(info);
        await this.snapshotAll();
        return info;
    }
    async findMigrate(name) {
        const items = await this._list();
        const item = items.find(item => item.name === name || item.id === name || item.timestamp === name);
        return item;
    }
    async getMigrate(nameOrId) {
        const items = await this._list();
        const item = items.find(item => item.name === nameOrId ||
            item.id === nameOrId ||
            item.timestamp === nameOrId);
        if (!item) {
            throw new Error(`找不到指定的迁移${nameOrId}文件，或迁移文件已被删除。`);
        }
        return item;
    }
    /**
     * 获取数据库架构
     * @returns
     */
    getDbSchema() {
        return this.schemaLoader.getDatabaseSchema(this.targetDatabase);
    }
    /**
     * 获取实体的架构
     * @returns
     */
    getEntitySchema() {
        return (0, orm_1.generateSchema)(this.dbContext);
    }
    get targetSchemaName() {
        return this._targetSchemaName;
    }
    // async useTargetDatabase() {
    //   const dbContext = await this.getDbContext();
    //   const database =
    //     dbContext.lube.provider.options.database || dbContext.metadata.database;
    //   if (dbContext.lube.opened)
    //     await dbContext.lube.close();
    //   dbContext.lube.changeDatabase(database);
    // }
    // async useDefaultDatabase() {
    //   const dbContext = await this.getDbContext();
    //   const database =
    //     dbContext.lube.provider.options.database || dbContext.metadata.database;
    //   if (dbContext.lube.opened) {
    //     await dbContext.lube.close();
    //     dbContext.lube.changeDatabase(null);
    //     // WARN: 当心：此处可能有坑，当权限足够，但是默认数据为当前数据库的用户使用时，将会招抛出此异常
    //     if (await dbContext.lube.getCurrentDatabase() === database) {
    //       throw new Error(`Permission denied, you must provide a server administrator user to migrate.`);
    //     }
    //   }
    // }
    /**
     * 更新到指定迁移
     * 无论是否比指定迁移更新
     * @param name
     */
    async update(name, output) {
        const target = name
            ? await this.getMigrate(name)
            : await this.getLastMigrate();
        if (!target) {
            throw new Error(`无法更新数据库，因为继续更新将删除数据库。`);
        }
        // const exists = await this.existsDatabase();
        const current = await this.getDbMigrate();
        if (!current && (await this.existsTargetDatabase())) {
            throw new Error(`数据库 ${this.targetDatabase} 已经存在，并且不受迁移工具管理，无法继续操作。`);
        }
        const scripts = await this._script(current, target);
        await this.ensureTargetDatabase();
        for (const cmd of scripts) {
            if (output) {
                outputCommand(cmd, output);
            }
            await this.connection.query(cmd);
            if (output) {
                console.info(`----------------------------------------------------`);
            }
        }
        console.info(`------------执行成功，已更新到${target.id}.----------------`);
    }
    async script(options) {
        let end;
        if (!options?.end || options?.end === '*') {
            end = await this.getLastMigrate();
        }
        else if (options?.end === '@') {
            end = await this.getDbMigrate();
        }
        else if (options.end !== '?') {
            end = await this.getMigrate(options.end);
        }
        // const migrates = await this._list();
        let start;
        if (!options?.start || options?.start === '@') {
            start = await this.getDbMigrate();
        }
        else if (options?.start === '*') {
            start = await this.getLastMigrate();
        }
        else if (options.start !== '?') {
            start = await this.getMigrate(options.start);
        }
        const scripts = await this._script(start, end);
        const text = this.connection.sqlifier.joinBatchSql(...scripts.map(cmd => cmd.sql));
        if (options?.outputPath) {
            const filePath = (0, path_1.resolve)(options.outputPath);
            const dir = (0, path_1.dirname)(filePath);
            if (!(0, fs_1.existsSync)(dir)) {
                await (0, promises_1.mkdir)(dir);
            }
            await writeFile(options.outputPath, text, 'utf-8');
            console.log(`Generate scripts successed, and output to file ${filePath}`.green);
        }
        else {
            console.log(text);
        }
        return text;
    }
    /**
     * 列出当前所有迁移
     */
    async _list() {
        if (this._migrateList)
            return this._migrateList;
        const results = [];
        if (!(0, fs_1.existsSync)(this.migrateDir)) {
            return [];
        }
        const items = await readdir(this.migrateDir);
        for (const item of items.sort()) {
            const path = (0, path_1.join)(this.migrateDir, item);
            const match = MIGRATE_FILE_REGX.exec(item);
            if (match && (await stat(path)).isFile()) {
                const fullPath = (0, path_1.resolve)(process.cwd(), path);
                const extname = match[3].toLowerCase();
                const snapshotPath = (0, path_1.resolve)((0, path_1.join)(this.migrateDir, `${match[1]}_${match[2]}.${this.connection.options.dialect}.snapshot.${extname}`));
                results.push({
                    id: `${match[1]}_${match[2]}`,
                    timestamp: match[1],
                    name: match[2],
                    path: fullPath,
                    snapshotPath,
                    kind: extname === 'js' ? 'javascript' : 'typescript',
                    // hash: (await import(snapshotPath)).hash,
                    index: 0,
                });
            }
        }
        this._migrateList = results.sort((a, b) => a.timestamp.localeCompare(b.timestamp));
        this._migrateList.forEach((item, index) => (item.index = index));
        return this._migrateList;
    }
    /**
     * 列出迁移文件
     */
    async list() {
        const list = await this._list();
        console.table(list, ['kind', 'name', 'timestamp', 'path']);
    }
    // async getDefaultSchema(): Promise<string> {
    //   await this.ensureDatabase();
    //   const defaultSchema = await this.runInTargetDatabase(async () => {
    //     return await this.connection.getDefaultSchema();
    //   });
    //   return defaultSchema;
    // }
    /**
     * 生成迁移文件代码
     * @param name
     * @param source
     * @param target
     * @param metadata
     * @returns
     */
    async generateMigrate(name, source, target, notResolverType) {
        const scripter = new code_migrate_scripter_1.CodeMigrateScripter(this.connection.provider.getSchemaComparator(), this.connection.sqlifier, notResolverType);
        const defaultSchema = await this.connection.getSchemaName();
        scripter.migrate(defaultSchema, source, target);
        const upCodes = scripter.getScripts();
        scripter.clear();
        scripter.migrate(defaultSchema, target, source);
        const downCodes = scripter.getScripts();
        return this.generateMigrateClass(name, upCodes, downCodes);
    }
    generateMigrateClass(name, upcodes, downcodes) {
        return `import { Migrate, SQL, DbType, MigrateBuilder } from 'lubejs';

export class ${name} implements Migrate {

  async up(
    builder: MigrateBuilder,
    dialect: string
  ): Promise<void> {
    ${(upcodes && upcodes.map(line => line + ';').join('\n    ')) || ''}
  }

  async down(
    builder: MigrateBuilder,
    dialect: string
  ): Promise<void> {
    ${(downcodes && downcodes.map(line => line + ';').join('\n    ')) || ''}
  }

}

export default ${name};
`;
    }
    /**
     * 同步架构
     */
    async sync(output) {
        const metadataSchema = (0, orm_1.generateSchema)(this.dbContext);
        const defaultSchema = await this.connection.getSchemaName();
        await this.ensureTargetDatabase();
        // 需要操作的数据库名称, 优先取连接字符串中的数据库名称，Metadata中的数据名称次之。
        const dbSchema = await this.schemaLoader.getDatabaseSchema(this.targetDatabase);
        const scripter = new sql_migrate_scripter_1.SqlMigrateScripter(this.provider.getSchemaComparator(), this.provider.getMigrateBuilder());
        scripter.migrate(defaultSchema, metadataSchema, dbSchema);
        const statements = scripter.getScripts();
        if (output) {
            output.write(`*************************************************\n`);
            output.write(`开始更新数据库架构，请稍候......\n`);
        }
        for (const statement of statements) {
            const cmd = this.connection.sqlifier.sqlify(statement);
            if (output) {
                outputCommand(cmd, output);
                output.write('----------------------------------------------------\n');
            }
            await this.connection.query(cmd);
        }
        if (output) {
            output.write(`更新数据库架构完成，数据库架构已经更新到与实体一致。\n`);
            output.write(`请注意，通过Sync更新的数据库，将不能再使用 'lubejs migrate update <migrate>' 命令更新，否则可能造成数据丢失！\n`);
        }
    }
    /**
     * 生成快照
     */
    async snapshotAll() {
        let lastSchema = undefined;
        const list = await this._list();
        let needRetrace = false;
        let currentHash = '';
        for (const item of list) {
            if (!currentHash) {
                currentHash = await getFileHash(item.path);
            }
            else {
                currentHash = (0, md5_1.default)(currentHash + (await (0, promises_1.readFile)(item.path)).toString('utf-8')).toString();
            }
            // 填充哈希码
            item.hash = currentHash;
            if (!needRetrace) {
                if (!(0, fs_1.existsSync)(item.snapshotPath)) {
                    needRetrace = true;
                }
            }
            if (!needRetrace) {
                // 较准哈希码
                const { hash, default: snapshot } = await Promise.resolve().then(() => __importStar(require(item.snapshotPath)));
                if (!currentHash !== hash) {
                    needRetrace = true;
                }
                else {
                    lastSchema = snapshot;
                    continue;
                }
            }
            if (needRetrace) {
                if (!lastSchema) {
                    lastSchema = {
                        name: this.connection.options.database ||
                            this.dbContext.metadata.database,
                        tables: [],
                        schemas: [],
                        views: [],
                        sequences: [],
                        procedures: [],
                        functions: [],
                        comment: this.dbContext.metadata.comment,
                    };
                }
                const tracker = new migrate_snapshot_1.SnapshotMigrateTracker(lastSchema, this.connection.sqlifier, this.targetSchemaName);
                const builder = new migrate_snapshot_1.SnapshotMigrateBuilder();
                const migrate = (await Promise.resolve().then(() => __importStar(require(item.path)))).default;
                const statements = await this.runMigrate(migrate, 'up', builder);
                tracker.run(statements);
                lastSchema = tracker.database;
                // 生成快照文件
                await writeFile(item.snapshotPath, `
import { DatabaseSchema } from "lubejs";
export const schema: DatabaseSchema = ${JSON.stringify(lastSchema, undefined, 2)};
export const dialect = '${this.connection.options.provider.dialect}';
export const hash = "${currentHash}";
export default schema;
`);
            }
        }
    }
}
exports.MigrateCli = MigrateCliClass;
async function importMigrate(info) {
    let imported;
    if (info.kind === 'typescript') {
        imported = await Promise.resolve().then(() => __importStar(require(info.path)));
    }
    else {
        imported = require(info.path);
    }
    const migrate = imported?.default || imported?.[info.name] || imported;
    if (!migrate)
        throw new Error(`Migrate file ${info.path} not found.`);
    return migrate;
}
async function getFileHash(path) {
    const data = await (await (0, promises_1.readFile)(path)).toString('utf-8');
    return (0, md5_1.default)(data).toString();
}
