import { MigrateBuilder } from './migrate-builder';
import { DatabaseSchema } from '../orm';
import { XObjectName, XExpression, Scalar, Statement, Sqlifier } from '../core';
import { ExecutableStatement } from '../core/sql';
export declare class SnapshotMigrateBuilder extends MigrateBuilder {
    constructor();
    readonly statements: Statement[];
    renameSequence(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    dropSchemaComment(name: string): ExecutableStatement | ExecutableStatement[];
    dropSequenceComment(name: XObjectName<string>): ExecutableStatement | ExecutableStatement[];
    dropProcedureComment(name: XObjectName<string>): ExecutableStatement | ExecutableStatement[];
    dropFunctionComment(name: XObjectName<string>): ExecutableStatement | ExecutableStatement[];
    dropTableComment(name: XObjectName<string>): ExecutableStatement | ExecutableStatement[];
    dropColumnComment(table: XObjectName<string>, name: string): ExecutableStatement | ExecutableStatement[];
    dropIndexComment(table: XObjectName<string>, name: string): ExecutableStatement | ExecutableStatement[];
    dropConstraintComment(table: XObjectName<string>, name: string): ExecutableStatement | ExecutableStatement[];
    setAutoRowflag(table: XObjectName<string>, column: string): ExecutableStatement | ExecutableStatement[];
    dropAutoRowflag(table: XObjectName<string>, column: string): ExecutableStatement | ExecutableStatement[];
    setDefaultValue(table: XObjectName, column: string, defaultValue: XExpression<Scalar>): ExecutableStatement | ExecutableStatement[];
    dropDefaultValue(table: XObjectName, column: string): ExecutableStatement | ExecutableStatement[];
    setIdentity(table: XObjectName, column: string, startValue: number, increment: number): ExecutableStatement | ExecutableStatement[];
    dropIdentity(table: XObjectName<string>, column: string): ExecutableStatement | ExecutableStatement[];
    setProcedureComment(name: XObjectName, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setFunctionComment(name: XObjectName, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setSequenceComment(name: XObjectName, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setTableComment(name: XObjectName, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setViewComment(name: XObjectName, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setColumnComment(table: XObjectName, name: string, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setIndexComment(table: XObjectName, name: string, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setConstraintComment(table: XObjectName<string>, name: string, comment: string | null): ExecutableStatement | ExecutableStatement[];
    setSchemaComment(name: string, comment: string | null): ExecutableStatement | ExecutableStatement[];
    renameTable(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    renameColumn(table: XObjectName, name: string, newName: string): ExecutableStatement | ExecutableStatement[];
    renameView(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    renameIndex(table: XObjectName, name: string, newName: string): ExecutableStatement | ExecutableStatement[];
    renameProcedure(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
    renameFunction(name: XObjectName, newName: string): ExecutableStatement | ExecutableStatement[];
}
export declare class SnapshotMigrateTracker {
    database: DatabaseSchema | undefined;
    private sqlifier;
    private defaultSchema;
    constructor(database: DatabaseSchema | undefined, sqlifier: Sqlifier, defaultSchema: string);
    run(statements: Statement[]): void;
    assertDatabaseExists(schema: DatabaseSchema | undefined): asserts schema;
    private getTable;
    private findTable;
    private findView;
    private assertTableNotExists;
    private assertViewNotExists;
    private assertProcedureNotExists;
    private assertFunctionNotExists;
    private assertSequenceNotExists;
    private assertSchemaNotExists;
    private assertColumnNotExists;
    private assertIndexNotExists;
    private assertPrimaryNotExists;
    private assertForeignKeyExists;
    private assertConstraintNotExists;
    private addTableMembers;
    private createTable;
    private alterTable;
    private getSequence;
    private getProcedure;
    private getFunction;
    private getView;
    private getColumn;
    private getIndex;
    private getPrimaryKey;
    private getForeignKey;
    private getConstraint;
}
