"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapshotMigrateTracker = exports.SnapshotMigrateBuilder = void 0;
const migrate_builder_1 = require("./migrate-builder");
const util_1 = require("./util");
const core_1 = require("../core");
class SnapshotMigrateBuilder extends migrate_builder_1.MigrateBuilder {
    // existsTable(table: CompatiableObjectName<string>): Condition {
    //   throw new Error('Method not implemented.')
    // }
    // existsDatabase(database: string): Condition {
    //   throw new Error('Method not implemented.')
    // }
    // existsView(name: string): Condition {
    //   throw new Error('Method not implemented.')
    // }
    // existsFunction(name: CompatiableObjectName<string>): Condition {
    //   throw new Error('Method not implemented.')
    // }
    // existsProcedure(name: CompatiableObjectName<string>): Condition {
    //   throw new Error('Method not implemented.')
    // }
    // existsSequence(name: CompatiableObjectName<string>): Condition {
    //   throw new Error('Method not implemented.')
    // }
    constructor() {
        super();
        this.statements = [];
    }
    // throw(msg: string): ExecutableStatement {
    //   return new VirtualStatement(this.throw.name, [msg]);
    // }
    renameSequence(name, newName) {
        return new core_1.VirtualStatement(this.renameSequence.name, [name, newName]);
    }
    // renameDatabase(
    //   name: string,
    //   newName: string
    // ): ExecutableStatement | ExecutableStatement[] {
    //   return new VirtualStatement(this.renameDatabase.name, [name, newName]);
    // }
    dropSchemaComment(name) {
        return new core_1.VirtualStatement(this.dropSchemaComment.name, [name]);
    }
    dropSequenceComment(name) {
        return new core_1.VirtualStatement(this.dropSequenceComment.name, [name]);
    }
    dropProcedureComment(name) {
        return new core_1.VirtualStatement(this.dropProcedureComment.name, [name]);
    }
    dropFunctionComment(name) {
        return new core_1.VirtualStatement(this.dropFunctionComment.name, [name]);
    }
    dropTableComment(name) {
        return new core_1.VirtualStatement(this.dropTableComment.name, [name]);
    }
    dropColumnComment(table, name) {
        return new core_1.VirtualStatement(this.dropColumnComment.name, [table, name]);
    }
    dropIndexComment(table, name) {
        return new core_1.VirtualStatement(this.dropIndexComment.name, [table, name]);
    }
    dropConstraintComment(table, name) {
        return new core_1.VirtualStatement(this.dropConstraintComment.name, [table, name]);
    }
    setAutoRowflag(table, column) {
        return new core_1.VirtualStatement(this.setAutoRowflag.name, [table, column]);
    }
    dropAutoRowflag(table, column) {
        return new core_1.VirtualStatement(this.dropAutoRowflag.name, [table, column]);
    }
    setDefaultValue(table, column, defaultValue) {
        return new core_1.VirtualStatement(this.setDefaultValue.name, [
            table,
            column,
            core_1.Expression.ensureExpression(defaultValue),
        ]);
    }
    dropDefaultValue(table, column) {
        return new core_1.VirtualStatement(this.dropDefaultValue.name, [table, column]);
    }
    setIdentity(table, column, startValue, increment) {
        return new core_1.VirtualStatement(this.setIdentity.name, [
            table,
            column,
            startValue,
            increment,
        ]);
    }
    dropIdentity(table, column) {
        return new core_1.VirtualStatement(this.dropIdentity.name, [table, column]);
    }
    setProcedureComment(name, comment) {
        return new core_1.VirtualStatement(this.setProcedureComment.name, [name, comment]);
    }
    setFunctionComment(name, comment) {
        return new core_1.VirtualStatement(this.setFunctionComment.name, [name, comment]);
    }
    setSequenceComment(name, comment) {
        return new core_1.VirtualStatement(this.setSequenceComment.name, [name, comment]);
    }
    setTableComment(name, comment) {
        return new core_1.VirtualStatement(this.setTableComment.name, [name, comment]);
    }
    setViewComment(name, comment) {
        return new core_1.VirtualStatement(this.setViewComment.name, [name, comment]);
    }
    setColumnComment(table, name, comment) {
        return new core_1.VirtualStatement(this.setColumnComment.name, [
            table,
            name,
            comment,
        ]);
    }
    setIndexComment(table, name, comment) {
        return new core_1.VirtualStatement(this.setIndexComment.name, [
            table,
            name,
            comment,
        ]);
    }
    setConstraintComment(table, name, comment) {
        return new core_1.VirtualStatement(this.setConstraintComment.name, [
            table,
            name,
            comment,
        ]);
    }
    setSchemaComment(name, comment) {
        return new core_1.VirtualStatement(this.setSchemaComment.name, [name, comment]);
    }
    renameTable(name, newName) {
        return new core_1.VirtualStatement(this.renameTable.name, [name, newName]);
    }
    renameColumn(table, name, newName) {
        return new core_1.VirtualStatement(this.renameColumn.name, [table, name, newName]);
    }
    renameView(name, newName) {
        return new core_1.VirtualStatement(this.renameView.name, [name, newName]);
    }
    renameIndex(table, name, newName) {
        return new core_1.VirtualStatement(this.renameIndex.name, [table, name, newName]);
    }
    renameProcedure(name, newName) {
        return new core_1.VirtualStatement(this.renameProcedure.name, [name, newName]);
    }
    renameFunction(name, newName) {
        return new core_1.VirtualStatement(this.renameFunction.name, [name, newName]);
    }
}
exports.SnapshotMigrateBuilder = SnapshotMigrateBuilder;
class SnapshotMigrateTracker {
    constructor(database, sqlifier, defaultSchema) {
        this.database = database;
        this.sqlifier = sqlifier;
        this.defaultSchema = defaultSchema;
    }
    run(statements) {
        for (const statement of statements) {
            if (statement.$kind === core_1.STATEMENT_KIND.CREATE_DATABASE) {
                if (this.database) {
                    throw new Error('Database is exists.');
                }
            }
            else {
                this.assertDatabaseExists(this.database);
            }
            switch (statement.$kind) {
                case core_1.STATEMENT_KIND.CREATE_DATABASE:
                    this.database = {
                        name: statement.$name,
                        collate: statement.$collate,
                        tables: [],
                        views: [],
                        procedures: [],
                        functions: [],
                        sequences: [],
                        schemas: [],
                    };
                    break;
                case core_1.STATEMENT_KIND.ALTER_DATABASE:
                    this.assertDatabaseExists(this.database);
                    this.database.collate = statement.$collate;
                    break;
                case core_1.STATEMENT_KIND.DROP_DATABASE:
                    this.database = undefined;
                    break;
                case core_1.STATEMENT_KIND.CREATE_TABLE:
                    this.createTable(statement);
                    break;
                case core_1.STATEMENT_KIND.ALTER_TABLE:
                    this.alterTable(statement);
                    break;
                case core_1.STATEMENT_KIND.DROP_TABLE: {
                    const index = this.database?.tables.findIndex(p => (0, util_1.isSameObject)(p.name, statement.$name));
                    if (index === undefined || index < 0) {
                        throw new Error(`Table ${this.sqlifier.sqlifyObjectName(statement.$name)} not found.`);
                    }
                    this.database.tables.splice(index, 1);
                    break;
                }
                case core_1.STATEMENT_KIND.CREATE_VIEW: {
                    this.assertViewNotExists(statement.$name);
                    (0, util_1.assertAst)(statement.$body, 'CreateView body not found.');
                    const objName = this.sqlifier.parseObjectName(statement.$name);
                    this.database.views.push({
                        name: objName.name,
                        schema: objName.schema ?? this.defaultSchema,
                        scripts: this.sqlifier.sqlify(statement.$body).sql,
                    });
                    break;
                }
                case core_1.STATEMENT_KIND.ALTER_VIEW: {
                    const view = this.getView(statement.$name);
                    (0, util_1.assertAst)(statement.$body, 'AlterView body not found.');
                    view.scripts = this.sqlifier.sqlify(statement.$body).sql;
                    break;
                }
                case core_1.STATEMENT_KIND.DROP_VIEW: {
                    const index = this.database?.views.findIndex(p => (0, util_1.isSameObject)(p.name, statement.$name));
                    if (index === undefined || index < 0) {
                        throw new Error(`View ${this.sqlifier.sqlifyObjectName(statement.$name)} not found.`);
                    }
                    this.database.views.splice(index, 1);
                    break;
                }
                case core_1.STATEMENT_KIND.CREATE_PROCEDURE: {
                    this.assertProcedureNotExists(statement.$name);
                    const objName = this.sqlifier.parseObjectName(statement.$name);
                    this.database.procedures.push({
                        name: objName.name,
                        schema: objName.schema ?? this.defaultSchema,
                        scripts: this.sqlifier.sqlify(statement).sql,
                    });
                    break;
                }
                case core_1.STATEMENT_KIND.ALTER_PROCEDURE: {
                    const procedure = this.getProcedure(statement.$name);
                    procedure.scripts = this.sqlifier.sqlify(statement).sql;
                    break;
                }
                case core_1.STATEMENT_KIND.DROP_PROCEDURE: {
                    const index = this.database?.procedures.findIndex(p => (0, util_1.isSameObject)(p.name, statement.$name));
                    if (index === undefined || index < 0) {
                        throw new Error(`Procedure ${this.sqlifier.sqlifyObjectName(statement.$name)} not found.`);
                    }
                    this.database.procedures.splice(index, 1);
                    break;
                }
                case core_1.STATEMENT_KIND.CREATE_FUNCTION: {
                    this.assertFunctionNotExists(statement.$name);
                    const objName = this.sqlifier.parseObjectName(statement.$name);
                    this.database.functions.push({
                        name: objName.name,
                        schema: objName.schema ?? this.defaultSchema,
                        scripts: this.sqlifier.sqlify(statement).sql,
                    });
                    break;
                }
                case core_1.STATEMENT_KIND.ALTER_FUNCTION: {
                    const fn = this.getFunction(statement.$name);
                    fn.scripts = this.sqlifier.sqlify(statement).sql;
                    break;
                }
                case core_1.STATEMENT_KIND.DROP_FUNCTION: {
                    const index = this.database?.functions.findIndex(p => (0, util_1.isSameObject)(p.name, statement.$name));
                    if (index === undefined || index < 0) {
                        throw new Error(`Function ${this.sqlifier.sqlifyObjectName(statement.$name)} not found.`);
                    }
                    this.database.functions.splice(index, 1);
                    break;
                }
                case core_1.STATEMENT_KIND.CREATE_SEQUENCE: {
                    this.assertSequenceNotExists(statement.$name);
                    (0, util_1.assertAst)(statement.$dbType, 'CreateSequence dbtype not found.');
                    const objName = this.sqlifier.parseObjectName(statement.$name);
                    this.database.sequences.push({
                        name: objName.name,
                        schema: objName.schema ?? this.defaultSchema,
                        type: this.sqlifier.sqlifyType(statement.$dbType),
                        startValue: statement.$startValue.$value,
                        increment: statement.$increment.$value,
                    });
                    break;
                }
                case core_1.STATEMENT_KIND.DROP_SEQUENCE: {
                    const index = this.database?.sequences.findIndex(p => (0, util_1.isSameObject)(p.name, statement.$name));
                    if (index === undefined || index < 0) {
                        throw new Error(`Sequence ${this.sqlifier.sqlifyObjectName(statement.$name)} not found.`);
                    }
                    this.database.sequences.splice(index, 1);
                    break;
                }
                case core_1.STATEMENT_KIND.CREATE_INDEX: {
                    (0, util_1.assertAst)(statement.$table, 'Create Index table declare not found.');
                    (0, util_1.assertAst)(statement.$name, 'Create Index name not declared.');
                    (0, util_1.assertAst)(statement.$columns, 'Create Index columns declare not found.');
                    this.assertIndexNotExists(statement.$table, statement.$name);
                    const table = this.getTable(statement.$table);
                    table.indexes.push({
                        name: statement.$name,
                        columns: statement.$columns.map(c => ({
                            name: c.name,
                            isAscending: c.sort === 'ASC',
                        })),
                        isUnique: statement.$unique,
                        isClustered: statement.$clustered,
                    });
                    break;
                }
                case core_1.STATEMENT_KIND.DROP_INDEX: {
                    const table = this.getTable(statement.$table);
                    const index = table.indexes.findIndex(i => i.name === statement.$name);
                    if (index === undefined || index < 0) {
                        throw new Error(`Index ${statement.$name} not found.`);
                    }
                    table.indexes.splice(index, 1);
                    break;
                }
                case core_1.STATEMENT_KIND.STANDARD_STATEMENT:
                    switch (statement.$name) {
                        case 'renameDatabase': {
                            this.assertDatabaseExists(this.database);
                            if (this.database.name !== statement.$datas[0]) {
                                continue;
                            }
                            this.database.name = statement.$datas[1];
                            break;
                        }
                        case 'renameTable': {
                            const table = this.getTable(statement.$datas[0]);
                            table.name = statement.$datas[1];
                            break;
                        }
                        case 'renameColumn': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.name = statement.$datas[2];
                            break;
                        }
                        case 'renameView': {
                            const view = this.findView(statement.$datas[0]);
                            if (!view) {
                                throw new Error(`View ${this.sqlifier.sqlifyObjectName(statement.$datas[0])} is not found.`);
                            }
                            view.name = statement.$datas[1];
                            break;
                        }
                        case 'renameProcedure': {
                            const proc = this.database?.procedures.find(p => (0, util_1.isSameObject)(p.name, statement.$datas[0]));
                            if (!proc) {
                                throw new Error(`Procedure ${this.sqlifier.sqlifyObjectName(statement.$datas[0])} is not found.`);
                            }
                            proc.name = statement.$datas[1];
                            break;
                        }
                        case 'renameFunction': {
                            const func = this.database?.functions.find(p => (0, util_1.isSameObject)(p.name, statement.$datas[0]));
                            if (!func) {
                                throw new Error(`Function ${this.sqlifier.sqlifyObjectName(statement.$datas[0])} is not found.`);
                            }
                            func.name = statement.$datas[1];
                            break;
                        }
                        case 'renameSequence': {
                            const sequence = this.database?.sequences.find(p => (0, util_1.isSameObject)(p.name, statement.$datas[0]));
                            if (!sequence) {
                                throw new Error(`Function ${this.sqlifier.sqlifyObjectName(statement.$datas[0])} is not found.`);
                            }
                            sequence.name = statement.$datas[1];
                            break;
                        }
                        case 'setAutoFlag': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.isRowflag = true;
                            break;
                        }
                        case 'dropAutoFlag': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.isRowflag = false;
                            break;
                        }
                        case 'setIdentity': {
                            const [tableName, columnName, start, incerment] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.isIdentity = true;
                            column.identityStartValue = start;
                            column.identityIncrement = incerment;
                            break;
                        }
                        case 'dropIdentity': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.identityIncrement = undefined;
                            column.identityStartValue = undefined;
                            column.isIdentity = false;
                            break;
                        }
                        case 'setDefaultValue': {
                            const [tableName, columnName, defaultValue] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.defaultValue =
                                this.sqlifier.sqlifyExpression(defaultValue);
                            break;
                        }
                        case 'dropDefaultValue': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.defaultValue = undefined;
                            break;
                        }
                        case 'setAutoRowflag': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.isRowflag = true;
                            break;
                        }
                        case 'dropAutoRowflag': {
                            const [tableName, columnName] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.isRowflag = false;
                            break;
                        }
                        case 'setTableComment': {
                            const [tableName, comment] = statement.$datas;
                            const table = this.getTable(tableName);
                            table.comment = comment;
                            break;
                        }
                        case 'setViewComment': {
                            const [viewName, comment] = statement.$datas;
                            const view = this.getView(viewName);
                            view.comment = comment;
                            break;
                        }
                        case 'setColumnComment': {
                            const [tableName, columnName, comment] = statement.$datas;
                            const column = this.getColumn(tableName, columnName);
                            column.comment = comment;
                            break;
                        }
                        case 'setProcedureComment': {
                            const [procName, comment] = statement.$datas;
                            const proc = this.getProcedure(procName);
                            proc.comment = comment;
                            break;
                        }
                        case 'setFunctionComment': {
                            const [funcName, comment] = statement.$datas;
                            const func = this.getFunction(funcName);
                            func.comment = comment;
                            break;
                        }
                        case 'setSequenceComment': {
                            const [squName, comment] = statement.$datas;
                            const sequence = this.getSequence(squName);
                            sequence.comment = comment;
                            break;
                        }
                        case 'setIndexComment': {
                            const [tableName, indexName, comment] = statement.$datas;
                            const index = this.getIndex(tableName, indexName);
                            index.comment = comment;
                            break;
                        }
                        case 'setConstraintComment': {
                            const [tableName, constraintName, comment] = statement.$datas;
                            const constraint = this.getConstraint(tableName, constraintName);
                            constraint.comment = comment;
                            break;
                        }
                        // case 'setSchemaComment': {
                        //   const [schemaName, comment] = statement.$datas;
                        //   const schema = this.getSchema(schemaName);
                        //   schema.comment = comment;
                        //   break;
                        // }
                        case 'dropTableComment': {
                            const [tableName] = statement.$datas;
                            const table = this.getTable(tableName);
                            table.comment = undefined;
                            break;
                        }
                        case 'dropViewComment': {
                            const [viewName] = statement.$datas;
                            const view = this.getView(viewName);
                            view.comment = undefined;
                            break;
                        }
                        case 'dropProcedureComment': {
                            const [procName] = statement.$datas;
                            const proc = this.getProcedure(procName);
                            proc.comment = undefined;
                            break;
                        }
                        case 'dropFunctionComment': {
                            const [funcName] = statement.$datas;
                            const func = this.getFunction(funcName);
                            func.comment = undefined;
                            break;
                        }
                        case 'dropSequenceComment': {
                            const [squName] = statement.$datas;
                            const sequence = this.getSequence(squName);
                            sequence.comment = undefined;
                            break;
                        }
                        case 'dropIndexComment': {
                            const [tableName, indexName] = statement.$datas;
                            const index = this.getIndex(tableName, indexName);
                            index.comment = undefined;
                            break;
                        }
                        case 'dropConstraintComment': {
                            const [tableName, constraintName] = statement.$datas;
                            const constraint = this.getConstraint(tableName, constraintName);
                            constraint.comment = undefined;
                            break;
                        }
                        // case 'dropSchemaComment': {
                        //   const [schemaName] = statement.$datas;
                        //   const schema = this.getSchema(schemaName);
                        //   schema.comment = undefined;
                        //   break;
                        // }
                    }
            }
        }
    }
    assertDatabaseExists(schema) {
        if (!schema) {
            throw new Error('Database is not exists.');
        }
    }
    getTable(name) {
        const table = this.findTable(name);
        if (!table) {
            throw new Error(`Table ${name} not exists in database.`);
        }
        return table;
    }
    findTable(name) {
        this.assertDatabaseExists(this.database);
        return this.database.tables.find(p => (0, util_1.isSameObject)(name, p.name));
    }
    findView(name) {
        this.assertDatabaseExists(this.database);
        return this.database.views.find(p => (0, util_1.isSameObject)(name, p.name));
    }
    assertTableNotExists(name) {
        this.assertDatabaseExists(this.database);
        const table = this.database.tables.find(p => (0, util_1.isSameObject)(p.name, name));
        if (table)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} is exists.`);
    }
    assertViewNotExists(name) {
        this.assertDatabaseExists(this.database);
        const view = this.database.views.find(p => (0, util_1.isSameObject)(p.name, name));
        if (view)
            throw new Error(`View ${this.sqlifier.sqlifyObjectName(view.name)} is exists.`);
    }
    assertProcedureNotExists(name) {
        this.assertDatabaseExists(this.database);
        const proc = this.database.procedures.find(p => (0, util_1.isSameObject)(p.name, name));
        if (proc)
            throw new Error(`Procedure ${this.sqlifier.sqlifyObjectName(proc.name)} is exists.`);
    }
    assertFunctionNotExists(name) {
        this.assertDatabaseExists(this.database);
        const func = this.database.functions.find(p => (0, util_1.isSameObject)(p.name, name));
        if (func)
            throw new Error(`Procedure ${this.sqlifier.sqlifyObjectName(func.name)} is exists.`);
    }
    assertSequenceNotExists(name) {
        this.assertDatabaseExists(this.database);
        const sequence = this.database.sequences.find(p => (0, util_1.isSameObject)(p.name, name));
        if (sequence)
            throw new Error(`Sequence ${this.sqlifier.sqlifyObjectName(sequence.name)} is exists.`);
    }
    assertSchemaNotExists(name) {
        this.assertDatabaseExists(this.database);
        const schema = this.database.schemas.find(p => p === name);
        if (schema)
            throw new Error(`Schema ${this.sqlifier.sqlifyObjectName(schema)} is exists.`);
    }
    assertColumnNotExists(tableName, columnName) {
        const table = this.getTable(tableName);
        const column = table.columns.find(col => col.name === columnName);
        if (column)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Column ${this.sqlifier.sqlifyObjectName(columnName)} is exists.`);
    }
    assertIndexNotExists(tableName, name) {
        const table = this.getTable(tableName);
        const index = table.indexes.find(idx => idx.name === name);
        if (index)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Index ${this.sqlifier.sqlifyObjectName(name)} is exists.`);
    }
    assertPrimaryNotExists(tableName, name) {
        const table = this.getTable(tableName);
        if (table.primaryKey)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} PrimaryKey is exists.`);
    }
    assertForeignKeyExists(tableName, name) {
        const table = this.getTable(tableName);
        const fk = table.foreignKeys.find(idx => idx.name === name);
        if (fk)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} ForeignKey ${this.sqlifier.sqlifyObjectName(name)} is exists.`);
    }
    assertConstraintNotExists(tableName, name) {
        const table = this.getTable(tableName);
        const constraint = table.constraints.find(ct => ct.name === name);
        if (constraint)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Constraint ${this.sqlifier.sqlifyObjectName(name)} is exists.`);
    }
    addTableMembers(table, members) {
        members.forEach(item => {
            switch (item.$type) {
                case core_1.SQL_SYMBOLE.PRIMARY_KEY: {
                    if (table.primaryKey)
                        throw new Error(`Table ${table.name} primary key is exists.`);
                    (0, util_1.assertAst)(item.$name, 'Add Primary key table name not found.');
                    (0, util_1.assertAst)(item.$columns, 'Add Primary key, table columns not found.');
                    table.primaryKey = {
                        name: item.$name,
                        columns: item.$columns.map(col => ({
                            name: col.name,
                            isAscending: col.sort === 'ASC',
                        })),
                        isNonclustered: item.$nonclustered,
                    };
                    break;
                }
                case core_1.SQL_SYMBOLE.CREATE_TABLE_COLUMN: {
                    if (table.columns.find(col => col.name === item.$name)) {
                        throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} column ${this.sqlifier.sqlifyObjectName(item.$name)} is exists.`);
                    }
                    table.columns.push({
                        name: item.$name,
                        type: this.sqlifier.sqlifyType(item.$dbType),
                        isNullable: item.$nullable ?? false,
                        isIdentity: !!item.$identity,
                        identityStartValue: item.$identity?.startValue,
                        identityIncrement: item.$identity?.increment,
                        isCalculate: !!item.$calculate,
                        defaultValue: item.$default && this.sqlifier.sqlifyExpression(item.$default),
                        calculateExpression: item.$calculate &&
                            this.sqlifier.sqlifyExpression(item.$calculate),
                        isRowflag: false,
                    });
                    break;
                }
                case core_1.SQL_SYMBOLE.FOREIGN_KEY: {
                    (0, util_1.assertAst)(item.$name, 'Foreign key name not found.');
                    (0, util_1.assertAst)(item.$columns, 'Foreign key columns not found.');
                    (0, util_1.assertAst)(item.$referenceColumns, 'Foreign key referenceColumns not found.');
                    (0, util_1.assertAst)(item.$referenceTable, 'Foreign key ReferenceTable not found.');
                    const objName = this.sqlifier.parseObjectName(item.$referenceTable);
                    table.foreignKeys.push({
                        name: item.$name,
                        columns: item.$columns,
                        referenceColumns: item.$referenceColumns,
                        referenceTable: objName.name,
                        referenceSchema: objName.schema ?? this.defaultSchema,
                        isCascade: item.$deleteCascade || false,
                    });
                    break;
                }
                // case SQL_SYMBOLE.UNIQUE_KEY: {
                //   assertAst(item.$columns, 'Unique key name not found.');
                //   assertAst(item.$name, 'Unique key name not found.');
                //   table.constraints.push({
                //     name: item.$name,
                //     kind: 'UNIQUE',
                //     columns: item.$columns.map(c => ({
                //       name: c.name,
                //       isAscending: c.sort === 'ASC',
                //     })),
                //   });
                //   break;
                // }
                case core_1.SQL_SYMBOLE.CHECK_CONSTRAINT: {
                    (0, util_1.assertAst)(item.$name, 'Unique key name not found.');
                    table.constraints.push({
                        kind: 'CHECK',
                        name: item.$name,
                        sql: this.sqlifier.sqlifyCondition(item.$sql),
                    });
                    break;
                }
            }
        });
    }
    createTable(statement) {
        this.assertDatabaseExists(this.database);
        if (this.findTable(statement.$name)) {
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(statement.$name)} is exists.`);
        }
        (0, util_1.assertAst)(statement.$body, 'CreateTable statement member not found.');
        const objName = this.sqlifier.parseObjectName(statement.$name);
        const table = {
            name: objName.name,
            schema: objName.schema ?? this.defaultSchema,
            indexes: [],
            columns: [],
            foreignKeys: [],
            constraints: [],
        };
        this.addTableMembers(table, statement.$body);
        this.database.tables.push(table);
    }
    alterTable(statement) {
        this.assertDatabaseExists(this.database);
        const table = this.getTable(statement.$name);
        (0, util_1.assertAst)(statement.$adds || statement.$drop || statement.$alterColumn, 'AlterTable statement action not found.');
        if (statement.$adds) {
            this.addTableMembers(table, statement.$adds);
        }
        else if (statement.$alterColumn) {
            const column = table.columns.find(p => p.name === statement.$alterColumn?.$name);
            if (!column)
                throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Column ${this.sqlifier.sqlifyObjectName(statement.$alterColumn.$name)} not found.`);
            column.type = this.sqlifier.sqlifyType(statement.$alterColumn.$dbType);
            (0, util_1.assertAst)(statement.$alterColumn.$nullable !== undefined, 'Column nullable not found.');
            column.isNullable = statement.$alterColumn.$nullable;
        }
        else if (statement.$drop) {
            switch (statement.$drop.$kind) {
                case core_1.SQL_SYMBOLE_TABLE_MEMBER.CHECK_CONSTRAINT:
                case core_1.SQL_SYMBOLE_TABLE_MEMBER.UNIQUE_KEY: {
                    const index = table.constraints.findIndex(p => p.name === statement.$drop.$kind);
                    if (index < 0) {
                        throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Constraint ${this.sqlifier.sqlifyObjectName(statement.$drop.$name)} not found.`);
                    }
                    table.constraints.splice(index, 1);
                    break;
                }
                case core_1.SQL_SYMBOLE_TABLE_MEMBER.COLUMN: {
                    const index = table.columns.findIndex(p => p.name === statement.$drop.$kind);
                    if (index < 0) {
                        throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Column ${this.sqlifier.sqlifyObjectName(statement.$drop.$name)} not found.`);
                    }
                    table.columns.splice(index, 1);
                    break;
                }
                case core_1.SQL_SYMBOLE_TABLE_MEMBER.PRIMARY_KEY: {
                    if (table.primaryKey?.name !== statement.$drop.$name) {
                        throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Primary key ${this.sqlifier.sqlifyObjectName(statement.$drop.$name)} not found.`);
                    }
                    table.primaryKey = undefined;
                    break;
                }
                case core_1.SQL_SYMBOLE_TABLE_MEMBER.FOREIGN_KEY: {
                    const index = table.foreignKeys.findIndex(p => p.name === statement.$drop.$kind);
                    if (index < 0) {
                        throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} ForeignKey ${this.sqlifier.sqlifyObjectName(statement.$drop.$name)} not found.`);
                    }
                    table.foreignKeys.splice(index, 1);
                    break;
                }
            }
        }
    }
    // private getSchema(name: any): string {
    //   this.assertDatabaseExists(this.database);
    //   const schema = this.database.schemas.find(p => name === p);
    //   if (!schema) {
    //     throw new Error(
    //       `Schema ${this.sqlifier.sqlifyObjectName(name)} is not found.`
    //     );
    //   }
    //   return schema;
    // }
    getSequence(name) {
        this.assertDatabaseExists(this.database);
        const sequence = this.database.sequences.find(p => (0, util_1.isSameObject)(name, p.name));
        if (!sequence) {
            throw new Error(`Sequence ${this.sqlifier.sqlifyObjectName(name)} is not found.`);
        }
        return sequence;
    }
    getProcedure(name) {
        this.assertDatabaseExists(this.database);
        const proc = this.database.procedures.find(p => (0, util_1.isSameObject)(name, p.name));
        if (!proc) {
            throw new Error(`Procedure ${this.sqlifier.sqlifyObjectName(name)} is not found.`);
        }
        return proc;
    }
    getFunction(name) {
        this.assertDatabaseExists(this.database);
        const func = this.database.functions.find(p => (0, util_1.isSameObject)(name, p.name));
        if (!func) {
            throw new Error(`Function ${this.sqlifier.sqlifyObjectName(name)} is not found.`);
        }
        return func;
    }
    getView(name) {
        const view = this.database?.views.find(p => (0, util_1.isSameObject)(name, p.name));
        if (!view) {
            throw new Error(`View ${this.sqlifier.sqlifyObjectName(name)} is not found.`);
        }
        return view;
    }
    getColumn(tableName, name) {
        const table = this.getTable(tableName);
        const column = table.columns.find(col => col.name === name);
        if (!column)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Column ${this.sqlifier.sqlifyObjectName(name)} not found.`);
        return column;
    }
    getIndex(tableName, name) {
        const table = this.getTable(tableName);
        const index = table.indexes.find(col => col.name === name);
        if (!index)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Index ${this.sqlifier.sqlifyObjectName(name)} not found.`);
        return index;
    }
    getPrimaryKey(tableName, name) {
        const table = this.getTable(tableName);
        if (!table.primaryKey || table.primaryKey.name !== name)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} PrimaryKey ${this.sqlifier.sqlifyObjectName(name)} not found.`);
        return table.primaryKey;
    }
    getForeignKey(tableName, name) {
        const table = this.getTable(tableName);
        const fk = table.foreignKeys.find(col => col.name === name);
        if (!fk)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} ForeignKey ${this.sqlifier.sqlifyObjectName(name)} not found.`);
        return fk;
    }
    getConstraint(tableName, name) {
        const table = this.getTable(tableName);
        let constraint = table.constraints.find(c => c.name === name);
        if (!constraint) {
            constraint = table.foreignKeys.find(p => p.name === name);
        }
        if (!constraint) {
            constraint =
                table.primaryKey?.name === name ? table.primaryKey : undefined;
        }
        if (!constraint)
            throw new Error(`Table ${this.sqlifier.sqlifyObjectName(table.name)} Constraint ${this.sqlifier.sqlifyObjectName(name)} not found.`);
        return constraint;
    }
}
exports.SnapshotMigrateTracker = SnapshotMigrateTracker;
