"use strict";
/* eslint-disable @typescript-eslint/ban-types */
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSchema = void 0;
const core_1 = require("../core");
const util_1 = require("../orm/metadata/util");
/**
 * 从Metadata生成架构
 * @param sqlifier
 * @param context
 * @returns
 */
function generateSchema(context) {
    const sqlifier = context.connection.sqlifier;
    // const databaseName  = await context.lube.provider.getCurrentDatabase();
    // const defaultSchema = await context.lube.provider.getDefaultSchema();
    function genDbSchema(context) {
        // const tables = context.entities.filter(p => isTableEntity(p)).map(entity => genTableSchema(entity as TableEntityMetadata));
        const db = {
            name: context.metadata.database,
            tables: [],
            views: [],
            procedures: [],
            functions: [],
            sequences: [],
            schemas: [],
            comment: context.metadata.comment,
        };
        for (const entity of context.metadata.entities) {
            if ((0, util_1.isTableEntity)(entity)) {
                db.tables.push(genTableSchema(entity));
            }
        }
        // const tableMap = map(db.tables, table => table.name);
        for (const entity of context.metadata.entities) {
            // if (isTableEntity(entity)) {
            //   const foreignKeys = entity.relations
            //     .filter((p) => isForeignRelation(p))
            //     .map((p) =>
            //       genForeignKeySchema(entity, p as ForeignRelation)
            //     );
            //   tableMap[entity.tableName].foreignKeys.push(...foreignKeys);
            // }
            if ((0, util_1.isViewEntity)(entity)) {
                db.views.push(genViewSchema(entity));
            }
        }
        return db;
    }
    function genTableSchema(entity) {
        const columns = entity.columns.map(col => genColumnSchema(col));
        const indexes = entity.indexes.map(index => genIndexSchema(index));
        const foreignKeys = entity.relations
            .filter(p => (0, util_1.isForeignRelation)(p))
            .map(p => genForeignKeySchema(entity, p));
        const table = {
            name: entity.dbName,
            schema: entity.schema,
            primaryKey: {
                name: entity.key.constraintName,
                isNonclustered: entity.key.isNonclustered,
                comment: entity.key.comment,
                columns: [
                    {
                        name: entity.key.column.columnName,
                        isAscending: true,
                    },
                ],
            },
            columns,
            indexes,
            foreignKeys,
            constraints: [],
            comment: entity.comment,
            seedData: entity.data,
        };
        return table;
    }
    function genColumnSchema(column) {
        const col = {
            type: sqlifier.sqlifyType(column.dbType),
            name: column.columnName,
            isNullable: column.isNullable,
            isIdentity: column.isIdentity,
            identityStartValue: column.identityStartValue,
            identityIncrement: column.identityIncrement,
            isCalculate: column.isCalculate,
            isRowflag: column.isRowflag,
            calculateExpression: column.calculateExpression &&
                sqlifier.sqlifyExpression(column.calculateExpression),
            comment: column.comment,
            defaultValue: column.defaultValue && sqlifier.sqlifyExpression(column.defaultValue),
        };
        return col;
    }
    function genForeignKeySchema(entity, relation) {
        const fk = {
            name: relation.constraintName,
            referenceSchema: relation.referenceEntity.schema,
            referenceTable: relation.referenceEntity.dbName,
            columns: [relation.foreignColumn.columnName],
            referenceColumns: [relation.referenceEntity.key.column.columnName],
            isCascade: relation.isCascade,
        };
        return fk;
    }
    function genIndexSchema(index) {
        const idx = {
            name: index.name,
            isUnique: index.isUnique,
            isClustered: index.isClustered,
            columns: index.columns.map(cm => ({
                name: cm.column.columnName,
                isAscending: cm.sort === 'ASC',
            })),
            comment: index.comment,
        };
        return idx;
    }
    function genViewSchema(view) {
        const v = {
            name: view.dbName,
            schema: view.schema ?? sqlifier.options.defaultSchema,
            scripts: sqlifier.sqlify(core_1.SQL.createView(view.dbName).as(view.body)).sql,
            comment: view.comment,
        };
        return v;
    }
    return genDbSchema(context);
}
exports.generateSchema = generateSchema;
