"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pack = exports.assertAst = exports.isSameObject = exports.sortByDependency = void 0;
function sortByDependency(source, dependencies, throwOnCycle = false) {
    const sorted = [];
    const visited = new Set();
    function visit(item) {
        if (!visited.has(item)) {
            visited.add(item);
            for (const dep of dependencies(item))
                visit(dep);
            sorted.push(item);
        }
        else {
            if (throwOnCycle && !sorted.includes(item))
                throw new Error('Cyclic dependency found');
        }
    }
    for (const item of source)
        visit(item);
    return sorted;
}
exports.sortByDependency = sortByDependency;
/**
 * 对比是否是同一个表
 * 如果任何一个名称 不存在架构，均认为它们是有相同的架构
 * @param name1 架构
 * @param name2
 * @returns
 */
function isSameObject(name1, name2) {
    let schema1;
    let table1;
    let schema2;
    let table2;
    if (typeof name1 === 'object') {
        schema1 = name1.schema;
        table1 = name1.name;
    }
    else {
        table1 = name1;
    }
    if (typeof name2 === 'object') {
        schema2 = name2.schema;
        table2 = name2.name;
    }
    else {
        table2 = name2;
    }
    return ((schema1 === schema2 || schema1 === undefined || schema2 === undefined) &&
        table1 === table2);
}
exports.isSameObject = isSameObject;
function assertAst(value, message = 'AST syntax error.') {
    if (!value) {
        throw new Error('AST syntax error:' + message);
    }
}
exports.assertAst = assertAst;
function pack(item) {
    return Array.isArray(item) ? item : [item];
}
exports.pack = pack;
