/// <reference types="node" />
import { Binary, Decimal, DecimalConstructor, Uuid, UuidConstructor } from '../core';
import { Entity, EntityConstructor } from './entity';
/**
 * 过滤关联关系属性列表
 */
export declare type RelationKeyOf<T> = {
    [P in keyof T]: NonNullable<T[P]> extends Entity | Entity[] ? NonNullable<P> : never;
}[keyof T];
export declare type SingleReferenceKeyOf<T> = {
    [P in keyof T]: T[P] extends Entity ? P : never;
};
export declare type MultiReferenceKeyOf<T> = {
    [P in keyof T]: T[P] extends Entity[] ? P : never;
};
export declare type FetchRelations<T = any> = {
    [P in RelationKeyOf<T>]?: true | FetchRelations<T[P] extends Array<infer X> ? X : T[P]>;
};
export declare type DbEvents = 'save' | 'insert' | 'update' | 'delete' | 'saved' | 'inserted' | 'updated' | 'deleted';
export declare type DbContextEventHandler = (event: DbEvents, Entity: Entity, items: Entity[]) => void;
/**
 * 转换
 */
export declare type DataTypeOf<T> = T extends string ? StringConstructor : T extends number ? NumberConstructor : T extends bigint ? BigIntConstructor : T extends Date ? DateConstructor : T extends Decimal ? DecimalConstructor : T extends Uuid ? UuidConstructor : T extends boolean ? BooleanConstructor : T extends Binary ? typeof Buffer | ArrayBufferConstructor | SharedArrayBufferConstructor : T extends (infer M)[] ? DataTypeOf<M>[] : T extends Entity ? EntityConstructor<T> : T extends object ? new (...args: any) => T : never;
/**
 * 标量类型对应的JS原生类型，用于描述字段类型
 */
export declare type ScalarDataType = StringConstructor | DateConstructor | BooleanConstructor | NumberConstructor | DecimalConstructor | BigIntConstructor | ArrayBufferConstructor | typeof Buffer | SharedArrayBufferConstructor | UuidConstructor | JsonType<any> | ListType<BaseDataType>;
export declare type EntityType = EntityConstructor<Entity>;
/******************************* Model 相关声明 *********************************/
export declare type BaseDataType = StringConstructor | DateConstructor | BooleanConstructor | NumberConstructor | DecimalConstructor | BigIntConstructor | ArrayBufferConstructor | typeof Buffer | SharedArrayBufferConstructor | UuidConstructor;
export declare type ListType<T extends BaseDataType> = [T];
export declare type JsonType<T> = (new (value: object) => T) | [JsonType<T>] | StringConstructor | NumberConstructor | BooleanConstructor;
/**
 * Metadata中的数据类型
 */
export declare type DataType = ScalarDataType | EntityType | [EntityType];
export declare type RepositoryEventHandler<T> = (items: T[]) => void;
export declare function isBaseType(type: DataType): type is BaseDataType;
export declare function isJsonType(type: DataType): type is JsonType<any>;
export declare function isListType(type: DataType): type is ListType<BaseDataType>;
