import { Connection } from '../core';
import { Queryable } from './queryable';
import { DeleteOptions, FetchOptions, Repository, SaveOptions } from './repository';
import { DbContextMetadata } from './metadata';
import { Entity, EntityConstructor, EntityInstance, EntityKeyType } from './entity';
import { DbContextEventHandler, DbEvents } from './data-types';
export declare class DbContext {
    constructor(connection: Connection);
    private readonly _connection;
    get connection(): Connection;
    private _emitter;
    private _metadata;
    private _disposed;
    get metadata(): DbContextMetadata;
    protected set metadata(value: DbContextMetadata);
    private _repositories?;
    private get repositories();
    private _mgrs?;
    private get mgrs();
    /**
     * 获取一个可查询对象
     */
    getRepository<T extends Entity>(Entity: EntityConstructor<T>): Repository<T>;
    /**
     * 获取一个仓储库对象
     */
    getQueryable<T extends Entity>(Entity: EntityConstructor<T>): Queryable<T>;
    get<T extends Entity>(Entity: EntityConstructor<T>, key: EntityKeyType, options?: FetchOptions<T>): Promise<EntityInstance<T> | undefined>;
    insert<T extends Entity>(Entity: EntityConstructor<T>, items: T | T[], options?: SaveOptions<T>): Promise<void>;
    insert<T extends Entity>(items: EntityInstance<T> | EntityInstance<T>[], options?: SaveOptions<T>): Promise<void>;
    update<T extends Entity>(Entity: EntityConstructor<T>, items: T | T[], options?: SaveOptions<T>): Promise<void>;
    update<T extends Entity>(items: EntityInstance<T> | EntityInstance<T>[], options?: SaveOptions<T>): Promise<void>;
    delete<T extends Entity>(Entity: EntityConstructor<T>, items: T | T[], options?: DeleteOptions): Promise<void>;
    delete<T extends Entity>(items: EntityInstance<T> | EntityInstance<T>[], options?: DeleteOptions): Promise<void>;
    save<T extends Entity>(Entity: EntityConstructor<T>, items: T | T[], options?: SaveOptions<T>): Promise<void>;
    save<T extends Entity>(items: EntityInstance<T> | EntityInstance<T>[], options?: SaveOptions<T>): Promise<void>;
    /**
     * 数据提交事件
     */
    on(event: 'save', handler: DbContextEventHandler): this;
    on(event: 'insert', handler: DbContextEventHandler): this;
    on(event: 'update', handler: DbContextEventHandler): this;
    on(event: 'delete', handler: DbContextEventHandler): this;
    on(event: 'saved', handler: DbContextEventHandler): this;
    on(event: 'inserted', handler: DbContextEventHandler): this;
    on(event: 'updated', handler: DbContextEventHandler): this;
    on(event: 'deleted', handler: DbContextEventHandler): this;
    on(event: 'all', handler: (event: DbEvents, Entity: Entity, items: Entity[]) => void): this;
    private _emit;
    off(event: 'save', handler?: DbContextEventHandler): this;
    off(event: 'insert', handler?: DbContextEventHandler): this;
    off(event: 'update', handler?: DbContextEventHandler): this;
    off(event: 'delete', handler?: DbContextEventHandler): this;
    off(event: 'saved', handler: DbContextEventHandler): this;
    off(event: 'inserted', handler: DbContextEventHandler): this;
    off(event: 'updated', handler: DbContextEventHandler): this;
    off(event: 'deleted', handler: DbContextEventHandler): this;
    off(event: 'all', handler: (Entity: Entity, event: DbEvents, items: Entity[]) => void): this;
    dispose(): void;
}
export interface DbContextConstructor<T extends DbContext = DbContext> {
    new (lube: Connection): T;
}
/**
 * 数据库上下文对象
 */
