"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.identity = exports.type = exports.noDefaultValue = exports.defaultValue = exports.noAutogen = exports.autogen = exports.noCalculate = exports.calculate = exports.nullable = exports.rowflag = exports.column = exports.getColumnOptions = void 0;
const core_1 = require("../../core");
const util_1 = require("../util");
const entity_decorators_1 = require("./entity-decorators");
require("reflect-metadata");
const sql_1 = require("../../core/sql");
const COLUMN_KEY = Symbol('lubejs:column');
function getColumnOptions(target, key) {
    return Reflect.getMetadata(COLUMN_KEY, target, key);
}
exports.getColumnOptions = getColumnOptions;
function setColumnOptions(target, key, options) {
    let columnOptions = getColumnOptions(target, key);
    if (!columnOptions) {
        columnOptions = {};
        Reflect.defineMetadata(COLUMN_KEY, columnOptions, target, key);
    }
    Object.assign(columnOptions, options);
}
function column(nameOrTypeOrDbType, typeOrDbType, dbType) {
    return function (target, key) {
        let name;
        let dbType;
        let type;
        // 无参数
        if (typeof nameOrTypeOrDbType === 'string') {
            name = nameOrTypeOrDbType;
            if ((0, sql_1.isDbType)(typeOrDbType)) {
                dbType = typeOrDbType;
            }
            else {
                type = typeOrDbType;
            }
        }
        else if ((0, sql_1.isDbType)(nameOrTypeOrDbType)) {
            dbType = nameOrTypeOrDbType;
        }
        else if (typeof nameOrTypeOrDbType === 'function' ||
            Array.isArray(nameOrTypeOrDbType)) {
            type = nameOrTypeOrDbType;
            if ((0, sql_1.isDbType)(typeOrDbType)) {
                dbType = typeOrDbType;
            }
        }
        if (!name) {
            name = key;
        }
        const designType = Reflect.getMetadata('design:type', target, key);
        if (!type) {
            type = designType;
        }
        if (!(0, util_1.isScalarType)(type)) {
            throw new Error('Unsupport column type, you must declare property with ScalarType.');
        }
        setColumnOptions(target.constructor, key, {
            type: type,
            dbType,
            columnName: name,
        });
        (0, entity_decorators_1.addEntitiyColumn)(target.constructor, key);
    };
}
exports.column = column;
function rowflag(yesOrNo = true) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            dbType: core_1.DbType.rowflag,
            isRowflag: yesOrNo,
        });
    };
}
exports.rowflag = rowflag;
function nullable(yesOrNo = true) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            isNullable: yesOrNo,
        });
    };
}
exports.nullable = nullable;
function calculate(exprGetter) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            isCalculate: true,
            calculateExpressionGetter: exprGetter,
        });
    };
}
exports.calculate = calculate;
function noCalculate() {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            isCalculate: false,
            calculateExpressionGetter: undefined,
        });
    };
}
exports.noCalculate = noCalculate;
function autogen(generator) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            generator,
        });
    };
}
exports.autogen = autogen;
/**
 * 移除自动生成
 */
function noAutogen() {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            generator: undefined,
        });
    };
}
exports.noAutogen = noAutogen;
function defaultValue(exprGetter) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            defaultValueGetter: exprGetter,
        });
    };
}
exports.defaultValue = defaultValue;
function noDefaultValue() {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            defaultValueGetter: undefined,
        });
    };
}
exports.noDefaultValue = noDefaultValue;
/**
 * 单独标记属性类型
 * Uuid Decimal Date等对象类型必须使用type进行标记
 */
function type(type) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            type,
        });
    };
}
exports.type = type;
function identity(start, incerment) {
    return function (target, key) {
        setColumnOptions(target.constructor, key, {
            isIdentity: true,
            identityStartValue: start,
            identityIncrement: incerment,
        });
    };
}
exports.identity = identity;
