"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globalKey = exports.database = exports.connection = exports.getConnectionOptions = exports.getDecorateContexts = exports.getContextOptions = exports.context = void 0;
const config_1 = require("../../core/config");
const entity_decorators_1 = require("./entity-decorators");
const db_context_1 = require("../db-context");
require("reflect-metadata");
const CONTEXT_KEY = Symbol('lubejs:context');
const CONNECTION_KEY = Symbol('lubejs:connection');
const DECORATE_CONTEXTS_KEY = Symbol('lubejs:decorate:contexts');
function addDecorateContext(ctr) {
    let ctrs = getDecorateContexts();
    if (!ctrs) {
        ctrs = [];
        Reflect.defineMetadata(DECORATE_CONTEXTS_KEY, ctrs, db_context_1.DbContext);
    }
    if (!ctrs.includes(ctr)) {
        ctrs.push(ctr);
    }
}
function setContextOptions(target, data) {
    let options = getContextOptions(target);
    if (!options) {
        options = {};
        Reflect.defineMetadata(CONTEXT_KEY, options, target);
        addDecorateContext(target);
    }
    Object.assign(options, data);
}
/**
 * 为Entity绑定DbContext
 */
function context(getter) {
    return function (target) {
        (0, entity_decorators_1.setEntityOptions)(target, {
            contextGetter: getter,
        });
    };
}
exports.context = context;
function getContextOptions(target) {
    return Reflect.getMetadata(CONTEXT_KEY, target);
}
exports.getContextOptions = getContextOptions;
function getDecorateContexts() {
    return Reflect.getMetadata(DECORATE_CONTEXTS_KEY, db_context_1.DbContext);
}
exports.getDecorateContexts = getDecorateContexts;
function getConnectionOptions(target) {
    return Reflect.getMetadata(CONNECTION_KEY, target);
}
exports.getConnectionOptions = getConnectionOptions;
function connection(urlOrOptions) {
    return function (target) {
        let connectionOptions;
        if (typeof urlOrOptions === 'string') {
            connectionOptions = (0, config_1.parseConnectionUrl)(urlOrOptions);
        }
        else {
            connectionOptions = urlOrOptions;
        }
        Reflect.defineMetadata(CONNECTION_KEY, connectionOptions, target);
    };
}
exports.connection = connection;
function database(name) {
    return function (target) {
        setContextOptions(target, {
            database: name,
        });
    };
}
exports.database = database;
/**
 * 声明全局主键
 */
function globalKey(name, type) {
    return function (target) {
        setContextOptions(target, {
            globalKeyName: name,
            globalKeyType: type,
        });
    };
}
exports.globalKey = globalKey;
