import { Select } from '../../core';
import { DbContextConstructor } from '../db-context';
import { Entity, EntityConstructor } from '../entity';
import { CommonEntityMetadata, KeyMetadata } from '../metadata';
import { IndexOptions } from './index-decorators';
import 'reflect-metadata';
export interface EntityOptions extends Partial<Pick<CommonEntityMetadata, 'kind' | 'body' | 'dbName' | 'sql' | 'readonly'>> {
    contextGetter?: () => DbContextConstructor;
    seedDatas?: any[];
    indexes: IndexOptions[];
}
export declare function setEntityOptions(target: EntityConstructor, options: Partial<EntityOptions>): void;
export declare function getEntityOptions(target: EntityConstructor): EntityOptions | undefined;
export declare function getDecorateEntities(): EntityConstructor[] | undefined;
export declare function getEntityColumns(target: EntityConstructor): string[] | undefined;
export declare function getEntityIndexs(target: EntityConstructor): string[] | undefined;
export declare function getEntityRelations(target: EntityConstructor): string[] | undefined;
export declare function getEntityKeyOptions(target: EntityConstructor): KeyOptions | undefined;
export declare function addEntitiyColumn(target: EntityConstructor, key: string): void;
export declare function addEntitiyIndex(target: EntityConstructor, key: string): void;
export declare function addEntitiyRelation(target: EntityConstructor, key: string): void;
export declare function addDecorateEntities(ctr: EntityConstructor): void;
export declare type KeyOptions = Partial<KeyMetadata>;
export declare function key(constraintName?: string): PropertyDecorator;
export declare function key(constraintName: string, nonclustered: boolean): PropertyDecorator;
export declare function query<T extends Entity>(sql: Select<T>): (constructor: EntityConstructor<T>) => void;
/**
 * 表装饰器
 */
export declare function table<T extends Entity>(name?: string): (target: EntityConstructor<T>) => void;
export declare function view<T extends Entity>(body: () => Select<T>): (constructor: EntityConstructor<T>) => void;
export declare function view<T extends Entity>(name: string, body: () => Select<T>): (constructor: EntityConstructor<T>) => void;
/**
 * 种子数据
 */
export declare function data<T extends Entity>(datas: any[]): (constructor: EntityConstructor<T>) => void;
