"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.data = exports.view = exports.table = exports.query = exports.key = exports.addDecorateEntities = exports.addEntitiyRelation = exports.addEntitiyIndex = exports.addEntitiyColumn = exports.getEntityKeyOptions = exports.getEntityRelations = exports.getEntityIndexs = exports.getEntityColumns = exports.getDecorateEntities = exports.getEntityOptions = exports.setEntityOptions = void 0;
const console_1 = require("console");
const db_context_1 = require("../db-context");
require("reflect-metadata");
const ENTITY_KEY = Symbol('lubejs:entity');
const ENTITY_COLUMNS_KEY = Symbol('lubejs:entity:columns');
const ENTITY_RELATIONS_KEY = Symbol('lubejs:entity:relations');
const ENTITY_KEY_KEY = Symbol('lubejs:entity:key');
const ENTITY_INDEXES_KEY = Symbol('lubejs:entity:indexes');
const DECORATE_ENTITIES_KEY = Symbol('lubejs:decorate:entities');
function setEntityOptions(target, options) {
    let entityOptions = getEntityOptions(target);
    if (!entityOptions) {
        entityOptions = {
            kind: 'TABLE',
            indexes: [],
        };
        Reflect.defineMetadata(ENTITY_KEY, entityOptions, target);
    }
    Object.assign(entityOptions, options);
}
exports.setEntityOptions = setEntityOptions;
function getEntityOptions(target) {
    return Reflect.getMetadata(ENTITY_KEY, target);
}
exports.getEntityOptions = getEntityOptions;
function getDecorateEntities() {
    return Reflect.getMetadata(DECORATE_ENTITIES_KEY, db_context_1.DbContext);
}
exports.getDecorateEntities = getDecorateEntities;
function getEntityColumns(target) {
    return Reflect.getMetadata(ENTITY_COLUMNS_KEY, target);
}
exports.getEntityColumns = getEntityColumns;
function getEntityIndexs(target) {
    return Reflect.getMetadata(ENTITY_INDEXES_KEY, target);
}
exports.getEntityIndexs = getEntityIndexs;
function getEntityRelations(target) {
    return Reflect.getMetadata(ENTITY_RELATIONS_KEY, target);
}
exports.getEntityRelations = getEntityRelations;
function getEntityKeyOptions(target) {
    return Reflect.getMetadata(ENTITY_KEY_KEY, target);
}
exports.getEntityKeyOptions = getEntityKeyOptions;
function addEntitiyColumn(target, key) {
    let columns = getEntityColumns(target);
    const relations = getEntityRelations(target);
    if (!columns) {
        columns = [];
        Reflect.defineMetadata(ENTITY_COLUMNS_KEY, columns, target);
    }
    if (columns.includes(key) || relations?.includes(key)) {
        throw new Error('Column or relation is allowed decorate once only on property.');
    }
    columns.push(key);
}
exports.addEntitiyColumn = addEntitiyColumn;
function addEntitiyIndex(target, key) {
    let indexs = getEntityIndexs(target);
    const relations = getEntityRelations(target);
    if (!indexs) {
        indexs = [];
        Reflect.defineMetadata(ENTITY_INDEXES_KEY, indexs, target);
    }
    if (indexs.includes(key)) {
        throw new Error('Index or relation is allowed decorate once only on property.');
    }
    indexs.push(key);
}
exports.addEntitiyIndex = addEntitiyIndex;
function addEntitiyRelation(target, key) {
    (0, console_1.assert)(typeof key === 'string', 'Relation property key must typeof string');
    let relations = getEntityRelations(target);
    const columns = getEntityColumns(target);
    if (!relations) {
        relations = [];
        Reflect.defineMetadata(ENTITY_RELATIONS_KEY, relations, target);
    }
    if (relations.includes(key) || columns?.includes(key)) {
        throw new Error('Column or relation is allowed decorate once only on property.');
    }
    relations.push(key);
}
exports.addEntitiyRelation = addEntitiyRelation;
function addDecorateEntities(ctr) {
    let ctrs = getDecorateEntities();
    if (!ctrs) {
        ctrs = [];
        Reflect.defineMetadata(DECORATE_ENTITIES_KEY, ctrs, db_context_1.DbContext);
    }
    if (!ctrs.includes(ctr)) {
        ctrs.push(ctr);
    }
}
exports.addDecorateEntities = addDecorateEntities;
function setKeyOptions(target, options) {
    let keyOptions = getEntityKeyOptions(target);
    if (!keyOptions) {
        keyOptions = {};
        Reflect.defineMetadata(ENTITY_KEY_KEY, keyOptions, target);
    }
    Object.assign(keyOptions, options);
}
function key(constraintNameOrNonclustered, nonclustered) {
    let constraintName;
    if (nonclustered !== undefined) {
        constraintName = constraintNameOrNonclustered;
    }
    else {
        if (constraintNameOrNonclustered !== undefined) {
            nonclustered = constraintNameOrNonclustered;
        }
    }
    return function (target, key) {
        setKeyOptions(target.constructor, {
            property: key,
        });
        if (constraintName) {
            setKeyOptions(target.constructor, {
                constraintName,
            });
        }
        if (constraintNameOrNonclustered !== undefined) {
            setKeyOptions(target.constructor, {
                isNonclustered: nonclustered,
            });
        }
    };
}
exports.key = key;
function query(sql) {
    return function (target) {
        addDecorateEntities(target);
        setEntityOptions(target, {
            kind: 'QUERY',
            sql,
        });
    };
}
exports.query = query;
/**
 * 表装饰器
 */
function table(name) {
    return function (target) {
        addDecorateEntities(target);
        setEntityOptions(target, {
            kind: 'TABLE',
            dbName: name || target.name,
        });
    };
}
exports.table = table;
function view(nameOrBody, body) {
    let name;
    if (typeof nameOrBody === 'string') {
        name = nameOrBody;
    }
    else {
        body = nameOrBody;
    }
    return function (target) {
        addDecorateEntities(target);
        setEntityOptions(target, {
            kind: 'VIEW',
            dbName: name,
            body: body(),
        });
    };
}
exports.view = view;
/**
 * 种子数据
 */
function data(datas) {
    return function (target) {
        setEntityOptions(target, {
            kind: 'QUERY',
            seedDatas: datas,
        });
    };
}
exports.data = data;
