"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unique = exports.index = exports.getIndexOptions = void 0;
const entity_decorators_1 = require("./entity-decorators");
require("reflect-metadata");
const INDEX_KEY = Symbol('lubejs:index');
function getIndexOptions(target, key) {
    return Reflect.getMetadata(INDEX_KEY, target, key);
}
exports.getIndexOptions = getIndexOptions;
function setIndexOptions(target, key, options) {
    let indexOptions = getIndexOptions(target, key);
    if (!indexOptions) {
        indexOptions = {};
        Reflect.defineMetadata(INDEX_KEY, indexOptions, target, key);
    }
    Object.assign(indexOptions, options);
}
function index(name, properties, isUnique, isClustered) {
    if (properties) {
        return function (target) {
            const entityOption = (0, entity_decorators_1.getEntityOptions)(target.constructor);
            const indexOptions = {
                name: name,
                properties,
                isUnique,
                isClustered,
            };
            if (!entityOption) {
                (0, entity_decorators_1.setEntityOptions)(target.constructor, {
                    indexes: [],
                });
            }
            else {
                entityOption.indexes.push(indexOptions);
            }
        };
    }
    return function (target, key) {
        if (!name) {
            name = `IX_${target.constructor.name}_${key}`;
        }
        (0, entity_decorators_1.addEntitiyIndex)(target.constructor, key);
        setIndexOptions(target.constructor, key, {
            name,
            properties: [key],
        });
    };
}
exports.index = index;
function unique(isClustered = false) {
    return function (target, key) {
        setIndexOptions(target.constructor, key, {
            isUnique: true,
            isClustered,
        });
    };
}
exports.unique = unique;
