import { OneToOneMetadata, ManyToOneMetadata, OneToManyMetadata, ManyToManyMetadata } from '../metadata';
import { Scalar } from '../../core';
import { EntityConstructor, Entity } from '../entity';
import 'reflect-metadata';
export interface RelationOptions {
    kind: OneToOneMetadata['kind'] | ManyToOneMetadata['kind'] | OneToManyMetadata['kind'] | ManyToManyMetadata['kind'];
    referenceEntityGetter: () => EntityConstructor;
    referenceProperty?: string;
    relationProperty?: string;
    foreignProperty?: string;
    isRequired?: boolean;
    isPrimary?: boolean;
    isDetail?: boolean;
}
export declare function oneToOne<T extends Entity>(referenceEntityGetter: () => EntityConstructor<T>, referenceProperty?: (p: Required<T>) => object): PropertyDecorator;
/**
 * 将一对一关系声明为主导关系
 */
export declare function principal(): PropertyDecorator;
/**
 * 用于关系字段中，指定外键字段，注意，该装饰器仅可用在导航属性中。
 */
export declare function foreignKey<T extends Entity>(
/**
 * 导航属性名称
 */
foreignProperty: (p: Required<T>) => Scalar, required?: boolean): PropertyDecorator;
export declare function foreignKey(
/**
 * 导航属性名称
 */
foreignProperty?: string, required?: boolean): PropertyDecorator;
export declare function oneToMany<R extends Entity>(referenceEntityGetter: () => EntityConstructor<R>, referenceProperty: (p: Required<R>) => object): PropertyDecorator;
export declare function manyToOne<R extends Entity>(referenceEntityGetter: () => EntityConstructor<R>, referenceProperty?: (p: Required<R>) => object[]): PropertyDecorator;
export declare function manyToMany<R extends Entity>(referenceEntityGetter: () => EntityConstructor<R>, referenceProperty?: (p: Required<R>) => object[], relationProperty?: string): PropertyDecorator;
/**
 * 将导航属性声明为明细项
 */
export declare function detail(yesOrNo?: boolean): PropertyDecorator;
/**
 * 中间表关系
 */
export interface AmongOptions {
    leftEntityGetter: () => EntityConstructor;
    rightEngityGetter: () => EntityConstructor;
    leftProperty?: string;
    rightProperty?: string;
}
/**
 * 定义多对多关系中间表，并可在其中指定其外键属性
 */
export declare function among<T extends Entity, L extends Entity, R extends Entity>(
/**
 * 引用表1
 */
leftGetter: () => EntityConstructor<L>, 
/**
 * 引用表2
 */
rightGetter: () => EntityConstructor<R>, 
/**
 * 引用表1的导航属性
 */
leftProperty?: (p: Required<T>) => L, 
/**
 * 引用表2的导航属性
 */
rightProperty?: (p: Required<T>) => R): (target: EntityConstructor<T>) => void;
export declare function among(
/**
 * 引用表1
 */
leftGetter: () => EntityConstructor, 
/**
 * 引用表2
 */
rightGetter: () => EntityConstructor, 
/**
 * 引用表1的导航属性
 */
leftProperty?: string, // (p: Required<T>) => L,
/**
 * 引用表2的导航属性
 */
rightProperty?: string): (target: EntityConstructor) => void;
export declare function getAmong(target: EntityConstructor): AmongOptions;
export declare function getRelationOptions(target: EntityConstructor, key: string): RelationOptions | undefined;
