"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRelationOptions = exports.getAmong = exports.among = exports.detail = exports.manyToMany = exports.manyToOne = exports.oneToMany = exports.foreignKey = exports.principal = exports.oneToOne = void 0;
const util_1 = require("../util");
const entity_decorators_1 = require("./entity-decorators");
require("reflect-metadata");
const RELATION_KEY = Symbol('lubejs:relation');
const AMONG_KEY = Symbol('lubejs:among');
function oneToOne(referenceEntityGetter, referenceProperty) {
    return function (target, key) {
        const type = Reflect.getMetadata('design:type', target, key);
        if ((0, util_1.isScalarType)(type)) {
            throw new Error(`One to one relation property ${key} type error, property must type of reference entity array.`);
        }
        setRelationOptions(target.constructor, key, {
            kind: 'ONE_TO_ONE',
            referenceProperty: referenceProperty
                ? (0, util_1.selectProperty)(referenceProperty)
                : undefined,
            referenceEntityGetter,
        });
        (0, entity_decorators_1.addEntitiyRelation)(target.constructor, key);
    };
}
exports.oneToOne = oneToOne;
/**
 * 将一对一关系声明为主导关系
 */
function principal() {
    return function (target, key) {
        // const relationOptions = getRelationOptions(target.constructor as EntityConstructor, key);
        // if (relationOptions?.kind && relationOptions.kind !== 'ONE_TO_MANY') {
        //   throw new Error(`Principal is used in one to one relation property.`);
        // }
        setRelationOptions(target.constructor, key, {
            isPrimary: true,
        });
    };
}
exports.principal = principal;
function foreignKey(
/**
 * 导航属性名称
 */
foreignProperty, required = false) {
    return function (target, key) {
        // const relationOptions = getRelationOptions(target.constructor as EntityConstructor, key);
        // if (relationOptions?.kind === 'MANY_TO_MANY' || relationOptions?.kind === 'ONE_TO_MANY') {
        //   throw Error(`No foreign key on relation ${relationOptions.kind}`)
        // }
        setRelationOptions(target.constructor, key, {
            foreignProperty: typeof foreignProperty === 'function'
                ? (0, util_1.selectProperty)(foreignProperty)
                : foreignProperty,
            isRequired: required,
            isPrimary: false
        });
    };
}
exports.foreignKey = foreignKey;
function oneToMany(referenceEntityGetter, referenceProperty) {
    return function (target, key) {
        const type = Reflect.getMetadata('design:type', target, key);
        if (type !== Array) {
            throw new Error(`One to many relation property ${key} type error, property must type of reference entity array.`);
        }
        setRelationOptions(target.constructor, key, {
            kind: 'ONE_TO_MANY',
            referenceEntityGetter,
            referenceProperty: referenceProperty
                ? (0, util_1.selectProperty)(referenceProperty)
                : undefined,
        });
        (0, entity_decorators_1.addEntitiyRelation)(target.constructor, key);
    };
}
exports.oneToMany = oneToMany;
function manyToOne(referenceEntityGetter, referenceProperty) {
    return function (target, key) {
        const type = Reflect.getMetadata('design:type', target, key);
        if ((0, util_1.isScalarType)(type)) {
            throw new Error(`Many to one relation property ${key} type error, property must type of reference entity.`);
        }
        setRelationOptions(target.constructor, key, {
            kind: 'MANY_TO_ONE',
            referenceEntityGetter,
            referenceProperty: referenceProperty
                ? (0, util_1.selectProperty)(referenceProperty)
                : undefined,
        });
        (0, entity_decorators_1.addEntitiyRelation)(target.constructor, key);
    };
}
exports.manyToOne = manyToOne;
function manyToMany(referenceEntityGetter, referenceProperty, relationProperty) {
    return function (target, key) {
        const type = Reflect.getMetadata('design:type', target, key);
        if (type !== Array) {
            throw new Error(`Many to many relation property ${key.toString()} type error, property must type of reference entity array.`);
        }
        setRelationOptions(target.constructor, key, {
            kind: 'MANY_TO_MANY',
            referenceEntityGetter,
            referenceProperty: referenceProperty
                ? (0, util_1.selectProperty)(referenceProperty)
                : undefined,
            relationProperty,
        });
        (0, entity_decorators_1.addEntitiyRelation)(target.constructor, key);
    };
}
exports.manyToMany = manyToMany;
/**
 * 将导航属性声明为明细项
 */
function detail(yesOrNo = true) {
    return function (target, key) {
        setRelationOptions(target.constructor, key, {
            isDetail: yesOrNo
        });
    };
}
exports.detail = detail;
function among(
/**
 * 引用表1
 */
leftGetter, 
/**
 * 引用表2
 */
rightGetter, 
/**
 * 引用表1的导航属性
 */
leftProperty, 
/**
 * 引用表2的导航属性
 */
rightProperty) {
    return function (target) {
        const opt = getAmong(target);
        if (opt) {
            throw new Error(`Among is only allow decorate once.`);
        }
        Reflect.defineMetadata(AMONG_KEY, {
            leftEntityGetter: leftGetter,
            rightEngityGetter: rightGetter,
            leftProperty: typeof leftProperty === 'function'
                ? (0, util_1.selectProperty)(leftProperty)
                : leftProperty,
            // : leftProperty
            //   ? selectProperty(leftProperty)
            //   : undefined,
            rightProperty: typeof rightProperty === 'function'
                ? (0, util_1.selectProperty)(rightProperty)
                : rightProperty,
            // : rightProperty
            //   ? selectProperty(rightProperty)
            //   : undefined,
        }, target);
    };
}
exports.among = among;
function getAmong(target) {
    return Reflect.getMetadata(AMONG_KEY, target);
}
exports.getAmong = getAmong;
function getRelationOptions(target, key) {
    return Reflect.getMetadata(RELATION_KEY, target, key);
}
exports.getRelationOptions = getRelationOptions;
function setRelationOptions(target, key, options) {
    if (typeof key === 'symbol') {
        throw new Error(`Column is not allow use to symbol property.`);
    }
    let relationOptions = getRelationOptions(target, key);
    if (!relationOptions) {
        relationOptions = {};
        Reflect.defineMetadata(RELATION_KEY, relationOptions, target, key);
    }
    if (options.kind && relationOptions.kind) {
        throw Error(`Relation is only decorators once on property.`);
    }
    if (options.isPrimary && options.kind && options.kind !== 'ONE_TO_ONE') {
        throw Error(`principal is only use on one to one relation property.`);
    }
    Object.assign(relationOptions, options);
}
