import { RowObject } from '../core/sql';
import { RelationKeyOf } from './data-types';
declare const ENTITY_TAG: unique symbol;
declare type ENTITY_TAG = typeof ENTITY_TAG;
export declare type EntityData<T extends Entity> = Omit<{
    [P in keyof T]: NonNullable<T[P]> extends Entity ? EntityData<NonNullable<T[P]>> : NonNullable<T[P]> extends EntityArray<infer M> ? EntityData<M>[] : T[P];
}, ENTITY_TAG>;
/**
 * 实体类基类,仅为了typescript区分类型
 * 不一定非得从此继承
 */
export declare class Entity {
    /**
     * 实体类标记
     */
    [ENTITY_TAG]: true;
    static create<T extends Entity>(this: EntityConstructor<T>, data: EntityData<T>): EntityInstance<T>;
    static create<T extends Entity>(this: EntityConstructor<T>, data: EntityData<T>[]): EntityInstance<T>[];
}
export interface EntityConstructor<T extends Entity = any> {
    new (...args: any): T;
}
/**
 * 用于做实体主键类型
 * 用户可自行使用 合并声明扩展
 */
export interface EntityKey {
}
/**
 * 主键的类型
 */
export declare type EntityKeyType = EntityKey[keyof EntityKey] extends never ? number : Exclude<EntityKey[keyof EntityKey], undefined | null>;
declare type EntityArray<T extends Entity> = T[];
/**
 * 主键字段字面量类型
 */
export declare type EntityKeyName = keyof EntityKey extends never ? 'id' : keyof EntityKey;
export declare type EntityInstance<T extends object> = T & {
    [P in RelationKeyOf<T>]: T[P] extends EntityArray<infer M> ? EntityInstance<NonNullable<M>>[] : T[P] extends Entity ? EntityInstance<NonNullable<T[P]>> : T[P];
} & {
    constructor: EntityConstructor<ToEntity<T>>;
};
export declare type ToEntity<T extends RowObject> = T & {
    [ENTITY_TAG]: true;
};
export declare type EntityTypeOf<C extends EntityConstructor<any>> = C extends EntityConstructor<infer T> ? T : never;
export {};
