"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Entity = void 0;
const metadata_store_1 = require("./metadata-store");
const ENTITY_TAG = Symbol('#ENTITY_TAG');
/**
 * 实体类基类,仅为了typescript区分类型
 * 不一定非得从此继承
 */
class Entity {
    static create(data) {
        const metadata = metadata_store_1.metadataStore.getEntity(this);
        if (!metadata) {
            throw new Error(`Entity ${this.name} not register.`);
        }
        const init = (data) => {
            if (data instanceof this)
                return data;
            Object.setPrototypeOf(data, this.prototype);
            for (const relation of metadata.relations) {
                const propData = Reflect.get(data, relation.property);
                if (propData) {
                    Entity.create.call(relation.referenceClass, propData);
                }
            }
            return data;
        };
        if (Array.isArray(data)) {
            for (const item of data) {
                init(item);
            }
        }
        else {
            init(data);
        }
        return data;
    }
}
exports.Entity = Entity;
Entity.prototype[ENTITY_TAG] = true;
