"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.metadataStore = exports.MetadataStore = void 0;
/**********************************装饰器声明*************************************/
class MetadataStore {
    constructor() {
        this.entityMap = new Map();
        this.contextMap = new Map();
    }
    /**
     * 默认DbContext元数据
     */
    get defaultContext() {
        if (!this._defaultContext) {
            throw new Error(`No DbContext is registered.`);
        }
        return this._defaultContext;
    }
    /**
     * 可以通过实体构造函数/实体名称 获取已注册的元数据
     */
    getEntity(entityClass) {
        const entity = this.entityMap.get(entityClass);
        if (!entity) {
            throw new Error(`Entity ${entityClass.name} is not reigster.`);
        }
        return entity;
    }
    /**
     * 注册一个实体
     */
    registerEntity(metadata) {
        if (this.entityMap.has(metadata.class)) {
            throw new Error(`Entity ${metadata.class.name} is registered.`);
        }
        this.entityMap.set(metadata.class, metadata);
        return this;
    }
    getContext(contextClass) {
        let metadata;
        if (typeof contextClass === 'string') {
            metadata = Array.from(this.contextMap.values()).find(metadata => metadata.class.name === contextClass);
        }
        else {
            metadata = this.contextMap.get(contextClass);
        }
        if (!metadata) {
            throw new Error(`DbContext ${typeof contextClass === 'string' ? contextClass : contextClass.name} not register`);
        }
        return metadata;
    }
    /**
     * 注册上下文
     */
    registerContext(metadata) {
        if (this.contextMap.has(metadata.class)) {
            throw new Error(`DbContext ${metadata.class.name} is registered.`);
        }
        this.contextMap.set(metadata.class, metadata);
        for (const entity of metadata.entities) {
            this.registerEntity(entity);
        }
        if (!this._defaultContext) {
            this._defaultContext = metadata;
        }
        return this;
    }
}
exports.MetadataStore = MetadataStore;
/**
 * 公共元数据存储
 */
exports.metadataStore = new MetadataStore();
