"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityMetadata = void 0;
const util_1 = require("./util");
/**
 * 表格实体元数据
 */
class EntityMetadata {
    constructor() {
        this._columnMap = {};
        this._columns = [];
        this._relationMap = {};
        this._indexMap = {};
        this._reations = [];
        this._indexes = [];
    }
    get indexes() {
        return this._indexes;
    }
    addIndex(index) {
        if (this._indexMap[index.name]) {
            throw new Error(`Index ${index.name} is exists in Entity.`);
        }
        this._indexes.push(index);
        this._indexMap[index.name] = index;
        return this;
    }
    addColumn(item) {
        if (this._columnMap[item.property])
            throw new Error(`Column ${item.property} is exists in Entity.`);
        this._columnMap[item.property] = item;
        this._columns.push(item);
        return this;
    }
    addRelation(item) {
        if (this._relationMap[item.property])
            throw new Error(`Relation ${item.property} is exists in Entity.`);
        this._relationMap[item.property] = item;
        this._reations.push(item);
        return this;
    }
    get columns() {
        return this._columns;
    }
    getColumn(name) {
        return this._columnMap[name];
    }
    get relations() {
        return this._reations;
    }
    getRelation(name) {
        return this._relationMap[name];
    }
    getIndex(name) {
        return this._indexMap[name];
    }
    getDetailIncludes() {
        if (this._detailIncludes === undefined) {
            const detailRelations = this.relations.filter(r => ((0, util_1.isPrimaryOneToOne)(r) || (0, util_1.isOneToMany)(r) || (0, util_1.isManyToMany)(r)) &&
                r.isDetail);
            if (detailRelations.length === 0) {
                this._detailIncludes = undefined;
            }
            else {
                const detailIncludes = {};
                // TIPS: 先赋值，后取值，以此来规避死循环
                this._detailIncludes = detailIncludes;
                detailRelations.forEach(relation => {
                    detailIncludes[relation.property] =
                        relation.referenceEntity.getDetailIncludes() || true;
                });
            }
        }
        return this._detailIncludes;
    }
    getDetailRelations() {
        const detailRelations = this.relations.filter(relation => ((0, util_1.isPrimaryOneToOne)(relation) ||
            (0, util_1.isOneToMany)(relation) ||
            (0, util_1.isManyToMany)(relation)) &&
            relation.isDetail);
        return detailRelations;
    }
    getSubordinateRelations() {
        return this.relations.filter(relation => (0, util_1.isPrimaryOneToOne)(relation) ||
            (0, util_1.isOneToMany)(relation) ||
            (0, util_1.isManyToMany)(relation));
    }
    getSuperiorRelations() {
        return this.relations.filter(relation => (0, util_1.isForeignOneToOne)(relation) || (0, util_1.isManyToOne)(relation));
    }
}
exports.EntityMetadata = EntityMetadata;
