import { Entity, EntityConstructor } from '../entity';
import { XRowset } from '../../core';
import { EntityMetadata, TableEntityMetadata, ViewEntityMetadata, QueryEntityMetadata } from './entity-metadata';
import { RelationMetadata, OneToOneMetadata, PrimaryOneToOneMetadata, ForeignOneToOneMetadata, OneToManyMetadata, ManyToOneMetadata, ManyToManyMetadata, ForeignRelationMetadata } from './relation-metadata';
import { EntityType } from '../data-types';
import { DefaultRowObject } from '../../core/sql';
export declare function aroundRowset<T extends Entity = any>(rowset: XRowset<T>, metadata: EntityMetadata): XRowset<T>;
/**
 * 从实体创建行集对象
 * @param entity
 * @returns
 */
export declare function makeRowset<T extends Entity = any>(entity: EntityConstructor<T>): XRowset<DefaultRowObject>;
/**
 * 是否一对一关系
 */
export declare function isOneToOne(relation: RelationMetadata): relation is OneToOneMetadata;
/**
 * 是否主要一对一关系
 */
export declare function isPrimaryOneToOne(relation: RelationMetadata): relation is PrimaryOneToOneMetadata;
/**
 * 是否外键一对一关系
 */
export declare function isForeignOneToOne(relation: RelationMetadata): relation is ForeignOneToOneMetadata;
/**
 * 是否一对多关系
 */
export declare function isOneToMany(relation: RelationMetadata): relation is OneToManyMetadata;
/**
 * 是否多对一关系
 */
export declare function isManyToOne(relation: any): relation is ManyToOneMetadata;
/**
 * 是否多对多关系
 */
export declare function isManyToMany(relation: RelationMetadata): relation is ManyToManyMetadata;
/**
 * 表示此关系所在的实体，在数据库关系中是否处于外键地位
 * @param relation
 * @returns
 */
export declare function isForeignRelation(relation: RelationMetadata): relation is ForeignRelationMetadata;
/**
 * 是否实体类型
 */
export declare function isEntityType(type: any): type is EntityType;
export declare function isEntityMetadata(value: any): value is EntityMetadata;
export declare function isTableEntity(value: any): value is TableEntityMetadata;
export declare function isViewEntity(value: any): value is ViewEntityMetadata;
export declare function isQueryEntity(value: any): value is QueryEntityMetadata;
