"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isQueryEntity = exports.isViewEntity = exports.isTableEntity = exports.isEntityMetadata = exports.isEntityType = exports.isForeignRelation = exports.isManyToMany = exports.isManyToOne = exports.isOneToMany = exports.isForeignOneToOne = exports.isPrimaryOneToOne = exports.isOneToOne = exports.makeRowset = exports.aroundRowset = void 0;
const core_1 = require("../../core");
const util_1 = require("../util");
const entity_metadata_1 = require("./entity-metadata");
const metadata_store_1 = require("../metadata-store");
const ENTITY_COLUMN_MAPS = new Map();
function aroundRowset(rowset, metadata) {
    if (!ENTITY_COLUMN_MAPS.has(metadata)) {
        const map = {};
        metadata.columns.forEach(col => {
            map[col.property] = col.columnName;
        });
        ENTITY_COLUMN_MAPS.set(metadata, map);
    }
    rowset.$around(ENTITY_COLUMN_MAPS.get(metadata));
    return rowset;
}
exports.aroundRowset = aroundRowset;
/**
 * 从实体创建行集对象
 * @param entity
 * @returns
 */
function makeRowset(entity) {
    const metadata = metadata_store_1.metadataStore.getEntity(entity);
    if (!metadata)
        throw new Error(`No metadata found ${entity}`);
    let rowset;
    if (isQueryEntity(metadata)) {
        rowset = metadata.sql.as('_');
    }
    else if (isTableEntity(metadata)) {
        rowset = core_1.Table.create(metadata.dbName);
        // SQL.table(metadata.tableName);
    }
    else {
        rowset = core_1.Table.create(metadata.dbName);
    }
    return aroundRowset(rowset, metadata);
}
exports.makeRowset = makeRowset;
/**
 * 是否一对一关系
 */
function isOneToOne(relation) {
    return relation.kind === 'ONE_TO_ONE';
}
exports.isOneToOne = isOneToOne;
/**
 * 是否主要一对一关系
 */
function isPrimaryOneToOne(relation) {
    return isOneToOne(relation) && relation.isPrimary === true;
}
exports.isPrimaryOneToOne = isPrimaryOneToOne;
/**
 * 是否外键一对一关系
 */
function isForeignOneToOne(relation) {
    return isOneToOne(relation) && relation.isPrimary === false;
}
exports.isForeignOneToOne = isForeignOneToOne;
/**
 * 是否一对多关系
 */
function isOneToMany(relation) {
    return relation.kind === 'ONE_TO_MANY';
}
exports.isOneToMany = isOneToMany;
/**
 * 是否多对一关系
 */
function isManyToOne(relation) {
    return relation.kind === 'MANY_TO_ONE';
}
exports.isManyToOne = isManyToOne;
/**
 * 是否多对多关系
 */
function isManyToMany(relation) {
    return relation.kind === 'MANY_TO_MANY';
}
exports.isManyToMany = isManyToMany;
/**
 * 表示此关系所在的实体，在数据库关系中是否处于外键地位
 * @param relation
 * @returns
 */
function isForeignRelation(relation) {
    return isManyToOne(relation) || isForeignOneToOne(relation);
}
exports.isForeignRelation = isForeignRelation;
/**
 * 是否实体类型
 */
function isEntityType(type) {
    return typeof type === 'function' && (0, util_1.isClass)(type);
}
exports.isEntityType = isEntityType;
function isEntityMetadata(value) {
    return (value?.kind &&
        (value.kind === 'TABLE' || value.kind === 'VIEW' || value.kind === 'QUERY'));
}
exports.isEntityMetadata = isEntityMetadata;
function isTableEntity(value) {
    return value instanceof entity_metadata_1.EntityMetadata && value.kind === 'TABLE';
}
exports.isTableEntity = isTableEntity;
function isViewEntity(value) {
    return value instanceof entity_metadata_1.EntityMetadata && value.kind === 'VIEW';
}
exports.isViewEntity = isViewEntity;
function isQueryEntity(value) {
    return value instanceof entity_metadata_1.EntityMetadata && value.kind === 'QUERY';
}
exports.isQueryEntity = isQueryEntity;
