import { UuidConstructor } from "../../core";
import { ConnectOptions } from "../../core/base/connection";
import { Scalar } from "../../core/sql";
import { DbContext } from "../db-context";
import { Entity, EntityConstructor } from "../entity";
import { DbContextMetadata } from "../metadata";
import { ScalarDataType } from "../data-types";
import { EntityBuilder } from "./entity-builder";
import { PropertyBuilder } from "./property-builder";
export declare class ContextBuilder<T extends DbContext = DbContext> {
    readonly metadata: DbContextMetadata;
    constructor(ctr: new (...args: any) => T);
    /**
     * 实体映射
     */
    private _entityMap;
    /**
     * 是否已经完成
     */
    private _isReady;
    get isReady(): boolean;
    /**
     * 指定数据连接
     */
    connectWith(connectionOptions: ConnectOptions): void;
    hasComment(comment: string): void;
    /**
     * 声明实体
     */
    entity<T extends Entity>(ctr: EntityConstructor<T>): EntityBuilder<T>;
    entity<T extends Entity>(ctr: EntityConstructor<T>, build: (builder: EntityBuilder<T>) => void): this;
    /**
     * 结束建模过程并完善数据并校验完整性
     */
    ready(): void;
    /**
     * 指定全局主键字段名称
     */
    hasGlobalKey<T extends NumberConstructor | StringConstructor | BigIntConstructor | UuidConstructor>(name: string, type: ScalarDataType): PropertyBuilder<Entity, Scalar>;
}
