"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasManyBuilder = void 0;
const util_1 = require("../util");
const many_to_many_builder_1 = require("./many-to-many-builder");
const one_to_many_builder_1 = require("./one-to-many-builder");
class HasManyBuilder {
    constructor(contextBuilder, entityBuilder, metadata) {
        this.contextBuilder = contextBuilder;
        this.entityBuilder = entityBuilder;
        this.metadata = metadata;
    }
    assertWith() {
        if (this.metadata.kind) {
            throw new Error(`HasManyMetadata is withed ${this.metadata.kind} relation.`);
        }
    }
    withOne(selector) {
        this.assertWith();
        this.metadata.kind = 'ONE_TO_MANY';
        if (selector) {
            this.metadata.referenceProperty = (0, util_1.selectProperty)(selector);
            if (!this.metadata.referenceProperty) {
                throw new Error('Please select a property');
            }
            const referenceRelation = this.metadata.referenceEntity.getRelation(this.metadata.referenceProperty);
            if (referenceRelation) {
                this.metadata.referenceRelation = referenceRelation;
                referenceRelation.referenceRelation = this
                    .metadata;
                referenceRelation.referenceProperty = this.metadata.property;
            }
        }
        const manyToOne = new one_to_many_builder_1.OneToManyBuilder(this.contextBuilder, this.entityBuilder, this.metadata);
        return manyToOne;
    }
    withMany(selector) {
        this.assertWith();
        this.metadata.kind = 'MANY_TO_MANY';
        const metadata = this
            .metadata;
        if (selector) {
            metadata.referenceProperty = (0, util_1.selectProperty)(selector);
            if (!metadata.referenceProperty) {
                throw new Error('Please select a property');
            }
            const referenceRelation = this.metadata.referenceEntity.getRelation(this.metadata.referenceProperty);
            if (referenceRelation) {
                // 相互关联对向关系
                this.metadata.referenceRelation = referenceRelation;
                referenceRelation.referenceRelation = this
                    .metadata;
                referenceRelation.referenceProperty = this.metadata.property;
            }
        }
        const builder = new many_to_many_builder_1.ManyToManyBuilder(this.contextBuilder, this.entityBuilder, this.metadata);
        return builder;
    }
}
exports.HasManyBuilder = HasManyBuilder;
