"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasOneBuilder = void 0;
const util_1 = require("../util");
const one_to_many_builder_1 = require("./one-to-many-builder");
const one_to_one_map_builder_1 = require("./one-to-one-map-builder");
class HasOneBuilder {
    constructor(contextBuilder, entityBuilder, metadata) {
        this.contextBuilder = contextBuilder;
        this.entityBuilder = entityBuilder;
        this.metadata = metadata;
    }
    assertWith() {
        if (this.metadata.kind) {
            throw new Error(`HasManyMetadata is withed ${this.metadata.kind} relation.`);
        }
    }
    withOne(selector) {
        this.assertWith();
        this.metadata.kind = 'ONE_TO_ONE';
        if (this.metadata.referenceProperty) {
            const referenceRelation = this.metadata.referenceEntity.getRelation(this.metadata.referenceProperty);
            if (referenceRelation) {
                // 相互关联对向关系
                this.metadata.referenceRelation = referenceRelation;
                referenceRelation.referenceRelation = this.metadata;
                referenceRelation.referenceProperty = this.metadata.property;
            }
        }
        if (selector) {
            this.metadata.referenceProperty = (0, util_1.selectProperty)(selector);
            if (!this.metadata.referenceProperty) {
                throw new Error('Please select a property');
            }
        }
        const oneToOne = new one_to_one_map_builder_1.OneToOneMapBuilder(this.contextBuilder, this.entityBuilder, this.metadata);
        return oneToOne;
    }
    withMany(selector) {
        this.assertWith();
        this.metadata.kind = 'MANY_TO_ONE';
        if (selector) {
            this.metadata.referenceProperty = (0, util_1.selectProperty)(selector);
            if (!this.metadata.referenceProperty) {
                throw new Error('Please select a property');
            }
            const referenceRelation = this.metadata.referenceEntity.getRelation(this.metadata.referenceProperty);
            if (referenceRelation) {
                // 相互关联对向关系
                this.metadata.referenceRelation = referenceRelation;
                referenceRelation.referenceRelation = this
                    .metadata;
                referenceRelation.referenceProperty = this.metadata.property;
            }
        }
        const oneToMay = new one_to_many_builder_1.ManyToOneBuilder(this.contextBuilder, this.entityBuilder, this.metadata);
        return oneToMay;
    }
}
exports.HasOneBuilder = HasOneBuilder;
