import { DbContextConstructor, DbContext } from '../db-context';
import { ContextBuilder } from './context-builder';
export declare class ModelBuilder {
    private constructor();
    private buildDecoratorDbContext;
    private buildDecoratorEntity;
    private buildDecorators;
    static readonly instance: ModelBuilder;
    private contextMap;
    private isDecoratorBuilt;
    /**
     * 声明数据库上下文类
     * @param context 数据库上下文类
     * @param build 模型构建在此函数中完成，在build函数完成后会自动调用 builder.ensure()
     */
    context<T extends DbContext>(context: DbContextConstructor<T>, build: (builder: ContextBuilder<T>) => void | Promise<void>): this;
    context<T extends DbContext>(context: DbContextConstructor<T>): ContextBuilder<T>;
    /**
     * 完成建模工作后，必须调用该方法，才能正常使用ORM功能。
     * 此方法请在ORM准备完毕时调用
     */
    ready(): void;
}
export declare const modelBuilder: ModelBuilder;
