import { Entity } from '../entity';
import { OneToOneMetadata, PrimaryOneToOneMetadata, ForeignOneToOneMetadata } from '../metadata';
import { ContextBuilder } from './context-builder';
import { EntityBuilder } from './entity-builder';
export declare abstract class OneToOneBuilder<S extends Entity, D extends Entity> {
    private contextBuilder;
    private entityBuilder;
    readonly metadata: Partial<OneToOneMetadata>;
    constructor(contextBuilder: ContextBuilder, entityBuilder: EntityBuilder<S>, metadata: Partial<OneToOneMetadata>);
    hasComment(comment: string): Omit<this, 'hasComment'>;
}
export declare class PrimaryOneToOneBuilder<S extends Entity, D extends Entity> extends OneToOneBuilder<S, D> {
    readonly metadata: PrimaryOneToOneMetadata;
    /**
     * 声明为明细项
     * 获取或查询时传递withDetail选项，将自动附带
     * 删除时亦将同步删除，默认为联动删除
     */
    isDetail(yesOrNo?: boolean): Omit<this, 'isDetail'>;
}
export declare class ForeignOneToOneBuilder<S extends Entity, D extends Entity> extends OneToOneBuilder<S, D> {
    readonly metadata: ForeignOneToOneMetadata;
    hasConstraintName(name: string): Omit<this, 'hasConstraintName'>;
    isRequired(yesOrNo?: boolean): Omit<this, 'isNullable'>;
    isCascade(yesOrNo?: boolean): Omit<this, 'isCascade'>;
}
