"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OneToOneMapBuilder = void 0;
const util_1 = require("../util");
const one_to_one_builder_1 = require("./one-to-one-builder");
class OneToOneMapBuilder {
    constructor(contextBuilder, entityBuilder, metadata) {
        this.contextBuilder = contextBuilder;
        this.entityBuilder = entityBuilder;
        this.metadata = metadata;
    }
    /**
     * 声明当前实体在一对一关系中占主键地位
     */
    isPrimary() {
        this.assertPrimary();
        if (this.metadata.referenceProperty) {
            const referenceRelation = this.metadata.referenceEntity.getRelation(this.metadata.referenceProperty);
            if (referenceRelation) {
                if (referenceRelation.isPrimary === true) {
                    throw new Error(`Entity ${this.entityBuilder.metadata.class.name} PrimaryOneToOne relation ${this.metadata.property}, reference relation must be ForeignOneToOne relation, use .hasForeignKey()`);
                }
                else {
                    // 补齐
                    referenceRelation.isPrimary = false;
                }
                this.metadata.referenceRelation = referenceRelation;
            }
        }
        this.metadata.isPrimary = true;
        return new one_to_one_builder_1.PrimaryOneToOneBuilder(this.contextBuilder, this.entityBuilder, this.metadata);
    }
    /**
     * 声明为一对一（从）关系，并指定外键属性
     * @param selector
     * @returns
     */
    hasForeignKey(selector) {
        this.assertPrimary();
        this.metadata.isPrimary = false;
        if (selector) {
            const foreignProperty = (0, util_1.selectProperty)(selector);
            if (!foreignProperty) {
                throw new Error(`Pls select a property`);
            }
            this.metadata.foreignProperty =
                foreignProperty;
        }
        return new one_to_one_builder_1.ForeignOneToOneBuilder(this.contextBuilder, this.entityBuilder, this.metadata);
    }
    assertPrimary() {
        if (this.metadata.isPrimary !== undefined) {
            throw new Error(`Primary is ensured, pls do not repeat that.`);
        }
    }
}
exports.OneToOneMapBuilder = OneToOneMapBuilder;
