import { Scalar, DbType, XRowset, XExpression } from '../../core';
import { DbContext } from '../db-context';
import { Entity } from '../entity';
import { ColumnMetadata } from '../metadata';
import { ContextBuilder } from './context-builder';
import { EntityBuilder } from './entity-builder';
export declare class PropertyBuilder<T extends Entity, V extends Scalar = Scalar> {
    private readonly contextBuilder;
    private readonly entityBuilder;
    readonly metadata: Partial<ColumnMetadata<V>>;
    constructor(contextBuilder: ContextBuilder, entityBuilder: EntityBuilder<T>, metadata: Partial<ColumnMetadata<V>>);
    /**
     * 声明为列
     */
    hasColumnName(columnName: string): Omit<this, 'hasName'>;
    /**
     * 数据类型
     */
    hasType(dbType: DbType): Omit<this, 'hasType'>;
    /**
     * 列自动填充值，通过表达式生成，如调用函数等
     * @param generator
     * @returns
     */
    isAutogen(generator: (rowset: XRowset<T>, item: T, context: DbContext) => XExpression<V>): this;
    isNotAutogen(): this;
    /**
     * 是否可空
     */
    isNullable(yesOrNo?: boolean): Omit<this, 'isNullable'>;
    isRequired(yesOrNo?: boolean): this;
    /**
     * 行标记列，每次更新时自动变换值
     * 同时会修改列的类型为DbType.rowflag
     */
    isRowflag(yesOrNo?: boolean): Omit<this, 'isRowflag'>;
    /**
     * 标识列
     */
    isIdentity(seed?: number, step?: number): this;
    /**
     * 移除标识列
     */
    isNotIdentity(): this;
    /**
     * 默认值
     */
    hasDefaultValue(expr: XExpression<V>): this;
    /**
     * 移除默认值
     */
    hasNoDefaultValue(): this;
    /**
     * 摘要说明
     */
    hasComment(comment?: string): Omit<this, 'commentBy'>;
    /**
     * 将列声明为计算列
     */
    isCalculated(expr: XExpression<V>): this;
    isNotCalculated(): this;
}
