"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyBuilder = void 0;
const core_1 = require("../../core");
const util_1 = require("./util");
class PropertyBuilder {
    constructor(contextBuilder, entityBuilder, metadata) {
        this.contextBuilder = contextBuilder;
        this.entityBuilder = entityBuilder;
        this.metadata = metadata;
    }
    /**
     * 声明为列
     */
    hasColumnName(columnName) {
        this.metadata.columnName = columnName;
        return this;
    }
    /**
     * 数据类型
     */
    hasType(dbType) {
        this.metadata.dbType = dbType;
        this.metadata.isRowflag = dbType.type === 'ROWFLAG';
        return this;
    }
    /**
     * 列自动填充值，通过表达式生成，如调用函数等
     * @param generator
     * @returns
     */
    isAutogen(generator) {
        this.metadata.generator = generator;
        return this;
    }
    isNotAutogen() {
        delete this.metadata.generator;
        return this;
    }
    /**
     * 是否可空
     */
    isNullable(yesOrNo = true) {
        this.metadata.isNullable = yesOrNo;
        return this;
    }
    isRequired(yesOrNo = true) {
        this.metadata.isNullable = yesOrNo;
        return this;
    }
    /**
     * 行标记列，每次更新时自动变换值
     * 同时会修改列的类型为DbType.rowflag
     */
    isRowflag(yesOrNo = true) {
        if (yesOrNo) {
            this.metadata.dbType = core_1.DbType.rowflag;
        }
        else {
            if (this.metadata.type) {
                this.metadata.dbType = (0, util_1.dataTypeToDbType)(this.metadata.type);
            }
            else {
                throw new Error(`Property ${this.metadata.property} has not type yet.`);
            }
        }
        this.metadata.isRowflag = yesOrNo;
        return this;
    }
    /**
     * 标识列
     */
    isIdentity(seed, step) {
        this.metadata.isIdentity = true;
        this.metadata.identityStartValue = seed ?? 0;
        this.metadata.identityIncrement = step ?? 1;
        return this;
    }
    /**
     * 移除标识列
     */
    isNotIdentity() {
        this.metadata.isIdentity = false;
        delete this.metadata.identityStartValue;
        delete this.metadata.identityIncrement;
        return this;
    }
    /**
     * 默认值
     */
    hasDefaultValue(expr) {
        this.metadata.defaultValue = core_1.Expression.ensureExpression(expr);
        return this;
    }
    /**
     * 移除默认值
     */
    hasNoDefaultValue() {
        delete this.metadata.defaultValue;
        return this;
    }
    /**
     * 摘要说明
     */
    hasComment(comment) {
        this.metadata.comment = comment;
        return this;
    }
    /**
     * 将列声明为计算列
     */
    isCalculated(expr) {
        this.metadata.isCalculate = true;
        this.metadata.calculateExpression = core_1.Expression.ensureExpression(expr);
        return this;
    }
    isNotCalculated() {
        this.metadata.isCalculate = false;
        delete this.metadata.calculateExpression;
        return this;
    }
}
exports.PropertyBuilder = PropertyBuilder;
