import { DbType } from '../../core';
import { DbContextMetadata, EntityMetadata, ColumnMetadata, TableEntityMetadata, IndexMetadata, ManyToOneMetadata, OneToManyMetadata, ManyToManyMetadata, OneToOneMetadata, PrimaryOneToOneMetadata, ForeignOneToOneMetadata, RelationMetadata, ForeignRelationMetadata } from '../metadata';
import { DataType } from '../data-types';
import { ContextBuilder } from './context-builder';
/**
 * 修复列声明
 * @param ctx
 * @param entity
 * @param column
 */
export declare function fixColumn(column: ColumnMetadata): void;
export declare function fixEntityIndexes(entity: TableEntityMetadata): void;
export declare function fixIndex(entity: TableEntityMetadata, index: IndexMetadata): void;
export declare function fixRelationIndex(entity: TableEntityMetadata, relation: ForeignRelationMetadata): void;
/**
 * 修复多对一声明
 * @param ctx
 * @param entity
 * @param relation
 */
export declare function fixManyToOne(ctx: DbContextMetadata, entity: TableEntityMetadata, relation: ManyToOneMetadata): void;
/**
 * 修复一对多关系声明
 * @param ctx
 * @param entity
 * @param relation
 */
export declare function fixOneToMany(ctx: DbContextMetadata, entity: TableEntityMetadata, relation: OneToManyMetadata): void;
export declare function fixEntityKey(builder: ContextBuilder, entity: EntityMetadata): void;
export declare function fixEntity(builder: ContextBuilder, entity: EntityMetadata): void;
/**
 * 完成多对多声明
 * @param ctx
 * @param entity
 * @param relation
 */
export declare function fixManyToMany(ctxBuilder: ContextBuilder, entity: TableEntityMetadata, relation: ManyToManyMetadata): void;
/**
 * 修复一对一关系声明
 * @param ctx
 * @param entity
 * @param relation
 */
export declare function fixOneToOne(ctx: DbContextMetadata, entity: TableEntityMetadata, relation: OneToOneMetadata): void;
export declare function fixPrimaryOneToOne(ctx: DbContextMetadata, entity: TableEntityMetadata, relation: PrimaryOneToOneMetadata): void;
export declare function fixForeignOneToOne(ctx: DbContextMetadata, entity: TableEntityMetadata, relation: ForeignOneToOneMetadata): void;
/**
 * 确保外键生效
 * @param entity
 * @param relation
 */
export declare function fixForeignProperty(entity: TableEntityMetadata, relation: ManyToOneMetadata | ForeignOneToOneMetadata): void;
export declare function fixReferenceEntity(ctx: DbContextMetadata, entity: TableEntityMetadata, relation: RelationMetadata): void;
/**
 * 生成一个关系属性名称
 * @param entity
 * @param referenceType
 * @param type
 * @returns
 */
export declare function genRelationProperty(referenceType: string, type: 'one' | 'many'): string;
export declare function fixEntityRelations(builder: ContextBuilder, entity: TableEntityMetadata): void;
/**
 * Js类型转换为默认数据库类型
 * @param dataType
 * @returns
 */
export declare function dataTypeToDbType(dataType: DataType): DbType;
