import { XCondition, XExpression, RowObjectFrom, XRowset, InputObject, Sort, SortObject, Condition, RowObject, Executor } from '../core';
import { FetchRelations } from './data-types';
import { Entity, EntityConstructor, EntityInstance, ToEntity } from './entity';
import { DbContext } from './db-context';
export declare class Queryable<T extends RowObject> implements AsyncIterable<T> {
    protected context: DbContext;
    constructor(context: DbContext, Ctr?: EntityConstructor<ToEntity<T>>, rowset?: XRowset<T>);
    private get mgr();
    private _rowset;
    private _metadata?;
    private _includes?;
    private _withDetail;
    private _sql?;
    private get sql();
    private _tableMetadata?;
    protected get executor(): Executor;
    /**
     * 异步迭代器，延迟查询的关键点
     */
    [Symbol.asyncIterator](): AsyncIterator<T>;
    skip(count: number): this;
    take(count: number): this;
    fork(rowset: XRowset<T>): Queryable<T>;
    /**
     * 过滤数据并返回一个新的Queryable
     */
    filter(condition: (p: XRowset<T>) => XCondition<T>): this;
    count(): Queryable<{
        count: number;
    }>;
    sort(sorts: (p: XRowset<T>) => Sort[] | SortObject<T>): this;
    /**
     * 添加过滤条件，并限定返回头一条记录
     */
    find(filter: (p: XRowset<T>) => XCondition<T>): this;
    union(...queries: Queryable<T>[]): this;
    /**
     * 返回一个新的类型
     */
    map<G extends InputObject>(results: (p: XRowset<T>) => G): Queryable<RowObjectFrom<G>>;
    groupBy<G extends InputObject>(results: (p: XRowset<T>) => G, groups: (p: XRowset<T>) => XExpression[], having?: (p: XRowset<T>) => Condition): Queryable<RowObjectFrom<G>>;
    join<J extends Entity, G extends InputObject>(entity: EntityConstructor<J>, on: (left: XRowset<T>, right: XRowset<J>) => Condition, results: (left: XRowset<T>, right: XRowset<J>) => G): Queryable<RowObjectFrom<G>>;
    /**
     * 查询关联导航属性
     */
    include(props: FetchRelations<T>): this;
    /**
     * 同时返回明细属性
     * 即被标记为明细属性的导航属性
     */
    withDetail(): this;
    private getSqlForExecute;
    private loadRelations;
    /**
     * 执行查询并将结果转换为数组，并获取所有数据返回一个数组
     */
    fetchAll(): Promise<EntityInstance<T>[]>;
    /**
     * 执行查询，并获取第一行记录
     */
    fetchFirst(): Promise<EntityInstance<T>>;
}
