"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeFetchRelations = exports.isClass = exports.getCaseStyle = exports.CaseStyle = exports.camelCase = exports.upperFirst = exports.complex = exports.lowerFirst = exports.isScalarType = exports.selectProperty = void 0;
const core_1 = require("../core");
const PropertySelector = new Proxy({}, {
    get: (_, key) => {
        if (typeof key !== 'string') {
            throw new Error(`Invalid property type ${typeof key}, entity property is allow string key only.`);
        }
        return key;
    },
});
function selectProperty(selector) {
    const property = selector(PropertySelector);
    return property;
}
exports.selectProperty = selectProperty;
function isScalarType(value) {
    return (value === String ||
        value === Date ||
        value === Boolean ||
        value === Number ||
        value === core_1.Decimal ||
        value === BigInt ||
        value === ArrayBuffer ||
        value === Buffer ||
        value === SharedArrayBuffer ||
        value === Object ||
        value === Array ||
        value === core_1.Uuid);
}
exports.isScalarType = isScalarType;
function lowerFirst(str) {
    return str[0].toLowerCase() + str.substring(1);
}
exports.lowerFirst = lowerFirst;
/**
 * 转换为复数
 * @param word 单词
 * @returns
 */
function complex(word) {
    if (irregular[word.toLowerCase()]) {
        const style = getCaseStyle(word);
        const complexWord = irregular[word];
        switch (style) {
            case CaseStyle.lowerCase:
            case CaseStyle.lowerCamelCase:
                return complexWord;
            case CaseStyle.upperCase:
                return complexWord.toUpperCase();
            case CaseStyle.upperCamelCase:
                return upperFirst(complexWord);
        }
    }
    if (esSuffix.find(item => word.endsWith(item))) {
        return word + 'es';
    }
    if (word.endsWith('y') && !vowels.includes(word[word.length - 2])) {
        return word.substr(0, word.length - 1) + 'ies';
    }
    const ves = vesSuffix.find(item => word.endsWith(item));
    if (ves) {
        return word.substr(0, word.length - ves.length) + 'ves';
    }
    return word + 's';
}
exports.complex = complex;
function upperFirst(str) {
    return str[0].toUpperCase() + str.substring(1);
}
exports.upperFirst = upperFirst;
function camelCase(str) {
    const nodes = str.split(/-|_| /g);
    return nodes.map(node => lowerFirst(node)).join('');
}
exports.camelCase = camelCase;
/**
 * 书写风格
 */
var CaseStyle;
(function (CaseStyle) {
    CaseStyle[CaseStyle["upperCase"] = 1] = "upperCase";
    CaseStyle[CaseStyle["lowerCase"] = 2] = "lowerCase";
    CaseStyle[CaseStyle["upperCamelCase"] = 3] = "upperCamelCase";
    CaseStyle[CaseStyle["lowerCamelCase"] = 4] = "lowerCamelCase";
})(CaseStyle = exports.CaseStyle || (exports.CaseStyle = {}));
function getCaseStyle(str) {
    throw new Error(`未实现`);
}
exports.getCaseStyle = getCaseStyle;
const esSuffix = ['s', 'o', 'x', 'th'];
const vesSuffix = ['f', 'fe'];
const irregular = {
    mouse: 'mice',
    man: 'men',
    tooth: 'teeth',
};
const vowels = ['a', 'e', 'i', 'o', 'u'];
/**
 * 判断一个函数是否为类声明
 * 注意，此方法在编译目标为ES5及以下版本中无效！
 */
// HACK： 此方法为hack方法，存在不确定性，并且已知在编译目标为ES5及以下版本中无效！
function isClass(func) {
    return func.toString().startsWith('class ');
}
exports.isClass = isClass;
function mergeFetchRelations(...includes) {
    const merge = (dest, include) => {
        if (!dest)
            return include;
        if (!include)
            return dest;
        Object.entries(include).forEach(([key, value]) => {
            if (!value)
                return;
            const exists = dest[key];
            if (!exists) {
                if (typeof value === 'boolean') {
                    dest[key] = value;
                }
                else {
                    dest[key] = merge({}, value);
                }
                return;
            }
            if (value === true) {
                // true不覆盖任何值
                return;
            }
            if (exists === true) {
                dest[key] = merge({}, value);
            }
            else {
                merge(exists, value);
            }
        });
        return dest;
    };
    let result = includes[0];
    for (let i = 1; i < includes.length; i++) {
        const include = includes[i];
        result = merge(result, include);
    }
    return result;
}
exports.mergeFetchRelations = mergeFetchRelations;
